/* $Id: asca_imsg.h,v 1.2 2012/10/23 05:16:31 waka Exp $
 *
 * Any code and mechanism in this module may not be used in any from
 * without permissions. COPYRIGHT (C) 2009 SONY Corporation,
 * all right RESERVED.
 *
 * file:        $RCSfile: asca_imsg.h,v $
 * author:      sra_okada
 * edited by:   $Author: waka $
 * created:     2009/05/26
 * updated:     $Date: 2012/10/23 05:16:31 $
 * version:     $Revision: 1.2 $
 *
 * descriptions:
 *   IO&IT Internal Message Common Header File
 */

// CAUTION!!
// The Character Code of this file must be EUC (not Shift-JIS).
// The Line Feed Code of this file must be LF  (not CR+LF).

#include <stdint.h>

#ifndef __ASCACMN_IMSG_H
#define __ASCACMN_IMSG_H

#include "asca_imsg_def.h"

//******************************************************************************
// Referred from Internal Messages
//******************************************************************************

typedef uint8_t		AsDriveID;
typedef uint32_t	AsMaterialType;
typedef uint32_t	AsClipID;
typedef uint32_t	AsClipHandle;
typedef uint32_t	AsSequenceID;

typedef enum _AsFps {						/* FpsType */
	AS_FPS_UNKNOWN = 0,
	AS_FPS_NONE,
	AS_FPS_50_I,							/* 50i */
	AS_FPS_59_94_I,							/* 59.94i */
	AS_FPS_23_98_P,							/* 23.98p */
	AS_FPS_50_P,							/* 50p */
	AS_FPS_59_94_P,							/* 59.94p */
	AS_FPS_25_P,							/* 25p */
	AS_FPS_29_97_P,							/* 29.97p */
	AS_FPS_24_P,							/* 24p */
	AS_FPS_VAR_REC							/* for Variable Rec Mode */
} AsFps;

typedef enum _AsAspectRatio {		 		/* aspectRatio */
	AS_AR_UNKNOWN = 0,
	AS_AR_NONE,
	AS_AR_4_3,								/* 4:3 */
	AS_AR_16_9,								/* 16:9 */
	AS_AR_MIXED								/* mixed */
} AsAspectRatio;

typedef enum _AsVideoCodec {				/* VidioCodec */
	AS_VC_UNKNOWN = 0,
	AS_VC_NONE,
	AS_VC_DV25_411,							/* DV25_411 */
	AS_VC_DV25D_411,						/* DV25DATA_411 */
	AS_VC_DV25_420,							/* DV25_420 */
	AS_VC_DV25D_420,						/* DV25DATA_420 */
	AS_VC_IMX30,							/* IMX30 */
	AS_VC_IMX40,							/* IMX40 */
	AS_VC_IMX50,							/* IMX50 */
	AS_VC_MPEG2HD25CBR_1440_1080_MPH14L,	/* MPEG2HD25CBR_1440_1080_MP@H-14 */
	AS_VC_MPEG2HD12_5_CBR_1440_540_MPH14L,	/* MPEG2HD12.5CBR_1440_540_MP@H-14 */
	AS_VC_MPEG2HD17_5_1440_1080_MPHL,		/* MPEG2HD17.5_1440_1080_MP@HL */
	AS_VC_MPEG2HD25_1440_1080_MPHL,			/* MPEG2HD25_1440_1080_MP@HL */
	AS_VC_MPEG2HD35_1440_1080_MPHL,			/* MPEG2HD35_1440_1080_MP@HL */
	AS_VC_MPEG2HD8_75_1440_540_MPHL,		/* MPEG2HD8.75_1440_540_MP@HL */
	AS_VC_MPEG2HD12_5_1440_540_MPHL,		/* MPEG2HD12.5_1440_540_MP@HL */
	AS_VC_MPEG2HD17_5_1440_540_MPHL,		/* MPEG2HD17.5_1440_540_MP@HL */
	AS_VC_MPEG2HD35_1920_1080_422PHL,		/* MPEG2HD35_1920_1080_422P@HL */
	AS_VC_MPEG2HD50_1920_1080_422PHL,		/* MPEG2HD50_1920_1080_422P@HL */
	AS_VC_MPEG2HD25_1920_540_422PHL,		/* MPEG2HD25_1920_540_422P@HL */
	AS_VC_MPEG2HD50_CBR_1920_1080_422PHL,	/* MPEG2HD50CBR_1920_1080_422P@HL */
	AS_VC_MPEG2HD25_CBR_1920_540_422PHL,	/* MPEG2HD25CBR_1920_540_422P@HL */
	AS_VC_MPEG2HD50_1280_720_422PHL,		/* MPEG2HD50_1280_720_422P@HL */
	AS_VC_MPEG2HD50_CBR_1280_720_422PHL,	/* MPEG2HD50CBR_1280_720_422P@HL */
	AS_VC_MPEG2HD35_1280_720_MPHL,			/* MPEG2HD35_1280_720_MP@HL */
	AS_VC_MPEG2HD35_1920_1080_MPHL,			/* MPEG2HD35_1920_1080_MP@HL(for SmallCore) */
	AS_VC_MPEG2HD25_1280_720_MPHL,			/* MPEG2HD25_1280_720_MP@HL */
	AS_VC_MPEG2HD25CBR_1280_720_MPHL,		/* MPEG2HD25CBR_1280_720_MP@HL */
	AS_VC_AVC50CBG_1440_1080_H10IPL4,		/* AVC50CBG_1440_1080_H10IP@L4 */
	AS_VC_AVC100CBG_1920_1080_H422IPL41,	/* AVC100CBG_1920_1080_H422IP@L41 */
	AS_VC_MPEG2HD,							/* MPEG2HD */
} AsVideoCodec;

typedef enum _AsVideoPort { 				/* VideoPort */
	AS_VP_UNKNOWN = 0,
	AS_VP_NONE,
	AS_VP_ANALOG_COMPOSITE,					/* ANALOG_COMPOSITE */
	AS_VP_ANALOG_COMPONENT,					/* ANALOG_COMPONENT */
	AS_VP_DIGITAL_COMPOSITE,				/* DIGITAL_COMPOSITE */
	AS_VP_DIGITAL_COMPONENT,				/* DIGITAL_COMPONENT */
	AS_VP_SDTI_CP,							/* SDTI-CP */
	AS_VP_ILINK,							/* i.LINK */
	AS_VP_SDTI,								/* SDTI */
	AS_VP_DIRECT,							/* direct */
	AS_VP_MXF,								/* MXF */
	AS_VP_OTHERS							/* other port */
} AsVideoPort;

typedef enum _AsAudioCodec {				/* AudioCodec */
	AS_AC_UNKNOWN = 0,
	AS_AC_NONE,
	AS_AC_LPCM16,							/* LinearPCM16bit */
	AS_AC_LPCM24,							/* LinearPCM24bit */
	AS_AC_NONLPCM16,						/* Non LinearPCM16bit */
	AS_AC_NONLPCM24,						/* Non LinearPCM24bit */
} AsAudioCodec;

typedef enum _AsAudioPort {					/* AudioPort */
	AS_AP_UNKNOWN = 0,
	AS_AP_NONE,
	AS_AP_ANALOG,							/* ANALOG */
	AS_AP_AES_EBU,							/* AES-EBU */
	AS_AP_SDI,								/* SDI */
	AS_AP_SDTI_CP,							/* SDTI-CP */
	AS_AP_ILINK,							/* i.LINK */
	AS_AP_SDTI,								/* SDTI */
	AS_AP_MXF,								/* MXF */
	AS_AP_OTHERS,							/* others */
	AS_AP_DIRECT,							/* DIRECT added in ver.1.20 */
	AS_AP_DIGITAL_WRR						/* DIGITAL-WRR added in ver.1.30 */
} AsAudioPort;

typedef enum _AsNrtPdKind					/* VideoLayout/PullDownSetting/PullDownKind type */
{
	AS_NRT_PDSET_UNKNOWN = 0,
	AS_NRT_PDSET_1_1,						/* 1-1 */
	AS_NRT_PDSET_2_2,						/* 2-2 */
	AS_NRT_PDSET_2_3,						/* 2-3 */
	AS_NRT_PDSET_2_3_3_2,					/* 2-3-3-2 */
	AS_NRT_PDSET_24_25,						/* 24-25 */
	AS_NRT_PDSET_OTHER,						/* other */
} AsNrtPdKind;

typedef enum _AsRMetaKind {					/* RealTypeMeta Kind */
	AS_RMK_UNKNOWN = 0,
	AS_RMK_NONE,
	AS_RMK_BASIC,
	AS_RMK_S436M_ANC512,
	AS_RMK_S436M_ANC1024,
	AS_RMK_S436M_ANC1536,
	AS_RMK_S436M_ANC2048,
	AS_RMK_S436M_ANC2560,
	AS_RMK_S436M_ANC3072,
	AS_RMK_S436M_ANC3584,
	AS_RMK_S436M_ANC4096,
	AS_RMK_S436M_ANC4608,
	AS_RMK_S436M_ANC5120,
	AS_RMK_S436M_ANC5632,
	AS_RMK_CAMERA_META
} AsRMetaKind;

typedef enum _AsUmidEtcEnum {
	AS_UMID_BASIC_LENGTH		= 32,
	AS_UMID_EXTEND_LENGTH		= 64,
	AS_UMID_E_WHEN_LENGH		= 8,
	AS_UMID_E_WHERE_LENGTH		= 12,
	AS_UMID_E_WHO_LENGH			= 12,
	AS_UMID_LENGTH_BASIC		= 0x13,
	AS_UMID_LENGTH_EXTEND		= 0x33,
	AS_UMID_INS00_PROXY			= 0xff,		/* Instance Num. which is to be set when generating proxy Umid. Leading byte must be 0xff */
	AS_UMID_BYTE_MATETYPE		= 10,		/* Byte offset of MaterialType in Label:Byte11 corresponds to MaterialType */
	AS_UMID_MTYPE_MORE_PIC		= 0x06,		/* MaterialType:two or more picture components in single container */
	AS_UMID_MTYPE_SINGLE_AUD	= 0x08,		/* MaterialType:single audio component 08h */
	AS_UMID_MTYPE_MORE_AUD		= 0x09,		/* MaterialType:two or more audio components in single container e.g. AES3 */
	AS_UMID_MTYPE_MIX			= 0x0d,		/* MaterialType:mixed group of components in a single container */
} AsUmidEtcEnum;

typedef enum _AsDiskFullNotifyType {
	AS_DISKFULL_NOTIFY_CLIPMAX	= 0,
	AS_DISKFULL_NOTIFY_EDLMAX,
	AS_DISKFULL_NOTIFY_NRTCAPACITYZERO,
	AS_DISKFULL_NOTIFY_OTHERS
} AsDiskFullNotifyType;

typedef enum _AsOpenMtrlListType {
	AS_OPEN_MTRL_LIST_TYPE_FP1 = 1,
	AS_OPEN_MTRL_LIST_TYPE_FP2 = 2,
	AS_OPEN_MTRL_LIST_TYPE_FTP,
	AS_OPEN_MTRL_LIST_TYPE_VDCP
} AsOpenMtrlListType;

typedef enum _AsMtrlChgNotifyType {
	AS_MTRL_CHG_NOTIFY_TYPE_REALLOC = 0,
	AS_MTRL_CHG_NOTIFY_TYPE_ADD,
	AS_MTRL_CHG_NOTIFY_TYPE_DELETE,
	AS_MTRL_CHG_NOTIFY_TYPE_RENAME,
	AS_MTRL_CHG_NOTIFY_TYPE_UPDATE,
	AS_MTRL_CHG_NOTIFY_TYPE_RECCOMP,
	AS_MTRL_CHG_NOTIFY_TYPE_CHANGEUMID,
	AS_MTRL_CHG_NOTIFY_TYPE_FORMAT,
	AS_MTRL_CHG_NOTIFY_TYPE_RECPROGRESS,
	AS_MTRL_CHG_NOTIFY_TYPE_DELETEMULTIPLEMATERIALS,
	AS_MTRL_CHG_NOTIFY_TYPE_EDITSTART,
	AS_MTRL_CHG_NOTIFY_TYPE_EDITSTOP
} AsMtrlChgNotifyType;

typedef enum _AsNrtFileType {
	AS_NRT_FILETYPE_DISCMETA = 1,
	AS_NRT_FILETYPE_MEDIAPRO,
	AS_NRT_FILETYPE_SYSTEMPRO,
	AS_NRT_FILETYPE_ALIAS,
	AS_NRT_FILETYPE_NRTMETA,
	AS_NRT_FILETYPE_GENERALMETA,
	AS_NRT_FILETYPE_EDITLIST,
	AS_NRT_FILETYPE_INDEX
} AsNrtFileType;

typedef enum _AsRtFormatType {
    AS_RTFORMAT_TYPE_MP4 = 0,
    AS_RTFORMAT_TYPE_SXS_MXF,
    AS_RTFORMAT_TYPE_PD_MXF,
    AS_RTFORMAT_TYPE_DV_AVI_NTSC,
    AS_RTFORMAT_TYPE_DV_AVI_PAL
} AsRtFormatType;

typedef enum _AsHFGenMode {
	AS_HF_NORMAL_ALL			= 0,
	AS_HF_STREAMCHASE_ALL		= 1,
	AS_HF_STREAMCHASE_HDR		= 2,
	AS_HF_STREAMCHASE_FTR		= 3,
	AS_HF_STREAMCHASE_PROXY_ALL	= 10,
	AS_HF_STREAMCHASE_PROXY_HDR	= 11,
	AS_HF_STREAMCHASE_PROXY_FTR	= 12,
	AS_HF_STREAMCHASE_COPY_HDR	= 20,
	AS_HF_STREAMCHASE_COPY_FTR	= 21,
	AS_HF_MEMORYSTYLE_ALL		= 30
} AsHFGenMode;

typedef enum _AsMAReturnCode {
	/*------------------------------------------------------------------------*/
	/* Common	: 0x00000000 - 0x000000FF									  */
	/*------------------------------------------------------------------------*/
	AS_MA_OK_CODE = 0x00000000,			/*Succeeded							  */
	AS_MA_NG_CODE,						/*Failed							  */
	AS_MA_ERR_CMD_NOT_SUPPORTED,		/*Commandnotsupported				  */
	AS_MA_ERR_BAD_PARAMETER,			/*Illegalparameter					  */
	AS_MA_ERR_OUT_OF_MEMORY,			/*Memoryisinsufficient				  */
										/*(Failedallocation)				  */
	AS_MA_ERR_SHMAT_FAIL,				/*Failedinattachingoftheshared		  */
										/*memory(Or,itisnotattached)		  */
	AS_MA_ERR_SHMDT_FAIL,				/*Failedindettachingoftheshared		  */
										/*memory							  */
	AS_MA_ERR_INVALID_LIST_HANDLE,		/*InvalidListHandle					  */
	AS_MA_ERR_RETRY_GETMATERIALLIST,	/*Retrybecausenoadjustmentwas		  */
										/*generatedintheMaterialList		  */
	AS_MA_ERR_BAD_MEHF_SESSIONID,		/*Invalid Mehf SessionId			  */
	AS_MA_ERR_OTHER_REASON,				/*Othererrors						  */

	/*------------------------------------------------------------------------*/
	/* Clip		: 0x00000100 - 0x000001FF									  */
	/*------------------------------------------------------------------------*/
	AS_MA_ERR_CLIP_NOT_FOUND=0x00000100,	/*Clipdoesn'texist				  */
	AS_MA_ERR_CLIP_EXIST,				/*Clipalreadyexists					  */
	AS_MA_ERR_CLIP_IS_OPENED,			/*Cliphasalreadyopened				  */
	AS_MA_ERR_CLIP_IS_SUBCLIP_OF_OPERATING_EDL,	/*ClipisincludedinEDL		  */
												/*whileoperatingit			  */
	AS_MA_ERR_INVALID_CLIP_HANDLE,		/*ClipHandleisinvalid				  */
	AS_MA_ERR_TOO_MANY_OPEN_CLIPS,		/*ToomanyopenClips					  */
	AS_MA_ERR_MAX_CLIP,					/*ToomanyClips						  */
										/*(SSD:3000,PD:300,SxS:600)			  */
	AS_MA_ERR_CLIP_WRITE_PROTECT,		/*Clipisread-only					  */
	AS_MA_ERR_CLIP_READ,				/*FailedinClipRead					  */
	AS_MA_ERR_CLIP_WRITE,				/*FailedinClipWrite					  */
	AS_MA_ERR_CLIP_SEEK,				/*FailedinClipSeek					  */
	AS_MA_ERR_EDL_NOT_FOUND,			/*EDLdoesn'texist					  */
	AS_MA_ERR_EDL_EXIST,				/*EDLalreadyexists					  */
	AS_MA_ERR_EDL_IS_OPERATING,			/*EDLisOperating					  */
	AS_MA_ERR_DUR_LIMIT,				/*Too Long EDL Duration			  */
	AS_MA_ERR_MAX_EDL,					/*ToomanyEDL(SSD:1000,PD:900)		  */
	AS_MA_ERR_ILLEGAL_EDL,				/*IllegalEDL						  */
	AS_MA_ERR_MATERIAL_NOT_FOUND,		/*Materialdoesn'texist				  */
	AS_MA_ERR_MATERIAL_EXIST,			/*Materialalreadyexists				  */
	AS_MA_ERR_TOO_MANY_OPEN_MATERIALLISTS,	/*ToomanyopenMaterialLists		  */
	AS_MA_ERR_HEADER_PP_PARSE_FAILED,	/*Parse Header PP Error				  */
	AS_MA_ERR_HEADER_META_PARSE_FAILED,	/*Parse Header Meta Error			  */
	AS_MA_ERR_FOOTER_PP_PARSE_FAILED,	/*Parse Footer PP Error				  */
	AS_MA_ERR_FOOTER_META_PARSE_FAILED, /*Parse Footer Meta Error			  */
	AS_MA_ERR_FINALIZE_META_FAILED,		/*Finalize Meta Error				  */
	AS_MA_ERR_CLIP_READ_MEDIA_REMOVAL,	/*Media Removal while Reading Clip	  */
	AS_MA_ERR_REC_INHI_FORMAT,			/* RecInhi Format					  */
	AS_MA_ERR_EM_EXIST,					/*Emalreadexists					  */
	AS_MA_ERR_ADD_EM_FAIL,				/*FailedinAddEm						  */
	AS_MA_ERR_RETRY_GETMATERIALEMLIST,	/*Retry GetMaterialEmList			  */
	AS_MA_ERR_INVALID_EDL_HANDLE,		/*EdlHandleisinvalid				  */

	/*------------------------------------------------------------------------*/
	/* File		: 0x00000200 - 0x000002FF									  */
	/*------------------------------------------------------------------------*/
	AS_MA_ERR_FILE_NOT_FOUND=0x00000200,	/*Filedoesn'texist				  */
	AS_MA_ERR_FILE_EXIST,				/*Filealreadyexists					  */
	AS_MA_ERR_FILE_IS_OPENED,			/*Filehasalreadyopened				  */
	AS_MA_ERR_INVALID_FILE_HANDLE,		/*InvalidFileHandle					  */
	AS_MA_ERR_TOO_MANY_OPEN_FILES,		/*ToomanyopenFiles					  */
	AS_MA_ERR_MAX_FILE,					/*ToomanyFiles						  */
	AS_MA_ERR_FILE_WRITE_PROTECT,		/*Fileisread-only					  */
	AS_MA_ERR_FILE_READ,				/*FailedinFileRead					  */
	AS_MA_ERR_FILE_WRITE,				/*FailedinFileWrite					  */
	AS_MA_ERR_FILE_SEEK,				/*FailedinFileSeek					  */
	AS_MA_ERR_PARSE,					/*FailedinParse						  */
	AS_MA_ERR_SERIALIZE,				/*FailedinSerialize					  */

	/*------------------------------------------------------------------------*/
	/* Media	: 0x00000300 - 0x000003FF									  */
	/*------------------------------------------------------------------------*/
	AS_MA_ERR_INVALID_DRIVE=0x00000300,	/*Mediadoesn'texist					  */
	AS_MA_ERR_NOT_MOUNTED,				/*Mediaisnotmounted					  */
	AS_MA_ERR_PROTECTED,				/*Mediaisread-only					  */
	AS_MA_ERR_OUT_OF_STORAGE_SPACE,     /* Out of storage space               */
	AS_MA_ERR_NEAR_FULL,                /* No disk space to reserve           */
	AS_MA_ERR_DISK_FULL,                /* No disk space to record            */
	AS_MA_ERR_NO_SUPPORT_MEDIA,			/*No support Media					  */
	AS_MA_ERR_MEDIA_BUSY,			    /* Media is Formatting				  */
	AS_MA_ERR_REC_INHI,					/* Media is RecInhi					  */

	/*------------------------------------------------------------------------*/
	/* LocalConnect: 0x00000400 - 0x000004FF									  */
	/*------------------------------------------------------------------------*/
	AS_MA_ERR_DISCONNECTED=0x00000400	/*Disconnected						  */
} AsMAReturnCode;

typedef enum _AsNetApiReturnCode {
	AS_NETAPI_NO_ERROR          = 0x00000000,
	AS_NETAPI_ERR_BAD_PARAMETER = 0x00000003,  /*Illegalparameter */
	AS_NETAPI_ERR_OTHER_REASON  = 0x0000000A,  /*Othererrors */

	/*------------------------------------------------------------------------*/
	/* NetAPI: 0x05000000-                                                    */
	/*------------------------------------------------------------------------*/
	AS_NETAPI_ERR_UNKNOWN_CMD      = 0x05000000,
	AS_NETAPI_ERR_ILLEGAL_CMD      = 0x05000001,
	AS_NETAPI_ERR_NOT_AVAILABLE    = 0x05000002,
	AS_NETAPI_ERR_DATA_BUFFER_BUSY = 0x05000003,
	AS_NETAPI_ERR_BUFFER_OVERFLOW  = 0x05000004,
	AS_NETAPI_ERR_QUEUE_FULL       = 0x05000005
} AsNetApiReturnCode;

typedef enum _AsConvReturnCode {
	CONVD_OK_CODE				= AS_MA_OK_CODE,	/*Succeeded				  */
	CONVD_NG_CODE				= 0x01000000,		/* Failed				  */
	/*------------------------------------------------------------------------*/
	/* ConverterCommonError : 												  */
	/*------------------------------------------------------------------------*/
	CONVD_ERR_INTERNAL_ERR       = 0x01000001,	/* Converter internal error	  */
	CONVD_ERR_INVALID_COMMAND    = 0x01000002,	/* invalid command			  */
	/*------------------------------------------------------------------------*/
	/* ftp / cifs			 : 												  */
	/*------------------------------------------------------------------------*/
	CONVD_ERR_FS_SESSION_OVER    = 0x01000100,	/* ConFs-ConD session count over  */
	CONVD_ERR_FS_GET_RES_FAIL    = 0x01000101,	/* resource alloc failed		  */
	CONVD_ERR_FS_NO_SUPORT_FORMAT= 0x01000102,	/* not supported format			  */
	CONVD_ERR_READ_FAIL          = 0x01000103,	/* convert error in read process  */
	CONVD_ERR_WRITE_FAIL         = 0x01000104,	/* convert error in write process */
	CONVD_ERR_FS_CLOSE_FAIL      = 0x01000105,	/* convert error in close process */
	CONVD_ERR_SAME_UMID_EXIST    = 0x01000106,	/* ftp: unchanged same umid is fail */
    CONVD_ERR_CLIP_SYNTAX_ERR    = 0x01000107,	/* ftp/copy: Syntax chekc err	  */
	CONVD_ERR_FS_SYNC_FAIL		 = 0x01000108,	/* ftp: duration sync error  	  */
	CONVD_ERR_NO_SPACE_FOR_SWAP	 = 0x01000109,	/* no swap area of data			  */
	CONVD_ERR_NO_SUPORT_FORMAT_AUDIO_EMPASHIS = 0x0100010A,	/* Emphasis on error  */
	/*------------------------------------------------------------------------*/
	/* copy					: 												  */
	/*------------------------------------------------------------------------*/
	CONVD_ERR_COPY_SESSION_OVER     = 0x01000200,/* copy session count over   */
	CONVD_ERR_COPY_GET_RES_FAIL     = 0x01000201,/* resource alloc failed     */
	CONVD_ERR_COPY_NO_SUPORT_FORMAT = 0x01000202,/* not supported format	  */
	CONVD_ERR_COPY_ABORTED          = 0x01000203,/* copy process is aborted   */
	CONVD_ERR_COPY_CAN_NOT_LOCK     = 0x01000204,/* can not lock copy session */
	CONVD_ERR_COPY_CAN_NOT_FIND_KEY = 0x01000205,/* can not find lock key     */
	CONVD_WAR_SAME_UMID_EXIST       = 0x01000206,/* copy: same umid is already copyed  */
    CONVD_ERR_COPY_ABORT_FAIL       = 0x01000207,/* copy: abort fail          */
    CONVD_ERR_CAN_NOT_FIND_KEY      = 0x01000208,/* copy: find key fail       */
    CONVD_ERR_COPY_NOT_FOUND_SESSION= 0x01000209,/* copy: not found session   */
	CONVD_ERR_COPY_ABORTED_EJECT    = 0x0100020A,/* copy is aborted by Eject  */
	/*------------------------------------------------------------------------*/
	/* session 				: 												  */
	/*------------------------------------------------------------------------*/
	CONVD_ERR_SR_CONECTION_FAIL = 0x01000300,/* abnormal condition			  */
	CONVD_ERR_SR_REMOVE_FAIL    = 0x01000301,/* session router abnormal condition */
	CONVD_ERR_SR_SEND_PM_FAIL   = 0x01000302,/* failed in send external message	  */
	CONVD_ERR_SR_RECV_PM_FAIL   = 0x01000303,/* failed in recieve external message*/
	/*------------------------------------------------------------------------*/
	/* boot					: 												  */
	/*------------------------------------------------------------------------*/
	CONVD_ERR_SIGNAL_FAIL = 0x01000400,	/* error in termination				  */
	/*------------------------------------------------------------------------*/
	/* Thumbnail			:	 											  */
	/*------------------------------------------------------------------------*/
	CONVD_ERR_THUMBNAIL_SESSION_OVER      = 0x01000500,	/* thumbnail session over    */
	CONVD_ERR_THUMBNAIL_NOT_FOUND_SESSION = 0x01000501,	/* specified invalid handler */
	CONVD_ERR_THUMBNAIL_END_OF_DATA       = 0x01000502,	/* specified invalid frame	 */
	CONVD_ERR_THUMBNAIL_ABORT_FAIL        = 0x01000503,	/* abort failed (maybe success getting)	  */
	CONVD_ERR_THUMBNAIL_CANCELED          = 0x01000504,	/* GetThumbnail canceled     */
	CONVD_ERR_THUMBNAIL_GENERATE_FAIL     = 0x01000505,	/* thumbnail generate failed */
	/*------------------------------------------------------------------------*/
	/* Meta ErrInfo			:	 											  */
	/*------------------------------------------------------------------------*/
	/* FinalizeMeta			:												  */
	CONVD_WAR_FMETA_MPUMID_ERR	= 0x01000600,	/* MEHF_AS_WARN_MPUMID_ERR	  */
	CONVD_WAR_FMETA_UMID_F					,	/* MEHF_AS_WARN_UMID_F		  */
	CONVD_WAR_FMETA_FPS_UNCHANGE			,	/* MEHF_AS_WARN_FPS_UNCHANGE  */
	CONVD_WAR_FMETA_VCODEC_UNCHANGE			,	/* MEHF_AS_WARN_VCODEC_UNCHANGE */
	CONVD_WAR_FMETA_ASPECTRATIO_UNCHANGE	,	/* MEHF_AS_WARN_ASPECTRATIO_UNCHANGE */
	CONVD_WAR_FMETA_AQBITS_UNCHANGE			,	/* MEHF_AS_WARN_AQBITS_UNCHANGE */
	CONVD_WAR_FMETA_ORIGIN_UNCHANGE			,	/* MEHF_AS_WARN_ORIGIN_UNCHANGE */
	CONVD_WAR_FMETA_ABORT_UNCHANGE			,	/* MEHF_AS_WARN_ABORT_UNCHANGE */
	CONVD_WAR_FMETA_DUR_UNCHANGE			,	/* MEHF_AS_WARN_DUR_UNCHANGE  */
	CONVD_WAR_FMETA_TSS_DUR					,	/* MEHF_HI_WARN_TSS_DUR		  */
	CONVD_WAR_FMETA_ORIGIN					,	/* MEHF_AS_WARN_ORIGIN		  */
	CONVD_WAR_FMETA_WARN_FPS				,	/* MEHF_AS_WARN_FPS			  */
	CONVD_WAR_FMETA_VCODEC					,	/* MEHF_AS_WARN_VCODEC		  */
	CONVD_WAR_FMETA_ASPECTRATIO				,	/* MEHF_AS_WARN_ASPECTRATIO	  */
	CONVD_WAR_FMETA_AQBITS					,	/* MEHF_AS_WARN_AQBITS		  */
	CONVD_WAR_FMETA_DUR						,	/* MEHF_AS_WARN_DUR			  */
	CONVD_WAR_FMETA_ABORT					,	/* MEHF_AS_WARN_ABORT		  */
	CONVD_WAR_FMETA_UMID_DUP				,	/* MEHF_AS_WARN_UMID_DUP	  */
	/* ParseHeaderPP/ParseFooterPP	:										  */
	CONVD_ERR_PHFPP_KLV_STRUCT	= 0x01000620,	/* MEHF_HI_ERR_KLV_STRUCT	  */
	CONVD_ERR_PHFPP_HPS_UL					,	/* MEHF_HI_ERR_HPS_UL		  */
	CONVD_ERR_PHFPP_HPS_VER					,	/* MEHF_HI_ERR_HPS_VER		  */
	CONVD_ERR_PHFPP_HPS_KAG					,	/* MEHF_HI_ERR_HPS_KAG		  */
	CONVD_ERR_PHFPP_HPS_HDRSIZE				,	/* MEHF_HI_ERR_HPS_HDRSIZE	  */
	CONVD_ERR_PHFPP_HPS_IDXSIZE				,	/* MEHF_HI_ERR_HPS_IDXSIZE	  */
	CONVD_ERR_PHFPP_HPS_OP					,	/* MEHF_HI_ERR_HPS_OP		  */
	CONVD_ERR_PHFPP_HPS_EC					,	/* MEHF_HI_ERR_HPS_EC		  */
	/* ParseHeaderMeta/ParseFooterMeta	:									  */
	CONVD_ERR_PHFMETA_KLV_STRUCT = 0x01000640,	/* MEHF_HI_ERR_KLV_STRUCT	  */
	CONVD_ERR_PHFMETA_PP_UL					,	/* MEHF_HI_ERR_PP_UL		  */
	CONVD_ERR_PHFMETA_KLV_ITEM				,	/* MEHF_HI_ERR_KLV_ITEM		  */
	CONVD_ERR_PHFMETA_PS_VER				,	/* MEHF_HI_ERR_PS_VER		  */
												/* MEHF_HI_ERR_PP_ITEM		  */
												/* MEHF_HI_ERR_MEDS_ITEM	  */
	CONVD_ERR_PHFMETA_PS_OP					,	/* MEHF_HI_ERR_PS_OP		  */
	CONVD_ERR_PHFMETA_PS_EC					,	/* MEHF_HI_ERR_PS_EC		  */
	CONVD_ERR_PHFMETA_MPUMID				,	/* MEHF_HI_ERR_MPUMID		  */
	CONVD_ERR_PHFMETA_IDX_EUSIZE			,	/* MEHF_HI_ERR_IDX_EUSIZE	  */
	CONVD_WAR_PHFMETA_KLV_NOV	= 0x01000660,	/* MEHF_HI_WARN_KLV_NOV		  */
	CONVD_WAR_PHFMETA_KLV_UNK				,	/* MEHF_HI_WARN_KLV_UNK		  */
	CONVD_WAR_PHFMETA_PS_IDSIZE				,	/* MEHF_HI_WARN_PS_IDSIZE	  */
	CONVD_WAR_PHFMETA_TSS_DUR				,	/* MEHF_HI_WARN_TSS_DUR		  */
	CONVD_WAR_PHFMETA_TCS_TCSIZE			,	/* MEHF_HI_WARN_TCS_TCSIZE	  */
	CONVD_WAR_PHFMETA_PEDS_ITEM				,	/* MEHF_HI_WARN_PEDS_ITEM	  */
	CONVD_WAR_PHFMETA_GSEDS_ITEM			,	/* MEHF_HI_WARN_GSEDS_ITEM	  */
	CONVD_WAR_PHFMETA_ASEDS_ITEM			,	/* MEHF_HI_WARN_ASEDS_ITEM	  */
	CONVD_WAR_PHFMETA_NRT_EQUIP_LMT			,	/* MEHF_AS_WARN_NRT_EQUIP_LMT */
	/* ParseHeaderMeta		:												  */
	CONVD_ERR_PHMETA_SINGLE_AES	= 0x01000680,	/* MEHF_AS_ERR_SINGLE_AES	  */
	CONVD_ERR_PHMETA_FPS_UNKNOWN			,	/* MEHF_AS_ERR_FPS_UNKNOWN	  */
	CONVD_ERR_PHMETA_VIDEO_CODEC			,	/* MEHF_AS_ERR_VCODEC		  */
	CONVD_ERR_PHMETA_FPS_VIDEO_COMB			,	/* MEHF_AS_ERR_PAIR_FPS_VCODEC */
	CONVD_ERR_PHMETA_ORIGIN					,	/* MEHF_AS_ERR_ORIGIN		  */
	CONVD_ERR_PHMETA_DURATION				,	/* MEHF_AS_ERR_DUR			  */
	CONVD_ERR_PHMETA_AUDIO_BIT				,	/* MEHF_AS_ERR_AQBITS		  */
	CONVD_ERR_PHMETA_AUDIO_CH_NUM			,	/* MEHF_AS_ERR_AUDIOCH		  */
	CONVD_ERR_PHMETA_VIDEO_AUDIO_COMB		,	/* MEHF_AS_ERR_PAIR_VIDEO_AUDIO */
	CONVD_ERR_PHMETA_IDX_KLV_S				,	/* MEHF_HI_ERR_IDX_KLV_S	  */
	CONVD_ERR_PHMETA_IDX_UL					,	/* MEHF_HI_ERR_IDX_UL		  */
	/* WriteSysDataIte		:												  */
	CONVD_WAR_WSYSDI_RTTC_SI	= 0x010006A0,	/* MEHF_AS_WARN_RTTC_SI		  */
	CONVD_WAR_WSYSDI_RTTC_DI					/* MEHF_AS_WARN_RTTC_DI		  */
} AsConvReturnCode;

typedef enum _AsIOReturnCode {
	AS_IO_NO_ERROR	= 0x00000000,
	AS_IO_NG_CODE,						/*Failed							  */
	AS_IO_ERR_CMD_NOT_SUPPORTED,		/*Commandnotsupported				  */
	AS_IO_ERR_BAD_PARAMETER,			/*Illegalparameter					  */
	AS_IO_ERR_OUT_OF_MEMORY,			/*Memoryisinsufficient				  */
										/*(Failedallocation)				  */
	AS_IO_ERR_SHMAT_FAIL,				/*Failedinattachingoftheshared		  */
										/*memory(Or,itisnotattached)		  */
	AS_IO_ERR_SHMDT_FAIL,				/*Failedindettachingoftheshared		  */
										/*memory							  */
	AS_IO_ERR_INVALID_LIST_HANDLE,		/*InvalidListHandle					  */
	AS_IO_ERR_RETRY_GETMATERIALLIST,	/*Retrybecausenoadjustmentwas		  */
										/*generatedintheMaterialList		  */
	AS_IO_ERR_BAD_MEHF_SESSIONID,		/*Invalid Mehf SessionId			  */
	AS_IO_ERR_OTHER_REASON,				/*Othererrors						  */

	/*------------------------------------------------------------------------*/
	/* Clip		: 0x00000100 - 0x000001FF									  */
	/*------------------------------------------------------------------------*/
	AS_IO_ERR_CLIP_NOT_FOUND=0x00000100,	/*Clipdoesn'texist				  */
	AS_IO_ERR_CLIP_EXIST,				/*Clipalreadyexists					  */
	AS_IO_ERR_CLIP_IS_OPENED,			/*Cliphasalreadyopened				  */
	AS_IO_ERR_CLIP_IS_SUBCLIP_OF_OPERATING_EDL,	/*ClipisincludedinEDL		  */
												/*whileoperatingit			  */
	AS_IO_ERR_INVALID_CLIP_HANDLE,		/*ClipHandleisinvalid				  */
	AS_IO_ERR_TOO_MANY_OPEN_CLIPS,		/*ToomanyopenClips					  */
	AS_IO_ERR_MAX_CLIP,					/*ToomanyClips						  */
										/*(SSD:3000,PD:300,SxS:600)			  */
	AS_IO_ERR_CLIP_WRITE_PROTECT,		/*Clipisread-only					  */
	AS_IO_ERR_CLIP_READ,				/*FailedinClipRead					  */
	AS_IO_ERR_CLIP_WRITE,				/*FailedinClipWrite					  */
	AS_IO_ERR_CLIP_SEEK,				/*FailedinClipSeek					  */
	AS_IO_ERR_EDL_NOT_FOUND,			/*EDLdoesn'texist					  */
	AS_IO_ERR_EDL_EXIST,				/*EDLalreadyexists					  */
	AS_IO_ERR_EDL_IS_OPERATING,			/*EDLisOperating					  */
	AS_IO_ERR_MAX_EDL,					/*ToomanyEDL(SSD:1000,PD:900)		  */
	AS_IO_ERR_ILLEGAL_EDL,				/*IllegalEDL						  */
	AS_IO_ERR_MATERIAL_NOT_FOUND,		/*Materialdoesn'texist				  */
	AS_IO_ERR_MATERIAL_EXIST,			/*Materialalreadyexists				  */
	AS_IO_ERR_TOO_MANY_OPEN_MATERIALLISTS,	/*ToomanyopenMaterialLists		  */

	/*------------------------------------------------------------------------*/
	/* File		: 0x00000200 - 0x000002FF									  */
	/*------------------------------------------------------------------------*/
	AS_IO_ERR_FILE_NOT_FOUND=0x00000200,	/*Filedoesn'texist				  */
	AS_IO_ERR_FILE_EXIST,				/*Filealreadyexists					  */
	AS_IO_ERR_FILE_IS_OPENED,			/*Filehasalreadyopened				  */
	AS_IO_ERR_INVALID_FILE_HANDLE,		/*InvalidFileHandle					  */
	AS_IO_ERR_TOO_MANY_OPEN_FILES,		/*ToomanyopenFiles					  */
	AS_IO_ERR_MAX_FILE,					/*ToomanyFiles						  */
	AS_IO_ERR_FILE_WRITE_PROTECT,		/*Fileisread-only					  */
	AS_IO_ERR_FILE_READ,				/*FailedinFileRead					  */
	AS_IO_ERR_FILE_WRITE,				/*FailedinFileWrite					  */
	AS_IO_ERR_FILE_SEEK,				/*FailedinFileSeek					  */
	AS_IO_ERR_PARSE,					/*FailedinParse						  */
	AS_IO_ERR_SERIALIZE,				/*FailedinSerialize					  */

	/*------------------------------------------------------------------------*/
	/* Media	: 0x00000300 - 0x000003FF									  */
	/*------------------------------------------------------------------------*/
	AS_IO_ERR_INVALID_DRIVE=0x00000300,	/*Mediadoesn'texist					  */
	AS_IO_ERR_NOT_MOUNTED,				/*Mediaisnotmounted					  */
	AS_IO_ERR_PROTECTED,				/*Mediaisread-only					  */
	AS_IO_ERR_OUT_OF_STORAGE_SPACE,		/* Out of storage space				  */
	AS_IO_ERR_NEAR_FULL,				/* No disk space to reserve			  */
	AS_IO_ERR_DISK_FULL,				/* No disk space to record			  */
	AS_IO_ERR_NO_SUPPORT_MEDIA,			/*No support Media					  */
	AS_IO_ERR_MEDIA_BUSY,			    /* Media is Formatting				  */
	AS_IO_ERR_REC_INHI,					/* Media is RecInhi					  */

	/*------------------------------------------------------------------------*/
	/* LocalConnect: 0x00000400 - 0x000004FF									  */
	/*------------------------------------------------------------------------*/
	AS_IO_ERR_DISCONNECTED=0x00000400,	/*Disconnected						  */

	/*------------------------------------------------------------------------*/
	/* IO: 0x02000000 - 													  */
	/*------------------------------------------------------------------------*/
	AS_IO_ERR_FAIL	= 0x02000000,
	AS_IO_ERR_INVALID_ID,
	AS_IO_ERR_WRONG_PORT_TYPE,
	AS_IO_ERR_PORT_NOT_IDLE,
	AS_IO_ERR_PORT_NOT_ACTIVE,
	AS_IO_ERR_CUE_NOT_DONE,
	AS_IO_ERR_NOT_READY_TO_PLAY,
	AS_IO_ERR_CANCELLED,
	AS_IO_ERR_TL_EV_NOT_ENOUGH_OFFSET = 0x02000100,
	AS_IO_ERR_TL_EV_REGISTERED
} AsIOReturnCode;

typedef enum _AsCMDReturnCode {
	AS_CMD_OK_CODE = 0x00000000,
	AS_CMD_NG_CODE,						/* Failed							  */
	AS_CMD_ERR_CMD_NOT_SUPPORTED,		/* Command not supported			  */
	AS_CMD_ERR_BAD_PARAMETER,			/* Illegal parameter				  */
	AS_CMD_ERR_NOT_AVAILABLE,
	/*------------------------------------------------------------------------*/
	/* IT-CMD: 0x06000000-                                                    */
	/*------------------------------------------------------------------------*/
	AS_CMD_ERR_MEDIA_INUSE = 0x06000000,
	AS_CMD_ERR_ALREADYLOCKED,
	AS_CMD_ERR_COPY_SESSION_OVER
} AsCMDReturnCode;

typedef enum _AsDMReturnCode {
	AS_DM_OK_CODE = 0x00000000,			/* Succeeded						  */
	AS_DM_NG_CODE,						/* Failed							  */
	AS_DM_ERR_BAD_PARAMETER,			/* Requested Parameteris Invalid	  */
	AS_DM_ERR_INVALID_PARAMETERS,		/* Invalid Parameters				  */
	AS_DM_ERR_OTHER_REASON,				/* Other Errors						  */
	AS_DM_ERR_CANNOT_ENTER_FIRMWARE_UPDATE_MODE = 0x04000000,
										/* Cannot Enter FirmWare Update Mode  */
	AS_DM_ERR_CANNOT_SEND_FIRMWARE,		/* Cannot Send FirmWare				  */
	AS_DM_ERR_INVALID_FIRMWARE_FORMAT,	/* Invalid FirmWare Format			  */
	AS_DM_ERR_FLASH_ROM_ACCESS_ERROR,	/* Flash ROM Access Error			  */
	AS_DM_ERR_CANNOT_READ_FIRMWARE_SIGNATURE,
										/* Cannot Read FirmWare Signature	  */
	AS_DM_ERR_CANNOT_WRITE_FIRMWARE,	/* Cannot Write FirmWare			  */
	AS_DM_ERR_FIRMWARE_VERIFY_ERROR,	/* FirmWare Verify Error			  */
	AS_DM_ERR_CANNOT_EXIT_FIRMWARE_UPDATE_MODE,
										/* Cannot Exit FirmWare Update Mode	  */
	AS_DM_ERR_REJECTED_BY_FREQUENT_DETECTION,
										/* Rejected By Frequent Detection	  */
	AS_DM_ERR_AC_FAIL,					/* AC Fail */
	AS_DM_ERR_AC_MEDIA_ABSENCE,			/* AC No Media */
	AS_DM_ERR_AC_INVALID_MEDIA,			/* AC Invalid Media */
	AS_DM_ERR_AC_INVALID_CATEGORY,		/* AC Invalid Category */
	AS_DM_ERR_AC_NO_AC_VALID,			/* AC No AC Valid */
	AS_DM_ERR_AC_ID_ABSENCE,			/* AC ID Absence */
	AS_DM_ERR_AC_INVALID_AUTH_METHOD,	/* AC Invalid Auth Method */
	AS_DM_ERR_AC_TOO_MANY_AUTH_FAIL,	/* AC Too Many Auth Fail */
	AS_DM_ERR_AC_INVALID_PASSWORD,		/* AC Invalid Password */
	AS_DM_ERR_AC_NO_MASTER_ACCOUNT,		/* AC No Master Account */
	AS_DM_ERR_AC_WRITE_PROTECT,			/* AC WriteProtect */
	AS_DM_ERR_AC_INVALID_STATE			/* AC Invalid State */
	/*------------------------------------------------------------------------*/
	/* IT-DM: 0x07000000-                                                     */
	/*------------------------------------------------------------------------*/
} AsDMReturnCode;

typedef enum _AsVLCueupMode {
	AS_VLCUEUP_PLAY	 = 0x00000001,
	AS_VLCUEUP_STILL = 0x00000002
} ASVLCueupMode;

typedef enum _AsMaterialDetailSubInfo {
	AS_MTRL_DETAIL_SUBINFO_NOTREADYTOPLAY		= 0x01,	
	AS_MTRL_DETAIL_SUBINFO_INREC				= 0x02
} AsMaterialDetailSubInfo;

typedef enum _AsMaterialDetailSubErrInfo {
	AS_MTRL_DETAIL_SUBERRINFO_EDITLISTREADPARSENG	= 0x01,
	AS_MTRL_DETAIL_SUBERRINFO_NRTMETAREADPARSENG	= 0x02,
	AS_MTRL_DETAIL_SUBERRINFO_ILLEGALFILEFORMAT		= 0x04,
	AS_MTRL_DETAIL_SUBERRINFO_UNPLAYABLESETTING		= 0x08,
	AS_MTRL_DETAIL_SUBERRINFO_UNPLAYABLESNAPSHOT	= 0x10,
} AsMaterialDetailSubErrInfo;

typedef enum _AsFpMaterialDetailShotMark {
	AS_FP_MTRL_DETAIL_NO_SHOTMARK	= 1,
	AS_FP_MTRL_DETAIL_SHOTMARK			,
} AsFpMaterialDetailShotMark;

typedef enum _AsLEDEmissionMode {
	 AS_LED_EMISSION_WORK_FREE		= 0x00,
	 AS_LED_EMISSION_OFF			= 0x01,
	 AS_LED_EMISSION_ON				= 0x03,
	 AS_LED_EMISSION_SHALLOW_BLINK	= 0x05,
	 AS_LED_EMISSION_DEEP_BLINK		= 0x07,
	 AS_LED_EMISSION_ON_AT_EJECTED	= 0x83
} AsLEDEmissionMode;

typedef enum _AsLEDColor {
	AS_LED_COLOR_RED				= 0x00,
	AS_LED_COLOR_GREEN				= 0x01
} AsLEDColor;

typedef enum _AsFirmWareType {
	AS_WINKLE_2ND_FW				= 0x00,		/* Winkle 2nd FirmWare	 */
	AS_WINKLE_FPGA_FW				= 0x01,		/* Winkle FPGA FirmWare	 */
	AS_PD_DRIVE_FW					= 0x02		/* PD Drive FirmWare	 */
} AsFirmWareType;

typedef enum _AsMetaEmKind {
	AS_META_EM_UNKNOWN 			= 0x00000000,	// Unknown
	AS_META_EM_REC_START		= 0x00000001,	// _RecStart
	AS_META_EM_FLASH 			= 0x00000002,	// _Flash
	AS_META_EM_OVR_A_LMT 		= 0x00000004,	// _OverAudioLimiter
	AS_META_EM_WHT_BLNCE_CHG	= 0x00000008,	// _WhiteBalanceChange
	AS_META_EM_FLT_CHG 			= 0x00000010,	// _FilterChange
	AS_META_EM_SHT_SPD_CHG 		= 0x00000020,	// _ShutterSpeedChange
	AS_META_EM_GAIN_CHG 		= 0x00000040,	// _GainChange
	AS_META_EM_SM1 				= 0x00000080,	// _ShotMark1
	AS_META_EM_SM2 				= 0x00000100,	// _ShotMark2
	AS_META_EM_SM3 				= 0x00000200,	// _ShotMark3
	AS_META_EM_SM4 				= 0x00000400,	// _ShotMark4
	AS_META_EM_SM5 				= 0x00000800,	// _ShotMark5
	AS_META_EM_SM6 				= 0x00001000,	// _ShotMark6
	AS_META_EM_SM7 				= 0x00002000,	// _ShotMark7
	AS_META_EM_SM8 				= 0x00004000,	// _ShotMark8
	AS_META_EM_SM9 				= 0x00008000,	// _ShotMark9
	AS_META_EM_SM0 				= 0x00010000,	// _ShotMark0
	AS_META_EM_CUT				= 0x00020000,	// _Cut
	AS_META_EM_OTHERS			= 0x80000000	// AllOtherEM
} AsMetaEmKind;

typedef enum _AsPBCondition {
	AS_PB_CONDITION_GREEN		= 0x00,
	AS_PB_CONDITION_YELLOW		= 0x01,
	AS_PB_CONDITION_RED			= 0x02,
	AS_PB_CONDITION_OTHER		= 0xff
} AsPBCondition;

typedef enum _AsDiscInLEDMode {
	AS_STOP_DISCIN_LED_BLINK		= 0x00,
	AS_START_DISCIN_LED_BLINK		= 0xc0
} AsDiscInLEDMode;

///
/// Warning Code
///
typedef enum _AsStatusTblWarnCode {
	AS_HKP_WARN_REQ_REPOWER                = 0x30900002,  // IT-Setup
	AS_HKP_WARN_HIGH_TEMP0                 = 0x30200030,  // IT-HKP
	AS_HKP_WARN_HIGH_TEMP1                 = 0x30200031,  // IT-HKP
	AS_HKP_WARN_HIGH_TEMP_DRV              = 0x30600011,  // DM			// DO NOT USE!!
	AS_HKP_WARN_TEMP_SENSOR_A              = 0x30370900,  // DM
	AS_HKP_WARN_TEMP_SENSOR_B              = 0x30370A00,  // DM
	AS_HKP_WARN_TEMP_70_FPGA               = 0x3095050E,  // DM
	AS_HKP_WARN_MOISTURE                   = 0x30600012,  // DM			// DO NOT USE!!
	AS_HKP_WARN_PSU0                       = 0x30200040,  // IT-HKP
	AS_HKP_WARN_PSU1                       = 0x30200041,  // IT-HKP
	AS_HKP_WARN_UPDATE                     = 0x30800010,  // FP
	AS_HKP_WARN_ILL_SETUP                  = 0x30300010,  // IT-Setup
	AS_HKP_WARN_HDD1_FAIL                  = 0x30700020,  // RAID
	AS_HKP_WARN_HDD2_FAIL                  = 0x30700021,  // RAID
	AS_HKP_WARN_HDD3_FAIL                  = 0x30700022,  // RAID
	AS_HKP_WARN_SSD1_FAIL                  = 0x30700024,  // RAID
	AS_HKP_WARN_SSD2_FAIL                  = 0x30700025,  // RAID
	AS_HKP_WARN_SSD3_FAIL                  = 0x30700026,  // RAID
	AS_HKP_WARN_6H_REC_REMAIN              = 0x30400071,  // IO
	AS_HKP_WARN_120H_REC_REMAIN            = 0x30400072,  // IO
	AS_HKP_WARN_LIFETIME_LASER_A           = 0x30020F2F,  // DM
	AS_HKP_WARN_LIFETIME_LASER_B           = 0x30020FAF,  // DM
	AS_HKP_WARN_CF_SMART                   = 0x30200050,  // IT-HKP
	AS_HKP_WARN_INPUT_SIG                  = 0x30100030,  // IO-HKP
	AS_HKP_WARN_AUDIO_EMPHASIS             = 0x30100040,  // IO-HKP
	AS_HKP_WARN_NON_STD_SIG                = 0x30100021,  // IO-HKP
	AS_HKP_WARN_ILL_REC                    = 0x30100031,  // IO-HKP
	AS_HKP_WARN_ILL_PLAY                   = 0x30100050,  // IO
	AS_HKP_WARN_NO_DATA                    = 0x30100042,  // IO
	AS_HKP_WARN_RAID_DIFF_FORMAT           = 0x30700023,  // RAID
	AS_HKP_WARN_CHG_BATT                   = 0x30200010,  // IT-HKP
	AS_HKP_WARN_PD_TEMP_SENSOR             = 0x30370500,  // DM			// DO NOT USE!!
	AS_HKP_WARN_PD_DEWFALL                 = 0x303C0500,  // DM			// DO NOT USE!!
	AS_HKP_WARN_PD_SEEK                    = 0x30600E00,  // DM			// DO NOT USE!!
	AS_HKP_WARN_PD_ACCELE_DETECT_SENSOR    = 0x30350500,  // DM			// DO NOT USE!!
	AS_HKP_WARN_STOP_DRV_FAN               = 0x30600010,  // DM			// DO NOT USE!!
	AS_HKP_WARN_DC_POW                     = 0x30200060,  // IT-HKP
	AS_HKP_WARN_NOT_ADJ_DRV                = 0x30600020,  // DM			// DO NOT USE!!
	AS_HKP_WARN_FS                         = 0x30400010,  // MA
	AS_HKP_WARN_ILL_INDEX                  = 0x30400032,  // Meta->MA
	AS_HKP_WARN_AUTO_FMT0                  = 0x30600017,  // MA
	AS_HKP_WARN_RD_DISC_ERR0               = 0x30600018,  // DM			// DO NOT USE!!
	AS_HKP_WARN_RD_DISC_ERR1               = 0x30600019,  // DM			// DO NOT USE!!
	AS_HKP_WARN_DISC_ERR                   = 0x30600023,  // IO
	AS_HKP_WARN_RD_DISC_A                  = 0x3052000B,  // DM
	AS_HKP_WARN_RD_DISC_B                  = 0x3052008B,  // DM
	AS_HKP_WARN_IRREG_DISC                 = 0x30600021,  // DM			// DO NOT USE!!
	AS_HKP_WARN_AUTO_FMT1                  = 0x30600022,  // MA			// DO NOT USE!!
	AS_HKP_WARN_DM_UNKNOWN_MEDIA_A         = 0x30600030,  // DM			// DO NOT USE!!
	AS_HKP_WARN_DM_UNKNOWN_MEDIA_B         = 0x30600031,  // DM			// DO NOT USE!!
	AS_HKP_WARN_NOT_REC_MEDIA_A            = 0x30600032,  // DM			// DO NOT USE!!
	AS_HKP_WARN_NOT_REC_MEDIA_B            = 0x30600033,  // DM			// DO NOT USE!!
	AS_HKP_WARN_NOT_USE_MEDIA_A            = 0x30600034,  // DM			// DO NOT USE!!
	AS_HKP_WARN_NOT_USE_MEDIA_B            = 0x30600035,  // DM			// DO NOT USE!!
	AS_HKP_WARN_UNSUP_FS_MEDIA_A           = 0x30600036,  // MA
	AS_HKP_WARN_UNSUP_FS_MEDIA_B           = 0x30600037,  // MA
	AS_HKP_WARN_REWRITING_LIMIT_MEDIA_A    = 0x30600038,  // DM->MA
	AS_HKP_WARN_REWRITING_LIMIT_MEDIA_B    = 0x30600039,  // DM->MA
	AS_HKP_WARN_MA_UNKNOWN_MEDIA_A         = 0x30400060,  // Meta->MA
	AS_HKP_WARN_MA_UNKNOWN_MEDIA_B         = 0x30400061,  // Meta->MA
	AS_HKP_WARN_MA_MEDIA_ERROR_A           = 0x30400062,  // FS->MA
	AS_HKP_WARN_MA_MEDIA_ERROR_B           = 0x30400063,  // FS->MA
	AS_HKP_WARN_DISC_SALVAGE               = 0x30400065,  // FS->MA
	AS_HKP_WARN_MA_MEDIA_ERROR_USER_XDROOT = 0x30400064,  // MA
	AS_HKP_WARN_MA_MEDIA_ERROR_USER_BPAV   = 0x30400066,  // MA
	AS_HKP_WARN_UNSUP_INDEX                = 0x30500015,  // MA
	AS_HKP_WARN_PB_HALTED_MEDIA_A          = 0x30800030,  // MA or IO	// DO NOT USE!!
	AS_HKP_WARN_PB_HALTED_MEDIA_B          = 0x30800031,  // MA or IO	// DO NOT USE!!
	AS_HKP_WARN_MEM_FULL                   = 0x30400041,  // IO
	AS_HKP_WARN_VERIFY                     = 0x30400042,  // MA
	AS_HKP_WARN_SYS_SETTINGS_1080_5994i    = 0x30400050,  // MA			// DO NOT USE!!
	AS_HKP_WARN_SYS_SETTINGS_1080_50i      = 0x30400051,  // MA			// DO NOT USE!!
	AS_HKP_WARN_SYS_SETTINGS_1080_2997P    = 0x30400052,  // MA			// DO NOT USE!!
	AS_HKP_WARN_SYS_SETTINGS_1080_25P      = 0x30400053,  // MA			// DO NOT USE!!
	AS_HKP_WARN_SYS_SETTINGS_1080_2398P    = 0x30400054,  // MA			// DO NOT USE!!
	AS_HKP_WARN_SYS_SETTINGS_720_5994P     = 0x30400055,  // MA			// DO NOT USE!!
	AS_HKP_WARN_SYS_SETTINGS_720_50P       = 0x30400056,  // MA			// DO NOT USE!!
	AS_HKP_WARN_SYS_SETTINGS_525_5994i     = 0x30400057,  // MA			// DO NOT USE!!
	AS_HKP_WARN_SYS_SETTINGS_625_50i       = 0x30400058,  // MA			// DO NOT USE!!
	AS_HKP_WARN_SYS_SETTINGS               = 0x30400059,  // MA
	AS_HKP_WARN_VIDEO_RESOL                = 0x30400051,  // MA			// DO NOT USE!!
	AS_HKP_WARN_VIDEO_REC_MODE             = 0x30400052,  // MA			// DO NOT USE!!
	AS_HKP_WARN_AUDIO_REC_MODE             = 0x30400053,  // MA			// DO NOT USE!!
	AS_HKP_WARN_NO_FS                      = 0x30600013,  // DM->MA
	AS_HKP_WARN_ILL_DISC0                  = 0x30600014,  // DM			// DO NOT USE!!
	AS_HKP_WARN_ILL_DISC1                  = 0x30600015,  // DM			// DO NOT USE!!
	AS_HKP_WARN_AUTO_FMTTING               = 0x30600016,  // MA			// DO NOT USE!!
	AS_HKP_WARN_RD_BCA                     = 0x30500010,  // DM
	AS_HKP_WARN_BCA_DATA                   = 0x30500011,  // DM
	AS_HKP_WARN_RD_PIC                     = 0x30510020,  // DM
	AS_HKP_WARN_PIC_DATA                   = 0x30510021,  // DM
	AS_HKP_WARN_UNSUP_DISC                 = 0x30510500,  // DM
	AS_HKP_WARN_NO_SPACE_ALT_AREA          = 0x30530500,  // DM
	AS_HKP_WARN_UNSUPPORTED_REC_MODE       = 0x30540000,  // DM
	AS_HKP_WARN_UNSUPPORTED_PAC            = 0x30540001,  // DM
	AS_HKP_WARN_UNSUPPORTED_SPARE_AREA     = 0x30540002,  // DM
	AS_HKP_WARN_UNSUPPORTED_DFL_ENTRY      = 0x30540003,  // DM
	AS_HKP_WARN_NO_TDMA_AREA               = 0x30540200,  // DM
	AS_HKP_WARN_FINALIZE                   = 0x30540201,  // DM
	AS_HKP_WARN_LACK_OF_OPC_AREA           = 0x30610000,  // DM
	AS_HKP_WARN_MA_DISC_DAMAGE             = 0x30600040,  // MA
	AS_HKP_WARN_STARTUP_ADJUST_A           = 0x306F0E00,  // DM
	AS_HKP_WARN_STARTUP_ADJUST_B           = 0x306F0E80,  // DM
	AS_HKP_WARN_STARTUP_ADJ_FAIL_A         = 0x306F0E03,  // DM
	AS_HKP_WARN_STARTUP_ADJ_FAIL_B         = 0x306F0E83,  // DM
	AS_HKP_WARN_FOCUS_JUMP_A               = 0x30030051,  // DM			// DO NOT USE!!
	AS_HKP_WARN_FOCUS_JUMP_B               = 0x300300D1,  // DM			// DO NOT USE!!
	AS_HKP_WARN_LOOP_GAIN_ADJUST_A         = 0x30030256,  // DM			// DO NOT USE!!
	AS_HKP_WARN_LOOP_GAIN_ADJUST_B         = 0x300302D6,  // DM			// DO NOT USE!!
	AS_HKP_WARN_TK_ADJUST_A                = 0x30030266,  // DM			// DO NOT USE!!
	AS_HKP_WARN_TK_ADJUST_B                = 0x300302E6,  // DM			// DO NOT USE!!
	AS_HKP_WARN_OVER_DUR_DISC              = 0x30400020,  // MA
	AS_HKP_WARN_OVER_DUR_MEDIA_A           = 0x30400021,  // MA
	AS_HKP_WARN_OVER_DUR_MEDIA_B           = 0x30400022,  // MA
	AS_HKP_WARN_OVER_DUR_MEDIA_USER_XDROOT = 0x30400023,  // MA
	AS_HKP_WARN_OVER_DUR_MEDIA_USER_BPAV   = 0x30400024,  // MA
	AS_HKP_WARN_UNSUP_CLIP                 = 0x30400031,  // Meta->MA
	AS_HKP_WARN_REC_INHI                   = 0x30400040,  // MA
	AS_HKP_WARN_STOP_FAN0                  = 0x30200022,  // IT-HKP
	AS_HKP_WARN_STOP_FAN1                  = 0x30200023,  // IT-HKP
	AS_HKP_WARN_STOP_FAN2                  = 0x30200025,  // IT-HKP
	AS_HKP_WARN_FPGA_FAN                   = 0x3095050F,  // DM
	AS_HKP_WARN_NO_INPUT_VIDEO             = 0x30100032,  // IO-HKP
	AS_HKP_WARN_HDD1_WARN                  = 0x30700010,  // RAID
	AS_HKP_WARN_HDD2_WARN                  = 0x30700011,  // RAID
	AS_HKP_WARN_HDD3_WARN                  = 0x30700012,  // RAID
	AS_HKP_WARN_SSD1_LIFE                  = 0x30700013,  // RAID
	AS_HKP_WARN_SSD2_LIFE                  = 0x30700014,  // RAID
	AS_HKP_WARN_SSD3_LIFE                  = 0x30700015,  // RAID
	AS_HKP_WARN_SSD1_WARN                  = 0x30700016,  // RAID
	AS_HKP_WARN_SSD2_WARN                  = 0x30700017,  // RAID
	AS_HKP_WARN_SSD3_WARN                  = 0x30700018,  // RAID
	AS_HKP_WARN_NET_CONFIG_CHANGE          = 0x30900001,  // IT-Setup
	AS_HKP_WARN_OP_PSU_DETECT              = 0x30900003,  // IT-HKP
	AS_HKP_WARN_OP_DVB_ASI_DETECT          = 0x30900004,  // IO
	AS_HKP_WARN_OP_SSD_DETECT              = 0x30900005,  // RAID
	AS_HKP_WARN_REBUILDING                 = 0x30700030,  // RAID
	AS_HKP_WARN_SLOW_DOWN_FAN0             = 0x30200020,  // IT-HKP
	AS_HKP_WARN_SLOW_DOWN_FAN1             = 0x30200021,  // IT-HKP
	AS_HKP_WARN_SLOW_DOWN_FAN2             = 0x30200024,  // IT-HKP
	AS_HKP_WARN_USB_DEV                    = 0x30800020,  // ?
	AS_HKP_WARN_LOW_TEMP0                  = 0x30200032,  // IT-HKP
	AS_HKP_WARN_LOW_TEMP1                  = 0x30200033,  // IT-HKP
	AS_HKP_WARN_LOW_REMAIN                 = 0x30400070,  // IO-CMD
	AS_HKP_WARN_REF_NOT_EXIST              = 0x30100022,  // IO-HKP
} AsStatusTblWarnCode;

typedef enum _AsStatusTblErrorCode {
	AS_HKP_ERR_OEIC_OFFSET_ADJUST_A        = 0x20020006,  // DM			// DO NOT USE!!
	AS_HKP_ERR_OEIC_OFFSET_ADJUST_B        = 0x20020086,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FPD_OFFSET_ADJUST_A         = 0x2002000A,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FPD_OFFSET_ADJUST_B         = 0x2002008A,  // DM			// DO NOT USE!!
	AS_HKP_ERR_MAXIM_LASER_A               = 0x20020020,  // DM
	AS_HKP_ERR_MAXIM_LASER_B               = 0x200200A0,  // DM
	AS_HKP_ERR_LASER_MAX_PLAY_A            = 0x20020820,  // DM
	AS_HKP_ERR_LASER_MAX_PLAY_B            = 0x200208A0,  // DM
	AS_HKP_ERR_DETERIO_LASER_A             = 0x20020F2E,  // DM			// DO NOT USE!!
	AS_HKP_ERR_DETERIO_LASER_B             = 0x20020FAE,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_NDFILTER           = 0x20020F37,  // DM			// DO NOT USE!!
	AS_HKP_ERR_LASER_OUTPUT_A              = 0x20020025,  // DM
	AS_HKP_ERR_LASER_OUTPUT_B              = 0x200200A5,  // DM
	AS_HKP_ERR_NOT_ADJUST_LASER_A          = 0x20020026,  // DM
	AS_HKP_ERR_NOT_ADJUST_LASER_B          = 0x200200A6,  // DM
	AS_HKP_ERR_NOT_ACTIVE_ADJUST_A         = 0x20020A26,  // DM
	AS_HKP_ERR_NOT_ACTIVE_ADJUST_B         = 0x20020AA6,  // DM
	AS_HKP_ERR_LASER_CUR_A                 = 0x20020027,  // DM
	AS_HKP_ERR_LASER_CUR_B                 = 0x200200A7,  // DM
	AS_HKP_ERR_NO_CARTRIDGE_A              = 0x20020028,  // DM
	AS_HKP_ERR_NO_CARTRIDGE_B              = 0x200200A8,  // DM
	AS_HKP_ERR_LCD_ATT_A                   = 0x20020037,  // DM
	AS_HKP_ERR_LCD_ATT_B                   = 0x200200B7,  // DM
	AS_HKP_ERR_FOCUS_JUMP_A                = 0x20030051,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUS_JUMP_B                = 0x200300D1,  // DM			// DO NOT USE!!
	AS_HKP_ERR_SIG_FOCUSSERVO_A            = 0x20030054,  // DM
	AS_HKP_ERR_SIG_FOCUSSERVO_B            = 0x200300D4,  // DM
	AS_HKP_ERR_FOCUSSERVO_0                = 0x20030057,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_1                = 0x20030157,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_2                = 0x20030257,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_3                = 0x20030357,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_4                = 0x20030457,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_5                = 0x20030557,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_6                = 0x20030657,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_7                = 0x20030757,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_8                = 0x20030857,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_9                = 0x20030957,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_A                = 0x20030A57,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_B                = 0x20030B57,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_C                = 0x20030C57,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_D                = 0x20030D57,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_E                = 0x20030E57,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FOCUSSERVO_F                = 0x20030F57,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_FOCUSSERVO_A          = 0x20030058,  // DM
	AS_HKP_ERR_EXCES_FOCUSSERVO_B          = 0x200300D8,  // DM
	AS_HKP_ERR_LOOP_GAIN_ADJUST_A          = 0x20030256,  // DM			// DO NOT USE!!
	AS_HKP_ERR_LOOP_GAIN_ADJUST_B          = 0x200302D6,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TK_ADJUST_A                 = 0x20030266,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TK_ADJUST_B                 = 0x200302E6,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_0             = 0x20030067,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_1             = 0x20030167,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_2             = 0x20030267,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_3             = 0x20030367,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_4             = 0x20030467,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_5             = 0x20030567,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_6             = 0x20030667,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_7             = 0x20030767,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_8             = 0x20030867,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_9             = 0x20030967,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_A             = 0x20030A67,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_B             = 0x20030B67,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_C             = 0x20030C67,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_D             = 0x20030D67,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_E             = 0x20030E67,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TRACKINGSERVO_F             = 0x20030F67,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_0       = 0x20030068,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_1       = 0x20030168,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_2       = 0x20030268,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_3       = 0x20030368,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_4       = 0x20030468,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_5       = 0x20030568,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_6       = 0x20030668,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_7       = 0x20030768,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_8       = 0x20030868,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_9       = 0x20030968,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_A       = 0x20030A68,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_B       = 0x20030B68,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_C       = 0x20030C68,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_D       = 0x20030D68,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_E       = 0x20030E68,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EXCES_TRACKINGSERVO_F       = 0x20030F68,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_INNER_DISC_A       = 0x2004007C,  // DM
	AS_HKP_ERR_NOT_MOVE_INNER_DISC_B       = 0x200400FC,  // DM
	AS_HKP_ERR_NOT_INIT_SEEK_MOTOR_A       = 0x20040073,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_INIT_SEEK_MOTOR_B       = 0x200400F3,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_0       = 0x2004007D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_1       = 0x2004017D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_2       = 0x2004027D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_3       = 0x2004037D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_4       = 0x2004047D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_5       = 0x2004057D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_6       = 0x2004067D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_7       = 0x2004077D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_8       = 0x2004087D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_9       = 0x2004097D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_A       = 0x20040A7D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_B       = 0x20040B7D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_C       = 0x20040C7D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_D       = 0x20040D7D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_E       = 0x20040E7D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NOT_MOVE_OUTER_DISC_F       = 0x20040F7D,  // DM			// DO NOT USE!!
	AS_HKP_ERR_MOVE_TARGET_POS_A           = 0x20060049,  // DM
	AS_HKP_ERR_MOVE_TARGET_POS_B           = 0x200600C9,  // DM
	AS_HKP_ERR_SAACT_POS_ADJUST_A          = 0x20060E41,  // DM
	AS_HKP_ERR_SAACT_POS_ADJUST_B          = 0x20060EC1,  // DM
	AS_HKP_ERR_SAACT_POS_INIT              = 0x20060F41,  // DM			// DO NOT USE!!
	AS_HKP_ERR_SPINDLEMOTOR                = 0x20080091,  // DM
	AS_HKP_ERR_STOP_SPINDLEMOTOR           = 0x20080095,  // DM
	AS_HKP_ERR_SPINDLEMOTOR_LODING         = 0x20080292,  // DM
	AS_HKP_ERR_SPINDLEMOTOR_UNLODING       = 0x20080992,  // DM
	AS_HKP_ERR_LORDINGMOTOR                = 0x20200018,  // DM
	AS_HKP_ERR_HOR                         = 0x20200111,  // DM
	AS_HKP_ERR_MISREG_CARTRIDGE_HOR        = 0x20200117,  // DM
	AS_HKP_ERR_LODINGMOTOR_HOR             = 0x20200118,  // DM
	AS_HKP_ERR_VERT                        = 0x20200211,  // DM
	AS_HKP_ERR_CARTRIDGE                   = 0x20200213,  // DM
	AS_HKP_ERR_MISREG_CARTRIDGE_VERT       = 0x20200217,  // DM
	AS_HKP_ERR_LODINGMOTOR_VERT            = 0x20200218,  // DM
	AS_HKP_ERR_EJECT                       = 0x20200811,  // DM
	AS_HKP_ERR_LODINGMOTOR_EJECT           = 0x20200818,  // DM
	AS_HKP_ERR_UNCHACKING                  = 0x20200911,  // DM
	AS_HKP_ERR_LODINGMOTOR_UNCHACKING      = 0x20200918,  // DM
	AS_HKP_ERR_LOADER_BOOTSEQ              = 0x20200F10,  // DM
	AS_HKP_ERR_OUTPUT_ABNORM_CODE          = 0x20340500,  // DM			// DO NOT USE!!
	AS_HKP_ERR_LOADER_POS_ABNORM_A         = 0x20340900,  // DM
	AS_HKP_ERR_LOADER_POS_ABNORM_B         = 0x20340A00,  // DM
	AS_HKP_ERR_SENSOR_HOLE_ABNORM_A        = 0x20340909,  // DM
	AS_HKP_ERR_SENSOR_HOLE_ABNORM_B        = 0x20340A09,  // DM
	AS_HKP_ERR_ACCELE_DETECT_SENSOR        = 0x20350500,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TEMP_SENSOR                 = 0x20370500,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TEMP_SENSOR_A               = 0x20370900,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TEMP_SENSOR_B               = 0x20370A00,  // DM			// DO NOT USE!!
	AS_HKP_ERR_DEWFALL                     = 0x203C0500,  // DM			// DO NOT USE!!
	AS_HKP_ERR_5VSAG_DETECT                = 0x203D0901,  // DM
	AS_HKP_ERR_RD_BCA                      = 0x20500010,  // DM			// DO NOT USE!!
	AS_HKP_ERR_BCA_DATA                    = 0x20500011,  // DM			// DO NOT USE!!
	AS_HKP_ERR_RD_PIC                      = 0x20510020,  // DM			// DO NOT USE!!
	AS_HKP_ERR_PIC_DATA                    = 0x20510021,  // DM			// DO NOT USE!!
	AS_HKP_ERR_UNSUP_DISC                  = 0x20510500,  // DM			// DO NOT USE!!
	AS_HKP_ERR_RD_DISC_A                   = 0x2052000B,  // DM			// DO NOT USE!!
	AS_HKP_ERR_RD_DISC_B                   = 0x2052008B,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NO_SPACE_ALT_AREA           = 0x20530500,  // DM			// DO NOT USE!!
	AS_HKP_ERR_UNSUPPORTED_REC_MODE        = 0x20540000,  // DM			// DO NOT USE!!
	AS_HKP_ERR_UNSUPPORTED_PAC             = 0x20540001,  // DM			// DO NOT USE!!
	AS_HKP_ERR_UNSUPPORTED_SPARE_AREA      = 0x20540002,  // DM			// DO NOT USE!!
	AS_HKP_ERR_UNSUPPORTED_DFL_ENTRY       = 0x20540003,  // DM			// DO NOT USE!!
	AS_HKP_ERR_NO_TDMA_AREA                = 0x20540200,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FINALIZE                    = 0x20540201,  // DM			// DO NOT USE!!
	AS_HKP_ERR_LACK_OF_OPC_AREA            = 0x20610000,  // DM			// DO NOT USE!!
	AS_HKP_ERR_SEEK                        = 0x20600E00,  // DM			// DO NOT USE!!
	AS_HKP_ERR_STARTUP_ADJUST_A            = 0x206F0E00,  // DM			// DO NOT USE!!
	AS_HKP_ERR_STARTUP_ADJUST_B            = 0x206F0E80,  // DM			// DO NOT USE!!
	AS_HKP_ERR_RAID_UNFORMAT               = 0x20710101,  // RAID		// DO NOT USE!!
	AS_HKP_ERR_RAID_FEW_DRIVE              = 0x20710102,  // RAID
	AS_HKP_ERR_COMMU_CPU_KEYCTRL           = 0x20910125,  // IT-NetAPI
	AS_HKP_ERR_COMMU_CPU_KEYCTRL_IT        = 0x20910215,  // FP
	AS_HKP_ERR_FLASH_MEM_IT                = 0x20910130,  // IT-HKP		// DO NOT USE!!
	AS_HKP_ERR_COMMU_CPU_IT_IO             = 0x20910145,  // IT-CMD
	AS_HKP_ERR_COMMU_CPU_DRV               = 0x20910155,  // DM
	AS_HKP_ERR_WINKLE_A_COM_ERR            = 0x20950102,  // DM
	AS_HKP_ERR_WINKLE_B_COM_ERR            = 0x20950103,  // DM
	AS_HKP_ERR_COMMU_CPU_IO_IT             = 0x20910415,  // IO-HKP
	AS_HKP_ERR_FLASH_MEM_IO                = 0x20910430,  // IO-HKP
	AS_HKP_ERR_COMMU_SATA                  = 0x20910515,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FW                          = 0x20910551,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FIRM_ABNORM                 = 0x20910557,  // DM
	AS_HKP_ERR_COMMU_UM4_FPGA_A            = 0x20910595,  // DM
	AS_HKP_ERR_COMMU_UM4_FPGA_B            = 0x209105A5,  // DM
	AS_HKP_ERR_RESP_SVDSP                  = 0x20910596,  // DM			// DO NOT USE!!
	AS_HKP_ERR_UPDATE_UM4_A                = 0x20910597,  // DM
	AS_HKP_ERR_UPDATE_UM4_B                = 0x209105A7,  // DM
	AS_HKP_ERR_EEPROM                      = 0x2091043C,  // IO-HKP
	AS_HKP_ERR_HOURS_METER                 = 0x2091043D,  // IO-HKP
	AS_HKP_ERR_INST_KEY                    = 0x209104D1,  // IO-HKP		// DO NOT USE!!
	AS_HKP_ERR_SERIAL_NO                   = 0x2091043A,  // IO-HKP
	AS_HKP_ERR_REFFOE                      = 0x20920101,  // IO-HKP
	AS_HKP_ERR_RECFOE                      = 0x20920102,  // IO-HKP
	AS_HKP_ERR_REF_VIDEO_IN_CONNECTER      = 0x20920103,  // IO-HKP		// DO NOT USE!!
	AS_HKP_ERR_24PREFFOE                   = 0x20920104,  // IO-HKP
	AS_HKP_ERR_COMMU_CPU_PCIBRIDGE         = 0x20950101,  // IO-HKP		// DO NOT USE!!
	AS_HKP_ERR_COMMU_CPU_MPG_IMX_ENC       = 0x20950404,  // IO-HKP
	AS_HKP_ERR_COMMU_CPU_VIDEO_ENC0        = 0x20950405,  // IO-HKP
	AS_HKP_ERR_COMMU_CPU_VIDEO_ENC1        = 0x20950406,  // IO-HKP
	AS_HKP_ERR_COMMU_CPU_MPG_ENC           = 0x20950407,  // IO-HKP
	AS_HKP_ERR_MPG_ENC                     = 0x20950408,  // IO-HKP
	AS_HKP_ERR_COMMU_CPU_MPG_DEC           = 0x20950409,  // IO-HKP
	AS_HKP_ERR_COMMU_CPU_PROXY_DEC         = 0x2095040F,  // IO-HKP		// DO NOT USE!!
	AS_HKP_ERR_PROXY_ENC                   = 0x20950410,  // IO-HKP		// DO NOT USE!!
	AS_HKP_ERR_PB_DSP0                     = 0x20950411,  // IO-HKP
	AS_HKP_ERR_PB_DSP1                     = 0x20950412,  // IO-HKP
	AS_HKP_ERR_REC_DSP0                    = 0x20950413,  // IO-HKP
	AS_HKP_ERR_LRZ_DSP0                    = 0x20950414,  // IO-HKP
	AS_HKP_ERR_LRZ_DSP1                    = 0x20950415,  // IO-HKP
	AS_HKP_ERR_LRZ_DSP2                    = 0x20950416,  // IO-HKP
	AS_HKP_ERR_LRZ_DSP3                    = 0x20950417,  // IO-HKP
	AS_HKP_ERR_HDMI                        = 0x20950418,  // IO-HKP
	AS_HKP_ERR_TS_OPT_MOD                  = 0x20950419,  // IO-HKP
	AS_HKP_ERR_NRFIFO                      = 0x2095041A,  // IO-HKP
	AS_HKP_ERR_BDC                         = 0x20950503,  // DM			// DO NOT USE!!
	AS_HKP_ERR_RD_ADJDATA_EEPROM           = 0x20950507,  // DM			// DO NOT USE!!
	AS_HKP_ERR_RD_HMDATA_EEPROM            = 0x20950508,  // DM			// DO NOT USE!!
	AS_HKP_ERR_RD_ADJDATA_DRV_EEPROM       = 0x20950509,  // DM			// DO NOT USE!!
	AS_HKP_ERR_SETTING_BDC_SDRAM           = 0x20950513,  // DM			// DO NOT USE!!
	AS_HKP_ERR_MPG_DEC                     = 0x2095040A,  // IO-HKP
	AS_HKP_ERR_ROVA                        = 0x2095040B,  // IO-HKP
	AS_HKP_ERR_LVIS0                       = 0x2095040C,  // IO-HKP
	AS_HKP_ERR_LVIS1                       = 0x2095040D,  // IO-HKP
	AS_HKP_ERR_APROC                       = 0x2095040E,  // IO-HKP
	AS_HKP_ERR_RD_HMDATA_DRV_EEPROM        = 0x2095050A,  // DM			// DO NOT USE!!
	AS_HKP_ERR_CONFIG_SYSPE                = 0x2095050C,  // DM			// DO NOT USE!!
	AS_HKP_ERR_TEMP_FPGA                   = 0x2095050D,  // DM
	AS_HKP_ERR_TEMP_70_FPGA                = 0x2095050E,  // DM			// DO NOT USE!!
	AS_HKP_ERR_FPGA_FAN                    = 0x2095050F,  // DM			// DO NOT USE!!
	AS_HKP_ERR_SPI_EEPROM                  = 0x20950510,  // DM
	AS_HKP_ERR_SPI_EEPROM_ABNORM           = 0x20950511,  // DM
	AS_HKP_ERR_BPI_FLASH_DATA              = 0x20950512,  // DM
	AS_HKP_ERR_BPI_FLASH_WRITE             = 0x20950513,  // DM
	AS_HKP_ERR_RTC_READ                    = 0x20950514,  // DM
	AS_HKP_ERR_SELF_RELEASE_SYSPE          = 0x2095051C,  // DM			// DO NOT USE!!
	AS_HKP_ERR_AUTO_SETTING_SYSPE          = 0x2095052C,  // DM			// DO NOT USE!!
	AS_HKP_ERR_EEPROM_ADJUST_A             = 0x20950907,  // DM
	AS_HKP_ERR_EEPROM_ADJUST_B             = 0x20950A07,  // DM
	AS_HKP_ERR_EEPROM_HOURSMETER_A         = 0x20950908,  // DM
	AS_HKP_ERR_EEPROM_HOURSMETER_B         = 0x20950A08,  // DM
	AS_HKP_ERR_SE_EEPROM_ADJUST_A          = 0x20950909,  // DM
	AS_HKP_ERR_SE_EEPROM_ADJUST_B          = 0x20950A09,  // DM
	AS_HKP_ERR_SE_EEPROM_HOURSMETER_A      = 0x2095090A,  // DM
	AS_HKP_ERR_SE_EEPROM_HOURSMETER_B      = 0x20950A0A,  // DM
	AS_HKP_ERR_PE_CONFIGURATION_A          = 0x2095090C,  // DM
	AS_HKP_ERR_PE_CONFIGURATION_B          = 0x20950A0C,  // DM
	AS_HKP_ERR_OPTICAL_BLOCK_A             = 0x20950930,  // DM
	AS_HKP_ERR_OPTICAL_BLOCK_B             = 0x20950A30,  // DM
	AS_HKP_ERR_EEPROM_DATA_A               = 0x20950940,  // DM
	AS_HKP_ERR_EEPROM_DATA_B               = 0x20950A40,  // DM
	AS_HKP_ERR_OPT_EEPROM_WRITE_A          = 0x20950941,  // DM
	AS_HKP_ERR_OPT_EEPROM_WRITE_B          = 0x20950A41,  // DM
	AS_HKP_ERR_SE_EEPROM_WRITE_A           = 0x20950942,  // DM
	AS_HKP_ERR_SE_EEPROM_WRITE_B           = 0x20950A42,  // DM
	AS_HKP_ERR_SYSTEM_ABNORMAL_A           = 0x20950960,  // DM
	AS_HKP_ERR_SYSTEM_ABNORMAL_B           = 0x20950A60,  // DM
} AsStatusTblErrorCode;

typedef struct _AsErrorStatus{
	uint32_t	error_code;
	uint32_t	priority;
} AsErrorStatus;

typedef struct _AsCmnMaterialID {
	AsMaterialType		type;				// MaterialType
	AsClipID			id;					// Clip ID / EDL ID
} AsMaterialID;

typedef struct _AsCmnMaterialProperty {
	uint32_t			dur;				// Duration
	uint32_t			total_clips;		// Number of Total Sub Clip
	uint8_t				proxy_exists;
	uint8_t				resv[3];
} AsMaterialProperty;

typedef struct _AsSearchKey {
	uint32_t			mode;				/* Clip Search Mode */
	uint32_t			clip_no;			/* Clip Number */
	uint32_t			ftc;				/* FTC */
	uint32_t			ltc;				/* LTC */
} AsSearchKey;

typedef struct _AsClipInfo {
	uint32_t			clip_no;			/* Clip Number */
	uint32_t			ftc;				/* FTC */
	uint32_t			frame_no;			/* Frame Number */
	uint32_t			ltc;				/* LTC */
	AsFps				fps;				/* FPS */
	AsAspectRatio		aspect_ratio;		/* Wide Info */
	AsVideoCodec		v_codec;			/* Video Codec */
	AsVideoPort			v_input;			/* Video Input Info */
	AsAudioCodec		a_codec[8];			/* Audio Codec */
	AsAudioPort			a_input[8];			/* Audio Input info */
	uint32_t			num_of_a_ch;		/* Number of Audio Channels */
	uint32_t			clip_id;			/* Clip ID */
	uint32_t			clip_elements;		/* Elements (File) Info */
	uint32_t			in_point;			/* Number of frames from head of a clip to IN point */
	uint32_t			duration;			/* Total number of frames in clipping range(cut) */
	uint32_t			clip_duration;		/* Total number of frames in clip */
	AsFps				capture_fps;		/* Capture Fps */
	AsNrtPdKind			pd_kind;			/* PullDown Type */
	uint32_t			a_frame_phase;		/* Start Position of A Frame */
	uint32_t			clip_begin;			/* Clip Begin */
	uint32_t			var_rec_rate;		/* Variable Rec Rate */
} AsClipInfo;

typedef struct _AsFileDataList{
	uint32_t			type;				/* File Type */
	uint32_t			add0;				/* SDRAM 0 Address */
	uint32_t			add0_size;			/* SDRAM 0 Size (byte) */
	uint32_t			add1;				/* SDRAM 1 Address */
	uint32_t			total_size;			/* Total Size (byte) */
} AsFileDataList;

typedef struct _AsDefectRubpoint {
	uint32_t			type;
	uint32_t			in_rub_no;			/* Defect Start RUB Number (-1:none / 0 or more:Defect Detection) */
	uint32_t			out_rub_no;			/* Defect End RUB Number   (-1:none / 0 or more:Defect Detection) */
} AsDefectRubpoint;

typedef struct _AsDefectList {
	uint32_t			type;
	uint32_t			offset;			/* Defect Start Position (0 or more:Defect Detection) */
	uint32_t			size;			/* Defect Size           (0:none / 1 or more:Defect Detection) */
} AsDefectList;

typedef struct _AsClipReadInfo {
	uint32_t			defect_list_num;
	AsDefectList		defect_list[AS_MAX_FILE_TYPE];
} AsClipReadInfo;

typedef struct _AsFileSeekList{
	uint32_t			type;				/* File Type */
	uint32_t			whence;				/* Whence */
	uint64_t			offset;				/* Offset (byte) */
} AsFileSeekList;

typedef struct _AsNrtChgPtData {
	uint32_t			frame_count;
	uint8_t				ltc_data[AS_NRT_LTC_DATA_LENGTH];
	uint8_t				umid_data[AS_NRT_UMID_DATA_LENGTH];
	uint8_t				klv_packet[AS_NRT_KLVPACKET_DATA_MAX];
	uint32_t			internal_klv;
} AsNrtChgPtData;

typedef struct _AsNrtCmnBuffer {
	uint8_t				clip_umid[AS_UMID_BASIC_LENGTH];
	AsRMetaKind			rmt_kind;
	AsNrtChgPtData		org_data[AS_NRT_RB_FRAME_LENGTH];
	AsNrtChgPtData		pre_data;
	AsNrtChgPtData		next_data;
} AsNrtCmnBuffer;

/*** Basic UMID Instance Number (RPS16) ***/
typedef struct _AsUmidInst16Rps {
	unsigned char		gen_info;			/* Generation Info			*/
	unsigned char		rnd[2];				/* 16bit Random Numbers		*/
} AsUmidInst16Rps;

/*** Basic UMID Instance Number (RPS24) ***/
typedef struct _AsUmidInst24Rps {
	unsigned char		rnd[3];				/* 24bit Random Numbers		*/
} AsUmidInst24Rps;

typedef union {
	AsUmidInst24Rps		rps24;				/* Instance Number (RPS24)	*/
	AsUmidInst16Rps		rps16;				/* Instance Number (RPS16)	*/
} AsUmidInstanceNumber;

typedef struct _AsUmidMaterialNumberSMPTE {
	unsigned char		snap_clk[4];		/* Snap Clock				*/
	unsigned char		mjd[3];				/* Correction Julian Day	*/
	unsigned char		tzone;				/* Time Zone				*/
	unsigned char		rnd[2];				/* Random Number			*/
	unsigned char		mac_node[6];		/* Device Identification Number	*/
} AsUmidMaterialNumberSMPTE;

typedef struct _AsUmidMaterialNumberI1394{
	unsigned char		snap_clk[4];		/* Snap Clock				*/
	unsigned char		mjd[3];				/* Correction Julian Day	*/
	unsigned char		tzone;				/* Time Zone				*/
	unsigned char		net_node[8];		/* Network Device Identification Number	*/
} AsUmidMaterialNumberI1394;

typedef struct _AsUmidBasic {
	unsigned char			label[12];		/* Universal Label			*/
	unsigned char			length;			/* Data Length 				*/
	AsUmidInstanceNumber	inst_no;		/* Instance Number			*/
	union {
		AsUmidMaterialNumberSMPTE	smpte;	/* Material Number (SMPTE method) */
		AsUmidMaterialNumberI1394	i1394;	/* Material Number (IEEE 1394 method) */
	} mate_no;
} AsUmidBasic;

typedef struct _AsDateAndTimeBlock {
	uint8_t			frame;
	uint8_t			second;
	uint8_t			minute;
	uint8_t			hour;
	uint8_t			day;
	uint8_t			month;
	uint8_t			lower_year;
	uint8_t			higher_year;
} AsDateAndTimeBlock;

typedef struct _AsSysStsBlock {
	uint8_t			io_sub_sts[8];
	uint32_t		hours_meter;
	uint32_t		hours_meter_resettable;
	uint8_t			install_key[3];
	uint8_t			dip_sw;
	AsErrorStatus	error_status;
} AsSysStsBlock;

typedef struct _AsPortStsBlock {
	uint8_t			port_sub_sts[8];
	uint8_t			alternative_sts[80];
	uint8_t			vitc[4];
	uint8_t			vitc_ub[4];
	uint8_t			ltc[4];
	uint8_t			ltc_ub[4];
	uint8_t			counter[4];
	uint32_t		offset;
	uint8_t			pull_down_time_code[4];
	uint32_t		total_offset;
	uint16_t		au_level[8];
	AsDriveID		drv_id;
	uint8_t			clip_opened_flg[2];
	uint8_t			reserve1[1];
	AsMaterialID	material_id;
	AsMaterialID	next_material_id;
} AsPortStsBlock;

typedef struct _AsBaseBandCopyBlock {
	uint8_t			basic_status;
	uint8_t			copy_handle;
	uint8_t			reserve[2];
} AsBaseBandCopyBlock;

typedef struct _AsSharedMemory {
	int32_t			key;
	uint32_t 		size;
} AsSharedMemory;

typedef struct _AsFpMaterialDetail {
	uint8_t			umid[32];
	uint8_t			create_datetime[26];
	uint8_t			mod_datetime[26];
	uint8_t			nrt_mod_datetime[26];
	uint8_t			reserve1[2];
	uint32_t		duration;
	uint32_t		key_frame;
	uint32_t		klv_packet_num;
	uint32_t		start_tc;
	uint8_t			tc_rate;
	uint8_t			clip_flag;
	uint8_t			v_format;
	uint8_t			v_frame_rate;
	uint8_t			a_info;
	uint8_t			a_ch_num;
	uint8_t			shot_mark;
	uint8_t			is_locked;
	uint8_t			sub_info;
	uint8_t			sub_err_info;
	uint8_t			reserve2;
	uint8_t			material_name_len;
	char			material_name[AS_MAX_CLIP_NAME_LEN];
} AsFpMaterialDetail;

typedef struct _AsFtpMaterialDetail {
	AsUmidBasic		basic_umid;
	uint8_t			create_datetime[26];
	uint8_t			mod_datetime[26];
	uint8_t			nrt_mod_datetime[26];
	uint8_t			reserve1[2];
	uint64_t		fs_mod_datetime;
	uint64_t		fs_nrt_mod_datetime;
	uint64_t		generalmeta_mod_datetime;
	uint32_t		duration;
	uint32_t		video_format;
	uint32_t		video_frame_rate;
	uint32_t		audio_ch_number;
	AsAudioCodec	audio_bit_len;
	AsRMetaKind		rt_meta_type;
	uint32_t		nrt_meta_file_size;
	uint32_t		edit_list_file_size;
	uint64_t		proxy_file_size;
	uint32_t		general_meta_file_size;
	uint8_t			is_locked;
	uint8_t			sub_info;
	uint8_t			sub_err_info;
	uint8_t			reserve2[5];
	AsMaterialID	material_id;
	uint32_t		material_name_len;
	char			material_name[AS_MAX_CLIP_NAME_LEN];
} AsFtpMaterialDetail;

typedef struct _AsVdcpMaterialDetail {
	AsMaterialID	material_id;
	uint8_t			is_locked;
	uint8_t			sub_info;
	uint8_t			sub_err_info;
	uint8_t			reserve;
	uint32_t		start_tc;
	uint32_t		Duration;
	uint32_t		frame_rate;
	uint32_t		video_codec;
} AsVdcpMaterialDetail;

typedef union _AsMaterialDetail {
	AsFpMaterialDetail		fp_materiall_detail;
	AsFtpMaterialDetail		ftp_materiall_detail;
	AsVdcpMaterialDetail	vdcp_materiall_detail;
} AsMaterialDetail;

typedef struct _AsStorageSize {
	uint64_t			total_remain_size;
	uint64_t			total_size;
} AsStorageSize;

typedef struct _AsChangedMaterial {
	char				material_name[AS_MAX_CLIP_NAME_LEN];
	AsMaterialID		material_id;
	AsUmidBasic			basic_umid;
} AsChangedMaterial;

typedef struct _AsRenamedMaterial {
	char				new_material_name[AS_MAX_CLIP_NAME_LEN];
	AsMaterialID		new_material_id;
	char				old_material_name[AS_MAX_CLIP_NAME_LEN];
	AsMaterialID		old_material_id;
	AsUmidBasic			basic_umid;
} AsRenamedMaterial;

typedef struct _AsUmidChangedMaterial {
	char				material_name[AS_MAX_CLIP_NAME_LEN];
	AsMaterialID		material_id;
	AsUmidBasic			new_umid;
	AsUmidBasic			old_umid;
} AsUmidChangedMaterial;

typedef struct _AsChangedMultipleMaterials {
	uint8_t				is_continue;
	uint8_t				return_material_count;
	uint8_t				reserve[2];
	AsChangedMaterial	changed_material[AS_CHANGED_MATERIAL_NUM];
} AsChangedMultipleMaterials;

typedef struct _AsFsStat {
	uint64_t	a_time; 
	uint64_t	m_time; 
	uint64_t	c_time; 
	uint32_t	size;
	uint8_t		resv[4];
} AsFsStat;

typedef struct _AsMetaStat {
	uint32_t 	size;
} AsMetaStat;

typedef union _AsStat {
	AsFsStat	fs_stat;
	AsMetaStat	meta_stat;
} AsStat;

typedef union _AsNotifyData {
	AsChangedMaterial	changed_material;
	AsRenamedMaterial	renamed_material;
	AsUmidChangedMaterial	umid_changed_material;
	AsChangedMultipleMaterials	changed_multiple_materials;
} AsNotifyData;

typedef enum _AsUmodeType {
	AS_CHANGE_MODE = 0,
	AS_UN_CHANGE_MODE,
	AS_INVALID_MODE
} AsUmodeType;

typedef struct _AsDataArea {
	uint32_t 	offset;
	uint32_t 	size;
} AsDataArea;

typedef struct _AsEjectPos {
	AsMaterialID	material_id;
	uint32_t		ftc;
} AsEjectPos;

typedef enum _AsITStatusArea {
	AS_STATUS_UNKNOWN = 0,
	AS_STATUS_MA,
	AS_STATUS_DM,
	AS_STATUS_HKP_RAID
} AsITStatusArea;

typedef struct _AsMAStatus {
	uint64_t	drive_status;
	uint32_t	total_clips;
	uint32_t	total_edls;
	uint64_t	rt_total_size;
	uint64_t	rt_total_remain_size;
	uint64_t	nrt_total_size;
	uint64_t	nrt_total_remain_size;
	uint32_t	max_total_frames;
	uint32_t	total_remain_frames;
} AsMAStatus;

typedef struct _AsDMStatus {
	uint8_t		hkp;
	uint8_t		reserved[3];
} AsDMStatus;

typedef struct _AsHKPRAIDStatus {
	uint8_t		raid1_status[8];
} AsHKPRAIDStatus;

typedef struct _AsMaterialCueupInfo {
	AsMaterialID	material_id;
	uint8_t			sub_info;
	uint8_t			sub_err_info;
	uint8_t			is_locked;
	uint8_t			reserve1;
	uint32_t		start_tc;
	uint32_t		duration;
	uint32_t		frame_rate;
	uint32_t		video_codec;
	uint32_t		position;
} AsMaterialCueupInfo;

typedef struct _AsSwapData {
	uint64_t		offset;
	uint64_t		size;
} AsSwapData;

typedef struct _AsSwapList {
	uint32_t		type;
	uint8_t			reserve[4];
	AsSwapData		header_tmp_data;
	AsSwapData		body_data;
	AsSwapData		body_tmp_data;
	AsSwapData		footer_tmp_data;
} AsSwapList;

//******************************************************************************
// Internal Messages
//******************************************************************************
typedef struct _GetMaterialId {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsGetMaterialId;

typedef struct _GetMaterialId_Comp {
	uint32_t			rc;
	AsMaterialID		material_id;
} AsGetMaterialId_Comp;

typedef struct _LockMaterialByName {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsLockMaterialByName;

typedef struct _LockMaterialByName_Comp {
	uint32_t			rc;
} AsLockMaterialByName_Comp;

typedef struct _UnLockMaterialByName {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsUnLockMaterialByName;

typedef struct _UnLockMaterialByName_Comp {
	uint32_t			rc;
} AsUnLockMaterialByName_Comp;

typedef struct _AsDeleteProtectMaterial {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
} AsDeleteProtectMaterial;

typedef struct _AsDeleteProtectMaterial_Comp {
	uint32_t			rc;
} AsDeleteProtectMaterial_Comp;

typedef struct _AsUnDeleteProtectMaterial {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
} AsUnDeleteProtectMaterial;

typedef struct _AsUnDeleteProtectMaterial_Comp {
	uint32_t			rc;
} AsUnDeleteProtectMaterial_Comp;

typedef struct _RenameMaterial {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	char				org_material_name[AS_MAX_CLIP_NAME_LEN];
	char				new_material_name[AS_MAX_CLIP_NAME_LEN];
} AsRenameMaterial;

typedef struct _RenameMaterial_Comp	{
	uint32_t			rc;
} AsRenameMaterial_Comp;

typedef struct _DeleteMaterial {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsDeleteMaterial;

typedef struct _DeleteMaterial_Comp	{
	uint32_t			rc;
} AsDeleteMaterial_Comp;

typedef struct _GetMaterialList {
	uint32_t			list_handle;
	uint32_t			material_count;
	uint32_t			dest_addr;
} AsGetMaterialList;

typedef struct _GetMaterialList_Comp {
	uint32_t			rc;
	uint8_t				is_continue;
	uint8_t				reserve[3];
	AsOpenMtrlListType	use_type;
	uint32_t			return_material_count;
	uint32_t			material_list_size;
} AsGetMaterialList_Comp;

typedef struct _GetMaterialPropertyByName {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsGetMaterialPropertyByName;

typedef struct _GetMaterialPropertyByName_Comp {
	uint32_t			rc;
	AsMaterialProperty	material_property;
} AsGetMaterialPropertyByName_Comp;

typedef struct _GetClipInfo {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
	AsSearchKey		 	key;
} AsGetClipInfo;

typedef struct _GetClipInfo_Comp {
	uint32_t			rc;
	AsClipInfo			clip_info;
} AsGetClipInfo_Comp;

typedef struct _MtrlChgNotify {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	AsMtrlChgNotifyType	notify_type;
} MtrlChgNotify;

typedef struct _OpenClip {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsClipID			clip_id;
	uint32_t			open_mode;
	uint32_t			open_elem;
} AsOpenClip;

typedef struct _OpenClip_Comp {
	uint32_t			rc;
	uint32_t			clip_handle;
	uint8_t				is_writing;
	uint8_t				resv[3];
} AsOpenClip_Comp;

typedef struct _OpenClipCreate {
	AsDriveID			drive_id;
	uint8_t				rename;
	uint8_t				resv1[6];
	char				clip_name[256];
	uint32_t			open_mode;
	uint32_t			rsv_frms;
	uint64_t			rsv_size;
	AsClipInfo			clip_info;
	uint32_t			open_elem;
} AsOpenClipCreate;

typedef struct _OpenClipCreate_Comp	{
	uint32_t			rc;
	AsClipHandle		clip_handle;
	AsMaterialID		material_id;
	uint32_t			rsv_frms;
	uint32_t			remain_frms;
	uint64_t			rsv_size;
	uint64_t			remain_size;
	uint32_t			eb;
	uint32_t			sfau;
} AsOpenClipCreate_Comp;

typedef struct _ReadClip {
	AsClipHandle		clip_handle;
	uint32_t			pri;
	uint16_t			timeout;
	uint8_t				resv[2];
	uint32_t			seg_type;
	uint32_t			file_list_num;
	AsFileDataList		file_list[AS_MAX_FILE_TYPE];
} AsReadClip;

typedef struct _ReadClip_Comp {
	uint32_t			rc;
	AsClipReadInfo		defect;
} AsReadClip_Comp;

typedef struct _WriteClip {
	AsClipHandle		clip_handle;
	uint32_t			pri;
	uint16_t			timeout;
	uint8_t				resv[2];
	uint32_t			seg_type;
	uint32_t			clip_dur;
	uint32_t			file_list_num;
	AsFileDataList		file_list[AS_MAX_FILE_TYPE];
} AsWriteClip;

typedef struct _WriteClip_Comp {
	uint32_t		rc;
	uint32_t		remain_frames;
	uint64_t		remain_size;
} AsWriteClip_Comp;

typedef struct _ReallocClip {
	AsClipHandle			clip_handle;
	uint32_t			rsv_frms;
	uint64_t			rsv_size;
} AsReallocClip;

typedef struct _ReallocClip_Comp {
	uint32_t			rc;
	uint8_t				reserve[4];
	uint32_t			rsv_frms;
	uint32_t			remain_frms;
	uint64_t			rsv_size;
	uint64_t			remain_size;
} AsReallocClip_Comp;

typedef struct _AsCancel {
	AsDriveID			drive_id;
	uint8_t				resv[3];
} AsCancel;

typedef struct _SeekClip {
	AsClipHandle		clip_handle;
	uint32_t			seek_list_num;
	AsFileSeekList		seek_list[AS_MAX_FILE_TYPE];
} AsSeekClip;

typedef struct _SeekClip_Comp {
	uint32_t			rc;
} AsSeekClip_Comp;

typedef struct _ReadPicPnt {
	AsClipHandle		clip_handle;
	uint32_t			frm_offset;
	uint32_t			num_frms;
} AsReadPicPnt;

typedef struct _ReadPicPnt_Comp	{
	uint32_t			rc;
	uint32_t			total_frm;
	uint32_t			num_frms;
	uint8_t				pic_ptr[1200*16];
} AsReadPicPnt_Comp;

typedef struct _WritePicPnt {
	AsClipHandle		clip_handle;
	uint32_t			frm_offset;
	uint32_t			num_frms;
	uint8_t				pic_ptr[1200*16];
} AsWritePicPnt;

typedef struct _WritePicPnt_Comp {
	uint32_t			rc;
} AsWritePicPnt_Comp;

typedef struct _ReadNrtMeta	{
	AsClipHandle		clip_handle;
	uint32_t			frm_offset;
	uint32_t			num_frms;
} AsReadNrtMeta;

typedef struct _ReadNrtMeta_Comp {
	uint32_t			rc;
	uint32_t			num_frms;
	AsNrtCmnBuffer		nrt_ring;
} AsReadNrtMeta_Comp;

typedef struct _WriteNrtMeta {
	AsClipHandle		clip_handle;
	uint8_t				is_end;
	uint8_t				resv[3];
	uint32_t			num_frms;
	AsNrtCmnBuffer		nrt_ring;
} AsWriteNrtMeta;

typedef struct _WriteNrtMeta_Comp {
	uint32_t			rc;
} AsWriteNrtMeta_Comp;

typedef struct _SetPlayEnable {
	AsClipHandle		clip_handle;
} AsSetPlayEnable;

typedef struct _SetPlayEnable_Comp {
	AsClipHandle		clip_handle;
	uint32_t			clip_begin;
} AsSetPlayEnable_Comp;

typedef struct _LockMaterial {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
} AsLockMaterial;

typedef struct _LockMaterial_Comp {
	uint32_t			rc;
} AsLockMaterial_Comp;

typedef struct _UnlockMaterial {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
} AsUnlockMaterial;

typedef struct _UnlockMaterial_Comp	{
	uint32_t			rc;
} AsUnlockMaterial_Comp;

typedef struct _GetMaterialProperty	{
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
} AsGetMaterialProperty;

typedef struct _GetMaterialProperty_Comp {
	uint32_t			rc;
	uint32_t			duration;
	uint32_t			total_clips;
	uint8_t				proxy_exists;
	uint8_t				is_writing;
	uint8_t				resv[2];
	uint32_t			material_status;
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsGetMaterialProperty_Comp;

typedef struct _GenerateBasicUmid {
	AsFps				fps;
} AsGenerateBasicUmid;

typedef struct _GenerateBasicUmid_Comp {
	uint32_t			rc;
	AsUmidBasic			basic_umid;
} AsGenerateBasicUmid_Comp;

typedef struct _AsGetMaterialName {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
} AsGetMaterialName;

typedef struct _AsGetMaterialName_Comp {
	uint32_t			rc;
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsGetMaterialName_Comp;

typedef struct _AsGetMaterialNameBySubclip {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
} AsGetMaterialNameBySubclip;

typedef struct _AsGetMaterialNameBySubclip_Comp {
	uint32_t			rc;
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsGetMaterialNameBySubclip_Comp;


typedef struct _GetFileSize {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	AsMaterialID		material_id;
	uint32_t			file_type;
} AsGetFileSize;

typedef struct _GetFileSize_Comp {
	uint32_t			rc;
	uint32_t			sector_size;
	uint64_t			file_size;
} AsGetFileSize_Comp;

typedef struct _AsCloseClip {
	uint32_t			clip_handle;
} AsCloseClip;

typedef struct _AsCloseClip_Comp {
	uint32_t			rc;
	uint32_t			clip_handle;
} AsCloseClip_Comp;

typedef struct _AsCloseDeleteClip {
	uint32_t			clip_handle;
} AsCloseDeleteClip;

typedef struct _AsCloseDeleteClip_Comp {
	uint32_t			rc;
	uint32_t			no_use;
} AsCloseDeleteClip_Comp;

typedef struct _AsCancel_Comp {
	uint32_t			rc;
} AsCancel_Comp;

typedef struct _OpenClipEdit {
	AsDriveID			drive_id;
	uint8_t				resv1[7];
	uint32_t			open_mode;
	uint32_t			rsv_frms;
	uint64_t			rsv_size;
	uint32_t			open_elem;
	uint32_t			clip_id;
} AsOpenClipEdit;

typedef struct _OpenClipEdit_Comp	{
	uint32_t			rc;
	AsClipHandle		tmp_clip_handle;
	uint32_t			rsv_frms;
	uint32_t			remain_frms;
	uint64_t			rsv_size;
	uint64_t			remain_size;
} AsOpenClipEdit_Comp;

typedef struct _AsSwapClipEdit {
	uint32_t		clip_handle;
	uint32_t		tmp_clip_handle;
	int32_t			picpnt_offset_cor_val;
	uint32_t		swap_list_num;
	AsSwapList		swap_list[0];
} AsSwapClipEdit;

typedef struct _AsSwapClipEdit_Comp {
	uint32_t			rc;
} AsSwapClipEdit_Comp;

typedef struct _AsCloseClipEdit {
	uint32_t			tmp_clip_handle;
} AsCloseClipEdit;

typedef struct _AsCloseClipEdit_Comp {
	uint32_t			rc;
} AsCloseClipEdit_Comp;

typedef enum _AsBootMode
{
	AS_SETUP_NORMAL = 0,
	AS_SETUP_RECOVERY = 1
} AsBootMode;

typedef struct _AsBootDone {
	AsBootMode			boot_mode;
} AsBootDone;

typedef struct _AsBootDone_Comp {
	uint32_t			rc;
	uint8_t				timecode[8];
	uint8_t				model_id;
	uint8_t				int_drv;
	uint8_t				pd_exists;
	uint8_t				sys_freq;
	uint8_t				dest;
	uint8_t				dvb_asi_ena;
	uint8_t				license_key_ena[3];
	uint8_t				reserve[3];
} AsBootDone_Comp;

typedef struct _AsFpSwitchInfo {
	int32_t				clip_numeric;
	uint32_t			switchInfo_size;
	uint8_t				switchInfo[0];
} AsFPSwitchInfo;

typedef struct _AsSetSystemTC {
	uint8_t				systemtime[8];
} AsSetSystemTC;

typedef struct _AsStartNotify {
	uint8_t				info[1];
} AsStartNotify;

typedef struct _AsStartNotify_Comp {
	uint32_t			rc;
} AsStartNotify_Comp;

typedef struct _AsShutdown_Comp {
	uint32_t			rc;
} AsShutdown_Comp;

typedef struct _AsSystemInfo {
	uint32_t			product_id;
	uint32_t			serial_no;
} AsSystemInfo;

typedef struct _AsGetITStatusTable {
	AsITStatusArea		area;
	AsDriveID			drive_id;
	uint8_t				resv[3];
} AsGetITStatusTable;

typedef struct _AsGetITStatusTable_Comp {
	uint32_t			rc;
	AsITStatusArea		area;
	AsDriveID			drive_id;
	uint8_t				resv[3];
	uint32_t			data_size;
	uint8_t				data[0];
} AsGetITStatusTable_Comp;

typedef struct _AsSelectMonitorOut {
	uint8_t				port;
	uint8_t				resv[3];
} AsSelectMonitorOut;

typedef struct _AsSelectMonitorOut_Comp {
	uint32_t			rc;
} AsSelectMonitorOut_Comp;

typedef struct _AsPresetRemote {
	uint8_t				port;
	uint8_t				data;
	uint8_t				resv[2];
} AsPresetRemote;

typedef struct _AsPresetRemote_Comp {
	uint32_t			rc;
} AsPresetRemote_Comp;

typedef struct _AsSelectMedia {
	uint8_t				port;
	AsDriveID			drive_id;
	uint8_t				resv[2];
} AsSelectMedia;

typedef struct _AsSelectMedia_Comp {
	uint32_t			rc;
} AsSelectMedia_Comp;

typedef struct _AsChgSwitch {
	uint32_t			switchinfo_size;
	uint8_t				switchinfo[5];
} AsChgSwitch;

typedef struct _AsNopassingInfo {
	uint32_t			duration;
	uint32_t			clip_begin;
	uint8_t				play_enable;
	uint8_t				resv[3];
	uint32_t			count;
	AsClipHandle		clip_handle[1];
} AsNopassingInfo;

typedef struct _AsCloseInfo {
	uint32_t			duration;
	uint32_t			count;
	AsClipHandle		clip_handle[1];
} AsCloseInfo;

typedef struct _AsPutEjectPos {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	AsEjectPos			eject_pos;
} AsPutEjectPos;

typedef struct _AsPutEjectPos_Comp {
	uint32_t			rc;
} AsPutEjectPos_Comp;

typedef struct _AsGetEjectPos {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
} AsGetEjectPos;

typedef struct _AsGetEjectPos_Comp {
	uint32_t			rc;
	AsMaterialID	material_id;
	char			material_name[AS_MAX_CLIP_NAME_LEN];
	uint32_t		ftc;
} AsGetEjectPos_Comp;

typedef struct _AsEject {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
} AsEject;

typedef struct _AsEject_Comp {
	uint32_t			rc;
} AsEject_Comp;

typedef struct _AsEjectReady {
	AsDriveID				drive_id;
	uint8_t					resv[3];
} AsEjectReady;

typedef struct _AsEjectReady_Comp {
	uint32_t			rc;
} AsEjectReady_Comp;

typedef struct _GetFtc {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	AsMaterialID		material_id;
} AsGetFtc;

typedef struct _GetFtc_Comp {
	uint32_t			rc;
	uint32_t			ftc;
} AsGetFtc_Comp;

typedef struct _AsGetMaterialCueupInfo {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsGetMaterialCueupInfo;

typedef struct _AsGetMaterialCueupInfo_Comp {
	uint32_t			rc;
	AsMaterialCueupInfo material_cueup_info;
} AsGetMaterialCueupInfo_Comp;

typedef struct _AsStandbyOffReady {
	AsDriveID				drive_id;
	uint8_t					resv[3];
} AsStandbyOffReady;

typedef struct _AsStandbyOffReady_Comp {
	uint32_t			rc;
} AsStandbyOffReady_Comp;

typedef struct _Finalize {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
} AsFinalize;

typedef struct _Finalize_Comp {
	uint32_t			rc;
} AsFinalize_Comp;

typedef struct _AsCopyNrtMeta {
	AsClipHandle		clip_handle;
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
	uint32_t			partial_out;
	uint32_t			offset_frame;
	uint32_t			duration;
} AsCopyNrtMeta;

typedef struct _AsCopyNrtMeta_Comp {
	uint32_t			rc;
} AsCopyNrtMeta_Comp;

typedef struct _AsCreateSnapshot {
	uint8_t				drive_id;
	uint8_t				reserve[3];
	char				cueup_material_name[AS_MAX_CLIP_NAME_LEN];
} AsCreateSnapshot;

typedef struct _AsCreateSnapshot_Comp {
	uint32_t			rc;
	uint32_t			cueup_material_position;
	AsMaterialID		material_id;
	AsUmidBasic			top_material_umid;
	AsUmidBasic			end_material_umid;
} AsCreateSnapshot_Comp;

typedef struct _AsDeleteSnapshot {
	uint8_t				drive_id;
	uint8_t				reserve[3];
	AsMaterialID		material_id;
} AsDeleteSnapshot;

typedef struct _AsDeleteSnapshot_Comp {
	uint32_t			rc;
} AsDeleteSnapshot_Comp;

typedef struct _AsSnapshotDurNotify {
	uint8_t				drive_id;
	uint8_t				reserve[3];
	AsMaterialID		material_id;
	AsMtrlChgNotifyType	notify_type;
	uint32_t			duration;
} AsSnapshotDurNotify;

typedef struct _AsSetEditOn {
	AsClipHandle		tmp_clip_handle;
	uint32_t			start_frame;
} AsSetEditOn;

typedef struct _AsSetEditOn_Comp {
	uint32_t			rc;
} AsSetEditOn_Comp;

typedef struct _AsSetEditOff {
	AsClipHandle		tmp_clip_handle;
	uint32_t			end_frame;
} AsSetEditOff;

typedef struct _AsSetEditOff_Comp {
	uint32_t			rc;
} AsSetEditOff_Comp;

typedef struct _AsOpenEdlCreate {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	uint32_t			start_tc;
} AsOpenEdlCreate;

typedef struct _AsOpenEdlCreate_Comp {
	uint32_t			rc;
	uint32_t			editlist_handle;
} AsOpenEdlCreate_Comp;

typedef struct _AsOpenEdlEdit {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	char				edl_name[AS_MAX_CLIP_NAME_LEN];
} AsOpenEdlEdit;

typedef struct _AsOpenEdlEdit_Comp {
	uint32_t			rc;
	uint32_t			editlist_handle;
} AsOpenEdlEdit_Comp;

typedef struct _AsCloseEdl {
	uint32_t			edl_handle;
} AsCloseEdl;

typedef struct _AsCloseEdl_Comp	{
	uint32_t			rc;
	char				edl_name[AS_MAX_CLIP_NAME_LEN];
} AsCloseEdl_Comp;

typedef struct _AsGetMediaProperty {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
} AsGetMediaProperty;

typedef struct _AsGetMediaProperty_Comp	{
	uint32_t			rc;
	uint32_t			update_count;
	uint8_t				media_properties[0];
} AsGetMediaProperty_Comp;

//******************************************************************************
// Internal Messages (NetAPI)
//******************************************************************************
typedef struct _AsNetPortStatusReport {
	AsDateAndTimeBlock	date_and_time;
	AsSysStsBlock		system_sts;
	AsPortStsBlock		port_sts_port1;
	AsPortStsBlock		port_sts_port2;
	AsBaseBandCopyBlock base_band_copy;
} AsNetPortStatusReport;

typedef struct _AsNetIdle {
	uint8_t				port;
} AsNetIdle;

typedef struct _AsNetIdle_Comp {
	uint32_t			rc;
} AsNetIdle_Comp;

typedef struct _AsNetPlay {
	uint8_t				port;
} AsNetPlay;

typedef struct _AsNetPlay_Comp {
	uint32_t			rc;
} AsNetPlay_Comp;

typedef struct _AsNetRecord {
	uint8_t				port;
} AsNetRecord;

typedef struct _AsNetRecord_Comp {
	uint32_t			rc;
} AsNetRecord_Comp;

typedef struct _AsNetRecordInit {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint8_t				start_tc[4];	//< frame, seconds, minutes, hours
	uint32_t			length;			//< num frames
	AsDriveID			drive_id;
	uint8_t				reserved2[3];
	uint8_t				clip_name_len;
	uint8_t				reserved3[3];
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsNetRecordInit;

typedef struct _AsNetRecordInit_Comp {
	uint32_t			rc;
} AsNetRecordInit_Comp;

typedef struct _AsNetPlayCue {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint8_t				start_tc[4];	//< frame, seconds, minutes, hours
	uint32_t			length;			//< num frames
	AsDriveID			drive_id;
	uint8_t				reserved2[3];
	uint8_t				clip_name_len;
	uint8_t				reserved3[3];
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsNetPlayCue;

typedef struct _AsNetPlayCue_Comp {
	uint32_t			rc;
} AsNetPlayCue_Comp;

typedef struct _AsNetPlayStop {
	uint8_t				port;
} AsNetPlayStop;

typedef struct _AsNetPlayStop_Comp {
	uint32_t			rc;
} AsNetPlayStop_Comp;

typedef struct _AsNetPlayNext {
	uint8_t				port;
} AsNetPlayNext;

typedef struct _AsNetPlayNext_Comp {
	uint32_t			rc;
} AsNetPlayNext_Comp;

typedef struct _AsNetStep {
	uint8_t				port;
	uint8_t				reserved1[3];
	int32_t				step_width;		//< frames
} AsNetStep;

typedef struct _AsNetStep_Comp {
	uint32_t			rc;
} AsNetStep_Comp;

typedef struct _AsNetShuttle {
	uint8_t				port;
	uint8_t				reserved1[3];
	int16_t				speed;			//< Speed Data
} AsNetShuttle;

typedef struct _AsNetShuttle_Comp {
	uint32_t			rc;
} AsNetSuttle_Comp;

typedef struct _AsNetCueWithData {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint8_t				position[4];	//< frame, seconds, minutes, hours
} AsNetCueWithData;

typedef struct _AsNetCueWithData_Comp {
	uint32_t			rc;
} AsNetCueWithData_Comp;

typedef struct _AsNetVLCueup {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			offset;
	uint32_t			mode;
	AsDriveID			drive_id;
	uint8_t				reserved2[3];
	uint8_t				clip_name_len;
	uint8_t				reserved3[3];
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsNetVLCueup;

typedef struct _AsNetVLCueup_Comp {
	uint32_t			rc;
} AsNetVLCueup_Comp;

typedef struct _AsNetVLRec {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			length;			//< num frames
	AsDriveID			drive_id;
	uint8_t				reserved2[3];
	uint8_t				clip_name_len;
	uint8_t				reserved3[3];
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsNetVLRec;

typedef struct _AsNetVLRec_Comp {
	uint32_t			rc;
} AsNetVLRec_Comp;

typedef struct _AsNetFFwd {
	uint8_t				port;
} AsNetFFwd;

typedef struct _AsNetFFwd_Comp {
	uint32_t			rc;
} AsNetFFwd_Comp;

typedef struct _AsNetRewind {
	uint8_t				port;
} AsNetRewind;

typedef struct _AsNetRewind_Comp {
	uint32_t			rc;
} AsNetRewind_Comp;

typedef struct _AsNetJog {
	uint8_t				port;
	uint8_t				reserved1[3];
	int16_t				speed;			//< Speed Data
} AsNetJog;

typedef struct _AsNetJog_Comp {
	uint32_t			rc;
} AsNetJog_Comp;

typedef struct _AsNetVar {
	uint8_t				port;
	uint8_t				reserved1[3];
	int16_t				speed;			//< Speed Data
} AsNetVar;

typedef struct _AsNetVar_Comp {
	uint32_t			rc;
} AsNetVar_Comp;

typedef struct _AsNetVLNext {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			offset;
	uint32_t			mode;
	uint32_t			clip_no;
	AsDriveID			drive_id;
	uint8_t				reserved2[3];
	uint8_t				clip_name_len;
	uint8_t				reserved3[3];
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsNetVLNext;

typedef struct _AsNetVLNext_Comp {
	uint32_t			rc;
} AsNetVLNext_Comp;

typedef struct _AsNetVLPrev {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			offset;
	uint32_t			mode;
	uint32_t			clip_no;
	AsDriveID			drive_id;
	uint8_t				reserved2[3];
	uint8_t				clip_name_len;
	uint8_t				reserved3[3];
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsNetVLPrev;

typedef struct _AsNetVLPrev_Comp {
	uint32_t			rc;
} AsNetVLPrev_Comp;

typedef struct _AsNetVLTop {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			reserved2;
	uint32_t			mode;
	uint32_t			clip_no;
	AsDriveID			drive_id;
	uint8_t				reserved3[3];
	uint8_t				clip_name_len;
	uint8_t				reserved4[3];
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsNetVLTop;

typedef struct _AsNetVLTop_Comp {
	uint32_t			rc;
} AsNetVLTop_Comp;

typedef struct _AsNetVLEnd {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			reserved2;
	uint32_t			mode;
	uint32_t			clip_no;
	AsDriveID			drive_id;
	uint8_t				reserved3[3];
	uint8_t				clip_name_len;
	uint8_t				reserved4[3];
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsNetVLEnd;

typedef struct _AsNetVLEnd_Comp {
	uint32_t			rc;
} AsNetVLEnd_Comp;

typedef struct _AsNetVLResumeCueup {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			offset;
	uint32_t			mode;
	AsDriveID			drive_id;
	uint8_t				reserved2[3];
	uint8_t				clip_name_len;
	uint8_t				reserved3[3];
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsNetVLResumeCueup;

typedef struct _AsNetVLResumeCueup_Comp {
	uint32_t			rc;
} AsNetVLResumeCueup_Comp;

typedef struct _AsNetVLEDLNext{
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			offset;
	uint32_t			mode;
} AsNetVLEDLNext;

typedef struct _AsNetVLEDLNext_Comp{
	uint32_t			rc;
} AsNetVLEDLNext_Comp;

typedef struct _AsNetVLEDLPrev{
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			offset;
	uint32_t			mode;
} AsNetVLEDLPrev;

typedef struct _AsNetVLEDLPrev_Comp{
	uint32_t			rc;
} AsNetVLEDLPrev_Comp;

typedef struct _AsNetVLEDLCur{
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			offset;
	uint32_t			mode;
} AsNetVLEDLCur;

typedef struct _AsNetVLEDLCur_Comp{
	uint32_t			rc;
} AsNetVLEDLCur_Comp;

typedef struct _AsNetVLEDLTop{
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			reserved2;
	uint32_t			mode;
} AsNetVLEDLTop;

typedef struct _AsNetVLEDLTop_Comp{
	uint32_t			rc;
} AsNetVLEDLTop_Comp;

typedef struct _AsNetVLEDLEnd{
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			reserved2;
	uint32_t			mode;
} AsNetVLEDLEnd;

typedef struct _AsNetVLEDLEnd_Comp{
	uint32_t			rc;
} AsNetVLEDLEnd_Comp;

typedef struct _AsNetPlayKey{
	uint8_t				port;
	uint8_t				reserved1[3];
} AsNetPlayKey;

typedef struct _AsNetPlayKey_Comp{
	uint32_t			rc;
} AsNetPlayKey_Comp;

typedef struct _AsNetRepeatPlay{
	uint8_t				port;
	uint8_t				reserved1[3];
} AsNetRepeatPlay;

typedef struct _AsNetRepeatPlay_Comp{
	uint32_t			rc;
} AsNetRepeatPlay_Comp;

typedef struct _AsNetRecEssenceMark{
	uint8_t				port;
	uint8_t				reserved[3];
	uint32_t			em_kind;
} AsNetRecEssenceMark;

typedef struct _AsNetRecEssenceMark_Comp{
	uint32_t			rc;
} AsNetRecEssenceMark_Comp;

typedef struct _AsIO2FPNotify {
	uint8_t				local_message_length;
	uint8_t				local_message;
	uint8_t				local_data[0];
} AsIO2FPNotify;

typedef struct _AsI2F_KeyDataSetNotify {
	uint8_t				key_data_set_seq_no;
	uint8_t				port;
	uint8_t				reserved1[2];
	uint8_t				key_on_data_count;
	uint8_t				reserved2[3];
	uint8_t				key_on_data[6];
	uint8_t				reserved3[2];
	uint8_t				key_off_data_count;
	uint8_t				reserved4[3];
	uint8_t				key_off_data[6];
	uint8_t				reserved5[2];
} AsI2F_KeyDataSetNotify;

typedef struct _AsEjectKeyNotify {
	AsDriveID			drive_id;
	uint8_t				reserved[3];
} AsEjectKeyNotify;

typedef struct _AsTcList {
	uint32_t			material_offset;
	uint32_t			ltc;
} AsTcList;

typedef struct _AsGetMaterialTcList {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
	uint32_t			material_offset_num;
	uint32_t			material_offset_list[AS_MAX_TC_NUM];
} AsGetMaterialTcList;

typedef struct _AsGetMaterialTcList_Comp {
	uint32_t			rc;
	uint32_t			tc_num;
	AsTcList			tc_list[AS_MAX_TC_NUM];
} AsGetMaterialTcList_Comp;

typedef struct _AsEmList {
	uint32_t			material_offset;
	uint32_t			ltc;
	AsMetaEmKind		em_kind;
	uint8_t				em_data[AS_EM_DATA_LEN];
} AsEmList;

typedef struct _AsGetMaterialEmList {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
	AsMetaEmKind		em_kind;
	uint32_t			chapter_start;
	uint32_t			chapter_num;
} AsGetMaterialEmList;

typedef struct _AsGetMaterialEmList_Comp {
	uint32_t			rc;
	uint32_t			total_em_num;
	uint32_t			em_num;
	AsEmList			em_list[AS_MAX_EM_NUM];
} AsGetMaterialEmList_Comp;

typedef struct _AsMediaLock {
	AsDriveID				drive_id;
	uint8_t					resv[3];
} AsMediaLock;

typedef struct _AsMediaLock_Comp {
	uint32_t				rc;
} AsMediaLock_Comp;

typedef struct _AsMediaUnLock {
	AsDriveID				drive_id;
	uint8_t					resv[3];
} AsMediaUnLock;

typedef struct _AsMediaUnLock_Comp {
	uint32_t				rc;
} AsMediaUnLock_Comp;

typedef struct _AsMediaInUseNotify {
	AsDriveID				drive_id;
	uint8_t					resv[3];
	uint32_t				start_end_flag;
} AsMediaInUseNotify;

typedef struct _AsAddMaterialEm {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
	AsMetaEmKind		em_kind;
	uint32_t			offset_frame;
} AsAddMaterialEm;

typedef struct _AsAddMaterialEm_Comp {
	uint32_t			rc;
	uint32_t			offset_frame;
	uint32_t			ltc;
} AsAddMaterialEm_Comp;

typedef struct _AsDeleteMaterialEm {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
	uint32_t			offset_frame_num;
	uint32_t			offset_frame_list[AS_OFFSET_FRAME_LIST_NUM];
} AsDeleteMaterialEm;

typedef struct _AsDeleteMaterialEm_Comp {
	uint32_t			rc;
	uint32_t			delete_em_num;
} AsDeleteMaterialEm_Comp;

typedef struct _AsAppendSubClip {
	uint32_t			edl_handle;
	uint32_t			in_offset;
	uint32_t			out_offset;
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsAppendSubClip;

typedef struct _AsAppendSubClip_Comp {
	uint32_t			rc;
} AsAppendSubClip_Comp;

typedef struct _AsDeleteSubClip {
	uint32_t			edl_handle;
	uint32_t			clip_no;
	uint32_t			clip_num;
} AsDeleteSubClip;

typedef struct _AsDeleteSubClip_Comp {
	uint32_t			rc;
	uint32_t			del_clip_num;
} AsDeleteSubClip_Comp;

//******************************************************************************
// Internal Messages (MA-Conv)
//******************************************************************************
typedef struct _AsPutNrtMetaXml {
	AsClipHandle		clip_handle;
	uint32_t			addr;
	uint32_t			size;
} AsPutNrtMetaXml;

typedef struct _AsPutNrtMetaXml_Comp {
	uint32_t			rc;
} AsPutNrtMetaXml_Comp;

typedef struct _AsGetNrtMetaXml {
	AsClipHandle		clip_handle;
	uint32_t			addr;
	uint32_t			size;
} AsGetNrtMetaXml;

typedef struct _AsGetNrtMetaXml_Comp {
	uint32_t			rc;
	uint32_t			size;
} AsGetNrtMetaXml_Comp;

typedef struct _AsPutGeneralMeta {
	AsClipHandle		clip_handle;
	uint32_t			addr;
	uint32_t			size;
} AsPutGeneralMeta;

typedef struct _AsPutGeneralMeta_Comp {
	uint32_t			rc;
} AsPutGeneralMeta_Comp;

typedef struct _AsGetGeneralMeta {
	AsClipHandle		clip_handle;
	uint32_t			addr;
	uint32_t			size;
} AsGetGeneralMeta;

typedef struct _AsGetGeneralMeta_Comp {
	uint32_t			rc;
	uint32_t			size;
} AsGetGeneralMeta_Comp;

typedef struct _AsAttachSharedMemory {
	AsClipHandle		clip_handle;
	AsSharedMemory		shared_mem;
} AsAttachSharedMemory;

typedef struct _AsOpenNrtFile {
	AsDriveID			drive_id;
	uint8_t				reserved[3];
	AsNrtFileType		file_type;
	char				material_name[AS_MAX_CLIP_NAME_LEN];
	uint32_t			open_mode;
	AsSharedMemory		shared_mem;
} AsOpenNrtFile;

typedef struct _AsOpenNrtFile_Comp {
	uint32_t			rc;
	uint32_t			file_handle;
} AsOpenNrtFile_Comp;

typedef struct _AsReadNrtFile {
	uint32_t			file_handle;
	uint32_t			dest_offset;
	uint32_t			dest_size;
} AsReadNrtFile;

typedef struct _AsReadNrtFile_Comp {
	uint32_t			rc;
	uint32_t			read_size;
} AsReadNrtFile_Comp;

typedef struct _AsWriteNrtFile {
	uint32_t			file_handle;
	uint32_t			src_offset;
	uint32_t			src_size;
} AsWriteNrtFile;

typedef struct _AsWriteNrtFile_Comp {
	uint32_t			rc;
} AsWriteNrtFile_Comp;

typedef struct _AsDeleteNrtFile {
	AsDriveID			drive_id;
	uint8_t				reserved[3];
	AsNrtFileType		file_type;
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsDeleteNrtFile;

typedef struct _AsDeleteNrtFile_Comp {
	uint32_t			rc;
} AsDeleteNrtFile_Comp;

typedef struct _AsSeekNrtFile {
	uint32_t	file_handle;
	uint32_t	whence;
	uint32_t	offset;
} AsSeekNrtFile;

typedef struct _AsSeekNrtFile_Comp {
	uint32_t	rc;
	uint32_t	offset;
} AsSeekNrtFile_Comp;

typedef struct _AsTruncateNrtFile {
	uint32_t			file_handle;
	uint32_t			offset;
} AsTruncateNrtFile;

typedef struct _AsTruncateNrtFile_Comp {
	uint32_t			rc;
} AsTruncateNrtFile_Comp;

typedef struct _AsReadPicPntById {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	AsMaterialID		material_id;
	uint32_t			frm_offset;
	uint32_t			num_frms;
} AsReadPicPntById;

typedef struct _AsReadPicPntById_Comp {
	uint32_t			rc;
	uint32_t			total_frms;
	uint32_t			num_frms;
	uint8_t				pic_ptr[1200*16];
} AsReadPicPntById_Comp;

typedef struct _AsCloseNrtFile {
	uint32_t			file_handle;
	AsUmodeType			umode;
} AsCloseNrtFile;

typedef struct _AsCloseNrtFile_Comp {
	uint32_t			rc;
} AsCloseNrtFile_Comp;

typedef struct _AsGetDiscNrtFileStat {
	AsDriveID			drive_id;
	uint8_t				reserved[3];
	AsNrtFileType		file_type;
} AsGetDiscNrtFileStat;

typedef struct _AsGetDiscNrtFileStat_Comp {
	uint32_t			rc;
	AsNrtFileType		file_type;
	char				file_name[AS_MAX_CLIP_NAME_LEN];
	AsStat				stat;
} AsGetDiscNrtFileStat_Comp;

typedef struct _AsUpdateClipDuration {
	AsClipHandle		clip_handle;
	uint32_t			clip_dur;
} AsUpdateClipDuration;

typedef struct _AsUpdateClipDuration_Comp {
	uint32_t			rc;
} AsUpdateClipDuration_Comp;

//******************************************************************************
// Internal Messages (Clip List)
//******************************************************************************
typedef struct _AsDiskFullNotify {
	AsDriveID				drive_id;
	uint8_t					resv1[3];
	AsDiskFullNotifyType	notify_type;
	uint8_t					is_disk_full;
	uint8_t					resv2[3];
} AsDiskFullNotify;

typedef struct _AsOpenMaterialList {
	AsDriveID				drive_id;
	uint8_t					reserved[3];
	AsOpenMtrlListType		use_type;
	AsSharedMemory		shared_mem;
} AsOpenMaterialList;

typedef struct _AsOpenMaterialList_Comp {
	uint32_t				rc;
	uint32_t				list_handle;
} AsOpenMaterialList_Comp;

typedef struct _AsCloseMaterialList {
	uint32_t				list_handle;
} AsCloseMaterialList;

typedef struct _AsCloseMaterialList_Comp {
	uint32_t				rc;
} AsCloseMaterialList_Comp;

typedef struct _AsOpenMaterialNameList {
	AsDriveID				drive_id;
	uint8_t					reserved[3];
	uint32_t				name_list_byte;
} AsOpenMaterialNameList;

typedef struct _AsOpenMaterialNameList_Comp {
	uint32_t				rc;
	uint32_t				list_handle;
} AsOpenMaterialNameList_Comp;

typedef struct _AsGetMaterialNameList {
	uint8_t					is_fixed_len;
	uint8_t					reserved[3];
	uint32_t				list_handle;
} AsGetMaterialNameList;

typedef struct _AsGetMaterialNameList_Comp {
	uint32_t				rc;
	uint32_t				remain_material_count;
	uint8_t					is_fixed_len;
	uint8_t					reserved[3];
	uint32_t				name_list_size;
	uint8_t					name_list[1];
} AsGetMaterialNameList_Comp;

typedef struct _AsCloseMaterialNameList {
	uint32_t				list_handle;
} AsCloseMaterialNameList;

typedef struct _AsCloseMaterialNameList_Comp {
	uint32_t				rc;
} AsCloseMaterialNameList_Comp;

typedef struct _AsGetMaterialDetail {
	AsDriveID				drive_id;
	uint8_t					reserved[3];
	char					material_name[AS_MAX_CLIP_NAME_LEN];
	AsOpenMtrlListType		use_type;
} AsGetMaterialDetail;

typedef struct _AsGetMaterialDetail_Comp {
	uint32_t				rc;
	AsOpenMtrlListType		use_type;
	uint8_t					material_detail[1];
} AsGetMaterialDetail_Comp;

typedef struct _GetMaterialAdditionalInfo {
	AsDriveID			drive_id;
	uint8_t				reserve[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsGetMaterialAdditionalInfo;

typedef struct _GetMaterialAdditionalInfo_Comp {
	uint32_t			rc;
	uint8_t				reserve[4];
	uint8_t				add_info[0];
} AsGetMaterialAdditionalInfo_Comp;

typedef struct _AsMaterialChangeNotify {
	AsDriveID				drive_id;
	uint8_t					reserved[3];
	AsMtrlChgNotifyType		notify_type;
	AsNotifyData			notify_deta;
} AsMaterialChangeNotify;

typedef struct _AsFormat {
	AsDriveID				drive_id;
	uint8_t					resv[3];
} AsFormat;

typedef struct _AsFormat_Comp {
	uint32_t				rc;
} AsFormat_Comp;

//******************************************************************************
// Internal Messages (Update)
//******************************************************************************
typedef struct _AsUpdateNotify {
	uint32_t				file_size;
	uint8_t					check_sum[16];
} AsUpdateNotify;

typedef struct _AsUpdatePKG_Comp {
	uint32_t				rc;
	uint32_t				file_continue;
	uint32_t				file_size;
	uint8_t					file[1];
} AsUpdatePKG_Comp;

//******************************************************************************
// Internal Messages (Copy)
//******************************************************************************
typedef struct _AsStartCopy {
	uint8_t					umid_mode;
	uint8_t					lock_key;
	uint8_t					session_no;
	AsProxyOption			proxy_option;
	uint32_t				in_time_code;
	uint32_t				duration;
	AsDriveID				src_drive_id;
	uint8_t					src_clipname_len;
	uint8_t					reserved2[2];
	char					src_clipname[AS_MAX_CLIP_NAME_LEN];
	AsDriveID				dest_drive_id;
	uint8_t					dest_clipname_len;
	uint8_t					reserved3[2];
	char					dest_clipname[AS_MAX_CLIP_NAME_LEN];
} AsStartCopy;

typedef struct _AsStartCopy_Return {
	uint32_t				status;
	uint32_t				handler;
} AsStartCopy_Return;

typedef struct _AsStartCopy_Comp {
	uint32_t				status;
	uint32_t				handler;
	uint32_t				end_frame;
} AsStartCopy_Comp;

typedef struct _AsAbortCopy {
	uint32_t				handler;
} AsAbortCopy;

typedef struct _AsAbortCopy_Comp {
	uint32_t				status;
	uint32_t				handler;
} AsAbortCopy_Comp;

typedef struct _AsEstimateClipSize {
       uint8_t                                 lock_key;
       AsProxyOption                           proxy_option;
       uint8_t                                 resv1[2];
       uint32_t                                offset_frame;
       uint32_t                                duration;
       AsDriveID                               src_drive_id;
       uint8_t                                 src_material_name_len;
       uint8_t                                 resv2[2];
       char                                    src_material_name[AS_MAX_CLIP_NAME_LEN];
       AsDriveID                               dest_drive_id;
       uint8_t                                 resv3[3];
} AsEstimateClipSize;

typedef struct _AsEstimateClipSize_Comp {
       uint32_t                                status;
       uint8_t                                 resv[4];
       uint64_t                                clip_size;
} AsEstimateClipSize_Comp;

typedef struct _AsLockCopySession_Comp {
       uint32_t                                status;
       uint8_t                                 lock_key;
       uint8_t                                 resv[3];
} AsLockCopySession_Comp;

typedef struct _AsUnLockCopySession {
       uint8_t                                 lock_key;
       uint8_t                                 resv[3];
} AsUnLockCopySession;

typedef struct _AsUnLockCopySession_Comp {
       uint32_t                                status;
       uint8_t                                 lock_key;
       uint8_t                                 resv[3];
} AsUnLockCopySession_Comp;

typedef struct _AsBasebandCopy {
	uint8_t				reserved1;
	uint8_t				reserved2;
	uint8_t				session_no;
	uint8_t				reserve1;
	uint32_t			src_in_point;
	uint32_t			src_duration;
	uint8_t				src_drive_id;
	uint8_t				src_material_name_length;
	uint8_t				reserve2[2];
	char				src_material_name[AS_MAX_CLIP_NAME_LEN];
	uint8_t				dest_drive_id;
	uint8_t				dest_material_name_length;
	uint8_t				reserve3[2];
	char				dest_material_name[AS_MAX_CLIP_NAME_LEN];
} AsBasebandCopy;

typedef struct _AsBasebandCopy_Comp {
	uint32_t			rc;
	uint32_t			reserved;
	uint32_t			end_frame;
} AsBasebandCopy_Comp;

typedef struct _AsAbortBasebandCopy {
	uint32_t			copy_handler;
} AsAbortBasebandCopy;

typedef struct _AsAbortBasebandCopy_Comp {
	uint32_t			rc;
	uint32_t			copy_handler;
} AsAbortBasebandCopy_Comp;

typedef struct _AsFPKeyNotify {
	uint8_t				key_inhi;
	uint8_t				variable_key;
	uint8_t				reserve[2];
} AsFPKeyNotify;

typedef struct _AsFPKeyStatusRequest_Comp {
	uint32_t			rc;
	uint8_t				key_inhi;
	uint8_t				variable_key;
	uint8_t				reserve[2];
	int16_t				port1_volume[8];
	int16_t				port2_volume[8];
} AsFPKeyStatusRequest_Comp;

typedef struct _AsF2I_FPModeChangeNotify{
	uint8_t				fp_status_seq_no;
	uint8_t				thumbnail_sts;
	uint8_t				clip_menu_sts;
	uint8_t				reserve[2];
} AsF2I_FPModeChangeNotify;

typedef struct _AsNetSSCueup {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			offset;
	uint32_t			mode;
	AsDriveID			drive_id;
	uint8_t				reserved2[3];
	uint8_t				clip_name_len;
	uint8_t				reserved3[3];
	char				clip_name[AS_MAX_CLIP_NAME_LEN];
} AsNetSSCueup;

typedef struct _AsNetSSCueup_Comp {
	uint32_t			rc;
} AsNetSSCueup_Comp;

typedef struct _AsNetSSResumeCueup {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			mode;
	AsDriveID			drive_id;
	uint8_t				reserved2[3];
} AsNetSSResumeCueup;

typedef struct _AsNetSSResumeCueup_Comp {
	uint32_t			rc;
} AsNetSSResumeCueup_Comp;

typedef struct _AsNetSSNext {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			offset;
	uint32_t			mode;
} AsNetSSNext;

typedef struct _AsNetSSNext_Comp {
	uint32_t			rc;
} AsNetSSNext_Comp;

typedef struct _AsNetSSPrev {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			offset;
	uint32_t			mode;
} AsNetSSPrev;

typedef struct _AsNetSSPrev_Comp {
	uint32_t			rc;
} AsNetSSPrev_Comp;

typedef struct _AsNetSSCur {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			offset;
	uint32_t			mode;
} AsNetSSCur;

typedef struct _AsNetSSCur_Comp {
	uint32_t			rc;
} AsNetSSCur_Comp;

typedef struct _AsNetSSTop {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			reserved2;
	uint32_t			mode;
} AsNetSSTop;

typedef struct _AsNetSSTop_Comp {
	uint32_t			rc;
} AsNetSSTop_Comp;

typedef struct _AsNetSSEnd {
	uint8_t				port;
	uint8_t				reserved1[3];
	uint32_t			reserved2;
	uint32_t			mode;
} AsNetSSEnd;

typedef struct _AsNetSSEnd_Comp {
	uint32_t			rc;
} AsNetSSEnd_Comp;

typedef struct _AsNetForcedEE {
	uint32_t			on_off_flag;
} AsNetForcedEE;

typedef struct _AsNetForcedEE_Comp {
	uint32_t			rc;
} AsNetForcedEE_Comp;

typedef struct _AsInternalStandbyOff {
	uint8_t				port;
	uint8_t				reserved1[3];
} AsInternalStandbyOff;

typedef struct _AsInternalStandbyOff_Comp {
	uint32_t			rc;
} AsInternalStandbyOff_Comp;

//******************************************************************************
// Internal Messages (Setup)
//******************************************************************************
typedef struct _AsReqSetup {
	uint32_t			setup_kind;
} AsReqSetup;

typedef struct _AsReqSetup_Comp {
	uint32_t			rc;
	uint32_t			setup_data_size;
	uint32_t			fmenu_data_size;
	uint8_t				setup_data[1];
} AsReqSetup_Comp;

typedef struct _AsChgSetup {
	uint32_t			menu_handle;
	uint32_t			setup_data_size;
	uint8_t				setup_data[1];
} AsChgSetup;

typedef struct _AsChgSetup_Comp {
	uint32_t			rc;
} AsChgSetup_Comp;

typedef struct _AsReloadSetup {
	uint32_t			setup_data_size;
	uint8_t				setup_data[1];
} AsReloadSetup;

typedef struct _AsReloadSetup_Comp {
	uint32_t			rc;
} AsReloadSetup_Comp;

typedef enum _AsSetupReturnCode {
	AS_SETUP_SUCCESS					= 0,
	AS_SETUP_ERR_OPEN_ID				= 0x03000000,
	AS_SETUP_ERR_MENU_ALREADY_OPEN		= 0x03000001,
	AS_SETUP_ERR_MENU_NOT_OPEN			= 0x03000002,
	AS_SETUP_ERR_LOAD_FAIL				= 0x03000010,
	AS_SETUP_ERR_SAVE_FAIL				= 0x03000011,
	AS_SETUP_ERR_CANCEL_FAIL			= 0x03000012,
	AS_SETUP_ERR_NO_DATA				= 0x03000013,
	AS_SETUP_ERR_ILLEGAL_ID_VALUE		= 0x03000020,
	AS_SETUP_ERR_REQ_CHGITEM_OF_FP		= 0x03000021,
	AS_SETUP_ERR_REQ_CHGITEM_OF_APL		= 0x03000022,
	AS_SETUP_ERR_OTHER_REASON			= 0x03000030,
	AS_SETUP_ERR_UNABLE_TO_CHG			= 0x03000031,
	AS_SETUP_ERR_NO_KEY					= 0x03000032,
	AS_SETUP_ERR_PASSWD_MISMATCH		= 0x03000033
} AsSetupReturnCode;

typedef enum _AsSetupSaveKind {
	AS_SETUP_CANCEL = 0,
	AS_SETUP_SAVE = 1
} AsSetupSaveKind;

typedef enum _AsSetupInfoId
{
	AS_SETUP_INFO_ID_ERRLOG				= 1,
	AS_SETUP_INFO_ID_SOFTWARE_VERSION,
	AS_SETUP_INFO_ID_SERIALNO,
	AS_SETUP_INFO_ID_DATA_TIME
} AsSetupInfoId;

typedef enum _AsSetupFuncId
{
	AS_SETUP_FUNC_ID_LOGCLEAR			= 0x00000001,
	AS_SETUP_FUNC_ID_FORMAT				= 0x00000002,
	AS_SETUP_FUNC_ID_REBOOT				= 0x00000003,
	AS_SETUP_FUNC_ID_OPE_HOURS_RESET	= 0x00000111,
	AS_SETUP_FUNC_ID_SEEK_HOURS0_RESET	= 0x00000113,
	AS_SETUP_FUNC_ID_SPDL_HOURS_RESET	= 0x00000114,
	AS_SETUP_FUNC_ID_LOAD_COUNT_RESET	= 0x00000115
} AsSetupFuncId;

typedef enum _AsSetupAdjId
{
	AS_SETUP_ADJ_ID_VIDEO_OUT1			= 0x0100,
	AS_SETUP_ADJ_ID_VIDEO_OUT2			= 0x0101,
	AS_SETUP_ADJ_ID_HD_74MHZ_VCO		= 0x0200,
	AS_SETUP_ADJ_ID_SD_27MHZ_VCO		= 0x0201,
	AS_SETUP_ADJ_ID_INPUT_LEVEL_CH1		= 0x0300,
	AS_SETUP_ADJ_ID_INPUT_LEVEL_CH2		= 0x0301,
	AS_SETUP_ADJ_ID_OUTPUT_LEVEL_CH1	= 0x0400,
	AS_SETUP_ADJ_ID_OUTPUT_LEVEL_CH2	= 0x0401,
	AS_SETUP_ADJ_ID_MONITOR_LEVEL_LCH	= 0x0402,
	AS_SETUP_ADJ_ID_MONITOR_LEVEL_RCH	= 0x0403
} AsSetupAdjId;

typedef enum _AsSetupSoftKeyId
{
	AS_SETUP_SOFT_KEY_ID_XDBK_103		= 0x01,
	AS_SETUP_SOFT_KEY_ID_AVC_OPTION		= 0x02,
	AS_SETUP_SOFT_KEY_ID_EDIT_OPTION	= 0x03
} AsSetupSoftKeyId;

typedef struct _AsOpenMenuData {
	uint32_t			open_id;
} AsOpenMenuData;

typedef struct _AsOpenMenuData_Comp {
	uint32_t			rc;
	uint32_t			menu_handle;
} AsOpenMenuData_Comp;

typedef struct _AsCloseMenuData {
	uint32_t			menu_handle;
	uint32_t			save_kind;
} AsCloseMenuData;

typedef struct _AsCloseMenuData_Comp {
	uint32_t			rc;
} AsCloseMenuData_Comp;

typedef struct _AsChgFMenu {
	uint32_t			menu_handle;
	uint32_t			fmenu_data_size;
	uint8_t				fmenu_data[1];
} AsChgFMenu;

typedef struct _AsChgFMenu_Comp {
	uint32_t			rc;
} AsChgFMenu_Comp;

typedef struct _AsIOChgSetup {
	uint32_t			setup_data_size;
	uint8_t				setup_data[1];
} AsIOChgSetup;

typedef struct _AsIOChgFMenu {
	uint32_t			fmenu_data_size;
	uint8_t				fmenu_data[1];
} AsIOChgFMenu;

typedef struct _AsIOReloadSetup {
	uint32_t			setup_data_size;
	uint8_t				setup_data[1];
} AsIOReloadSetup;

typedef struct _AsReqChangedSetup {
	uint32_t			menu_handle;
} AsReqChangedSetup;

typedef struct _AsReqChangedSetup_Comp {
	uint32_t			rc;
	uint32_t			setup_data_size;
	uint32_t			fmenu_data_size;
	uint8_t				setup_data[1];
} AsReqChangedSetup_Comp;

typedef struct _AsIOChgClipNumeric {
	int32_t				numeric;
} AsIOChgClipNumeric;

typedef struct _AsSetupChangeNotify {
	uint32_t			item_kind;
	uint32_t			item_value;
} AsSetupChangeNotify;

typedef struct AsReqSetupDisp {
	AsSetupInfoId		info_id;
} AsReqSetupDisp;

typedef struct AsReqSetupDisp_Comp {
	uint32_t			rc;
	uint32_t			info_data_size;
	uint8_t				info_data[1];
} AsReqSetupDisp_Comp;

typedef struct _AsChkBankData {
	uint32_t			menu_handle;
	uint32_t			bank_no;
} AsChkBankData;

typedef struct _AsChkBankData_Comp {
	uint32_t			rc;
	uint32_t			bank_status;
} AsChkBankData_Comp;

typedef struct _AsExecSetupFunc {
	uint32_t			menu_handle;
	AsSetupFuncId		func_id;
} AsExecSetupFunc;

typedef struct _AsExecSetupFunc_Comp {
	uint32_t			rc;
} AsExecSetupFunc_Comp;

typedef struct _AsResetHourMeter {
	uint32_t			func_id;
} AsResetHourMeter;

typedef struct _AsPresetAudioInputLevel {
	uint8_t				port;
	uint8_t				reserved[3];
	uint16_t			au_level[8];
} AsPresetAudioInputLevel;

typedef struct _AsPresetAudioInputLevel_Comp {
	uint32_t			rc;
} AsPresetAudioInputLevel_Comp;

typedef struct _AsPresetAudioOutputLevel {
	uint8_t				port;
	uint8_t				reserved[3];
	uint16_t			au_level[8];
} AsPresetAudioOutputLevel;

typedef struct _AsPresetAudioOutputLevel_Comp {
	uint32_t			rc;
} AsPresetAudioOutputLevel_Comp;

typedef struct _AsNetResetCounter {
	uint8_t				port;
	uint8_t				reserved[3];
} AsNetResetCounter;

typedef struct _AsNetResetCounter_Comp {
	uint32_t			rc;
} AsNetResetCounter_Comp;

typedef struct _AsStartBoardAdj {
	AsSetupAdjId		adj_id;
} AsStartBoardAdj;

typedef struct _AsStartBoardAdj_Comp {
	uint32_t			rc;
	uint32_t			adj_data;
} AsStartBoardAdj_Comp;

typedef struct _AsStopBoardAdj_Comp {
	uint32_t			rc;
} AsStopBoardAdj_Comp;

typedef struct _AsChgBoardAdj {
	uint32_t			adj_data;
} AsChgBoardAdj;

typedef struct _AsChgBoardAdj_Comp {
	uint32_t			rc;
} AsChgBoardAdj_Comp;

typedef struct _AsSaveBoardAdj {
	uint32_t			cancel;
} AsSaveBoardAdj;

typedef struct _AsSaveBoardAdj_Comp {
	uint32_t			rc;
} AsSaveBoardAdj_Comp;

typedef struct _AsReqSystem {
	uint8_t				command[0];
} AsReqSystem;

typedef struct _AsReqSystem_Comp {
	uint32_t			rc;
} AsReqSystem_Comp;

typedef struct _AsInstallSoftKey {
	uint32_t			key;
	uint8_t				passwd[AS_SOFT_KEY_PASSWD_LEN + 1];
} AsInstallSoftKey;

typedef struct _AsInstallSoftKey_Comp {
	uint32_t			rc;
} AsInstallSoftKey_Comp;

typedef struct _AsRemoveSoftKey {
	uint32_t			key;
	uint8_t				passwd[AS_SOFT_KEY_PASSWD_LEN + 1];
} AsRemoveSoftKey;

typedef struct _AsRemoveSoftKey_Comp {
	uint32_t			rc;
} AsRemoveSoftKey_Comp;

typedef struct _AsResetSoftKeyPasswd {
	uint32_t			key;
} AsResetSoftKeyPasswd;

typedef struct _AsResetSoftKeyPasswd_Comp {
	uint32_t			rc;
} AsResetSoftKeyPasswd_Comp;

typedef struct _AsIOLoadSoftKey_Comp {
	uint32_t			rc;
	uint32_t			softkey_size;
	uint8_t				softkey_data[0];
} AsIOLoadSoftKey_Comp;

typedef struct _AsIOSaveSoftKey {
	uint32_t			softkey_size;
	uint8_t				softkey_data[0];
} AsIOSaveSoftKey; 

typedef struct _AsIOSaveSoftKey_Comp {
	uint32_t			rc;
} AsIOSaveSoftKey_Comp;

//******************************************************************************
// Internal Messages (Converter Metalib)
//******************************************************************************
typedef struct _AsParseHeaderPP {
	AsUmodeType			umode;
	uint32_t			i_action;
	uint32_t			preset_tc;
	uint32_t			header_pp_size;
	uint8_t				header_pp_data[512];
} AsParseHeaderPP;

typedef struct _AsParseHeaderPP_Comp {
	uint32_t			rc;
	uint32_t			err_info;
	uint8_t				mehf_sessionid;
	uint8_t				reserve[3];
	uint32_t			header_meta_size;
	uint32_t			index_table_size;
	uint32_t			body_start_pos;
} AsParseHeaderPP_Comp;

typedef struct _AsParseHeaderMeta {
	AsDriveID			drive_id;
	uint8_t				mehf_sessionid;
	uint8_t				reserve[2];
	AsSharedMemory		shared_memory;
	uint32_t			header_meta_offset;
	uint32_t			header_meta_size;
} AsParseHeaderMeta;

typedef struct _AsParseHeaderMeta_Comp {
	uint32_t			rc;
	uint32_t			err_info;
	AsClipInfo			clip_info;
} AsParseHeaderMeta_Comp;

typedef struct _AsOpenClipCreateWithParsedHeaderFooter {
	AsDriveID			drive_id;
	uint8_t				mehf_sessionid;
	uint8_t				resv1[6];
	AsSharedMemory		shared_memory;
	char				clip_name[256];
	uint32_t			open_mode;
	uint32_t			reserve_frames;
	uint64_t			reserve_size;
	AsClipInfo			clip_info;
	uint32_t			open_elem;
} AsOpenClipCreateWithParsedHeaderFooter;

typedef struct _AsOpenClipCreateWithParsedHeaderFooter_Comp {
	uint32_t			rc;
	AsClipHandle		clip_handle;
	AsMaterialID		material_id;
	uint32_t			reserve_frames;
	uint32_t			remain_frames;
	uint64_t			reserve_size;
	uint64_t			remain_size;
	uint32_t			eb;
	uint32_t			sfau;
} AsOpenClipCreateWithParsedHeaderFooter_Comp;

typedef struct _AsCloseClipWithParsedHeaderFooter {
	AsClipHandle		clip_handle;
	uint8_t				mehf_sessionid;
	uint8_t				resv[3];
} AsCloseClipWithParsedHeaderFooter;

typedef struct _AsCloseClipWithParsedHeaderFooter_Comp {
	uint32_t			rc;
	AsClipHandle		clip_handle;
} AsCloseClipWithParsedHeaderFooter_Comp;

typedef struct _AsAsCloseSession {
	AsDriveID			drive_id;
	uint8_t				mehf_sessionid;
	uint8_t				resv[2];
} AsCloseSession;

typedef struct _AsCloseSession_Comp {
	uint32_t			rc;
} AsCloseSession_Comp;

typedef struct _AsParseFooterPP {
	AsClipHandle		clip_handle;
	uint32_t			footer_pp_size;
	uint8_t				footer_pp_data[512];
} AsParseFooterPP;

typedef struct _AsParseFooterPP_Comp {
	uint32_t			rc;
	uint32_t			err_info;
	uint32_t			header_meta_size;
} AsParseFooterPP_Comp;

typedef struct _AsParseFooterMeta {
	AsClipHandle		clip_handle;
	AsSharedMemory		shared_memory;
	uint32_t			footer_meta_offset;
	uint32_t			footer_meta_size;
} AsParseFooterMeta;

typedef struct _AsParseFooterMeta_Comp {
	uint32_t			rc;
	uint32_t			err_info;
	AsClipInfo			clip_info;
} AsParseFooterMeta_Comp;

typedef struct _AsWriteSysDataItem {
	AsClipHandle		clip_handle;
	uint32_t			clip_end;
	AsSharedMemory		shared_memory;
	uint32_t			frame_count;
	uint32_t			num_frames;
	AsDataArea			system[AS_META_LIB_NUM_FRAMES];
	AsDataArea			data[AS_META_LIB_NUM_FRAMES];
	AsDataArea			rt_meta_area;
	AsDataArea			proxy_sys_area;
} AsWriteSysDataItem;

typedef struct _AsWriteSysDataItem_Comp {
	uint32_t			rc;
	uint32_t			err_info;
	uint32_t			frame_count;
	uint32_t			num_frames;
	AsRMetaKind			rt_kind;
	AsDataArea			rt_meta_area;
	AsDataArea			proxy_sys_area;
} AsWriteSysDataItem_Comp;

typedef struct _AsCreateIntlvHeaderFooter {
	AsClipHandle		clip_handle;
	AsHFGenMode			hf_gen_mode;
	AsSharedMemory		shared_memory;
	uint32_t			audio_ch;
	uint8_t				reserve[4];
	uint64_t			body_size;
	uint32_t			duration;
	uint32_t			data_item_out;
	uint32_t			klv_out;
	uint32_t			partial_out;
	uint32_t			offset_frame;
	uint32_t			v_offset_frame;
	AsDataArea			video_area;
	AsDataArea			audio_area[AS_FILE_TYPE_AUDIO_CH8];
	AsDataArea			nrt_meta_area;
	AsDataArea			general_meta_area;
	AsDataArea			intlv_header_area;
	AsDataArea			intlv_footer_area;
	AsDataArea			intlv_body_pp_index_area;
	AsDataArea			uuid_area;
} AsCreateIntlvHeaderFooter;

typedef struct _AsCreateIntlvHeaderFooter_Comp {
	uint32_t			rc;
	AsDataArea			intlv_header_area;
	AsDataArea			intlv_footer_area;
	AsDataArea			intlv_body_pp_index_area;
	AsDataArea			uuid_area;
} AsCreateIntlvHeaderFooter_Comp;

typedef struct _AsCreateSysDataItem {
	AsClipHandle		clip_handle;
	AsSharedMemory		shared_memory;
	uint32_t			continue_count;
	uint32_t			frame_count;
	uint32_t			num_frames;
	AsRMetaKind			rt_kind;
	AsDataArea			rt_meta_area;
	AsDataArea			sys_data_area;
} AsCreateSysDataItem;

typedef struct _AsCreateSysDataItem_Comp {
	uint32_t			rc;
	uint32_t			frame_count;
	uint32_t			num_frames;
	AsDataArea			system[AS_META_LIB_NUM_FRAMES];
	AsDataArea			data[AS_META_LIB_NUM_FRAMES];
} AsCreateSysDataItem_Comp;

typedef struct _AsMakeRtMeta {
	AsClipHandle		clip_handle;
	uint8_t				is_end;
	uint8_t				resv[3];
	AsSharedMemory		shared_memory;
	uint32_t			frame_count;
	uint32_t			num_frames;
	AsRtFormatType		src_kind;
	AsDataArea			src_rt_meta_area;
	AsRtFormatType		dst_kind;
	AsDataArea			dst_rt_meta_area;
} AsMakeRtMeta;

typedef struct _AsMakeRtMeta_Comp {
	uint32_t			rc;
	uint32_t			frame_count;
	uint32_t			num_frames;
	AsDataArea			rt_meta_area;
} AsMakeRtMeta_Comp;

typedef struct _AsMakeNrtMeta {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
	AsSharedMemory		shared_memory;
	uint32_t			offset;
	uint32_t			size;
	AsUmodeType			umode_type;
	AsClipInfo			clip_info;
	uint32_t			partial_out;
	uint32_t			offset_frame;
	uint32_t			duration;
} AsMakeNrtMeta;

typedef struct _AsMakeNrtMeta_Comp {
	uint32_t			rc;
	uint32_t			size;
} AsMakeNrtMeta_Comp;

typedef struct _AsFinalizeMeta {
	AsClipHandle		clip_handle;
	uint8_t				is_abort;
	uint8_t				resv[3];
} AsFinalizeMeta;

typedef struct _AsFinalizeMeta_Comp {
	uint32_t			rc;
	uint32_t			err_info;
	uint32_t			duration;
} AsFinalizeMeta_Comp;

typedef struct _AsReadPicPntViaSharedMemory {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsClipID			clip_id;
	uint32_t			frame_offset;
	uint32_t			num_frames;
	AsSharedMemory		shared_memory;
	uint32_t			dest_offset;
	uint32_t			dest_size;
} AsReadPicPntViaSharedMemory;

typedef struct _AsReadPicPntViaSharedMemory_Comp {
	uint32_t			rc;
	uint32_t			total_frames;
	uint32_t			num_frames;
} AsReadPicPntViaSharedMemory_Comp;

typedef struct _AsCloseDeleteClipWithParsedHeaderFooter {
	uint32_t			clip_handle;
	uint8_t				mehf_sessionid;
	uint8_t				resv[3];
} AsCloseDeleteClipWithParsedHeaderFooter;

typedef struct _AsCloseDeleteClipWithParsedHeaderFooter_Comp {
	uint32_t			rc;
} AsCloseDeleteClipWithParsedHeaderFooter_Comp;

//******************************************************************************
// Internal Messages (Thumbnail)
//******************************************************************************
typedef struct _AsOpenGetThumbnail {
	uint32_t			frame;
	uint32_t			data_format;	//reserve for Ver1.1-
	AsDriveID			drive_id;
	uint8_t				resv[3];
	uint8_t				material_name_len;
	uint8_t				resv2[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsOpenGetThumbnail;

typedef struct _AsOpenGetThumbnail_Comp {
	uint32_t			status;
	uint32_t			handle;
} AsOpenGetThumbnail_Comp;

typedef struct _AsGetThumbnail {
	uint32_t			handle;
	uint32_t			size;
} AsGetThumbnail;

typedef struct _AsGetThumbnail_Comp {
	uint32_t			status;
	uint32_t			size;
	uint32_t			buffer_offset;
	uint32_t			remain_size;
} AsGetThumbnail_Comp;

typedef struct _AsCloseThumbnail {
	uint32_t			handle;
} AsCloseThumbnail;

typedef struct _AsCloseThumbnail_Comp {
	uint32_t			status;
} AsCloseThumbnail_Comp;

typedef enum _AsDeleteThumbnailCacheOption {
	DELETE_CLIP             = 0,
	DELETE_ALL              = 1,
} AsDeleteThumbnailCacheOption;
typedef struct _AsDeleteThumbnailCache {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsDeleteThumbnailCacheOption	option;
	uint8_t				material_name_len;
	uint8_t				resv2[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsDeleteThumbnailCache;


typedef struct _AsDeleteThumbnailCache_Comp {
	uint32_t			status;
} AsDeleteThumbnailCache_Comp;

typedef struct _AsRenameThumbnailCache {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	uint8_t				src_material_name_len;
	uint8_t				resv2[3];
	char				src_material_name[AS_MAX_CLIP_NAME_LEN];
	uint8_t				dst_material_name_len;
	uint8_t				resv3[3];
	char				dst_material_name[AS_MAX_CLIP_NAME_LEN];
} AsRenameThumbnailCache;

typedef struct _AsRenameThumbnailCache_Comp {
	uint32_t			status;
} AsRenameThumbnailCache_Comp;

typedef struct _AsCancelGetThumbnail {
	uint32_t			handle;
} AsCancelGetThumbnail;

typedef struct _AsCancelGetThumbnail_Comp {
	uint32_t			status;
} AsCancelGetThumbnail_Comp;

typedef struct _AsOpenThumbFile {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
	uint32_t			frame_number;
	uint32_t			open_mode;
	AsSharedMemory		shared_memory;
} AsOpenThumbFile;

typedef struct _AsOpenThumbFile_Comp {
	uint32_t			rc;
	uint32_t			file_handle;
} AsOpenThumbFile_Comp;

typedef struct _AsReadThumbFile {
	uint32_t			file_handle;
	uint32_t			dest_offset;
	uint32_t			size;
} AsReadThumbFile;

typedef struct _AsReadThumbFile_Comp {
	uint32_t			rc;
	uint32_t			read_size;
} AsReadThumbFile_Comp;

typedef struct _AsWriteThumbFile {
	uint32_t			file_handle;
	uint32_t			src_offset;
	uint32_t			size;
} AsWriteThumbFile;

typedef struct _AsWriteThumbFile_Comp {
	uint32_t			rc;
	uint32_t			write_size;
} AsWriteThumbFile_Comp;

typedef struct _AsCloseThumbFile {
	uint32_t			file_handle;
} AsCloseThumbFile;

typedef struct _AsCloseThumbFile_Comp {
	uint32_t			rc;
} AsCloseThumbFile_Comp;

typedef struct _AsSetMaterialStatusByName {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	uint32_t			material_status;
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsSetMaterialStatusByName;

typedef struct _AsSetMaterialStatusByName_Comp {
	uint32_t			rc;
} AsSetMaterialStatusByName_Comp;

typedef struct _AsSetMaterialKeyFrameByName {
	uint32_t			key_frame;
	AsDriveID			drive_id;
	uint8_t				resv[3];
	char				material_name[AS_MAX_CLIP_NAME_LEN];
} AsSetMaterialKeyFrameByName;

typedef struct _AsSetMaterialKeyFrameByName_Comp {
	uint32_t			rc;
} AsSetMaterialKeyFrameByName_Comp;

typedef struct _AsGetMaterialKeyFrame {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	AsMaterialID		material_id;
} AsGetMaterialKeyFrame;

typedef struct _AsGetMaterialKeyFrame_Comp {
	uint32_t			rc;
	uint32_t			key_frame;
} AsGetMaterialKeyFrame_Comp;

typedef struct _AsOpenGeneralFile {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	uint32_t			open_mode;
	AsSharedMemory		shared_memory;
	char				path[AS_MAX_GENERAL_PATH_LEN];
} AsOpenGeneralFile;

typedef struct _AsOpenGeneralFile_Comp {
	uint32_t			rc;
	uint32_t			file_handle;
} AsOpenGeneralFile_Comp;

typedef struct _AsWriteGeneralFile {
	uint32_t			file_handle;
	uint32_t			src_offset;
	uint32_t			size;
} AsWriteGeneralFile;

typedef struct _AsWriteGeneralFile_Comp {
	uint32_t			rc;
} AsWriteGeneralFile_Comp;

typedef struct _AsReadGeneralFile {
	uint32_t			file_handle;
	uint32_t			dest_offset;
	uint32_t			size;
} AsReadGeneralFile;

typedef struct _AsReadGeneralFile_Comp {
	uint32_t			rc;
	uint32_t			read_size;
} AsReadGeneralFile_Comp;

typedef struct _AsCloseGeneralFile {
	uint32_t			file_handle;
} AsCloseGeneralFile;

typedef struct _AsCloseGeneralFile_Comp {
	uint32_t			rc;
} AsCloseGeneralFile_Comp;

typedef struct _AsMkGeneralDir {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	char				path[AS_MAX_GENERAL_PATH_LEN];
} AsMkGeneralDir;

typedef struct _AsMkGeneralDir_Comp {
	uint32_t			rc;
} AsMkGeneralDir_Comp;

typedef struct _AsRenameGeneralFile {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	char				old_path[AS_MAX_GENERAL_PATH_LEN];
	char				new_path[AS_MAX_GENERAL_PATH_LEN];
} AsRenameGeneralFile;

typedef struct _AsRenameGeneralFile_Comp {
	uint32_t			rc;
} AsRenameGeneralFile_Comp;

typedef struct _AsDeleteGeneralFile {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	char				path[AS_MAX_GENERAL_PATH_LEN];
} AsDeleteGeneralFile;

typedef struct _AsDeleteGeneralFile_Comp {
	uint32_t			rc;
} AsDeleteGeneralFile_Comp;

typedef struct _AsRmGeneralDir {
	AsDriveID			drive_id;
	uint8_t				resv[3];
	char				path[AS_MAX_GENERAL_PATH_LEN];
} AsRmGeneralDir;

typedef struct _AsRmGeneralDir_Comp {
	uint32_t			rc;
} AsRmGeneralDir_Comp;

//******************************************************************************
// Internal Messages (Log/HouseKeeping)
//******************************************************************************
typedef enum _AsHKPReturnCode {
	AS_HKP_SUCCESS = 0,
	AS_HKP_FAILED
	/*------------------------------------------------------------------------*/
	/* HouseKeeping: 0x08000000-                                              */
	/*------------------------------------------------------------------------*/
} AsHKPReturnCode;

typedef struct _AsMessageDisp {
	uint32_t			rc;
} AsMessageDisp;

typedef struct _AsPDLogClear_Comp {
	uint32_t rc;
} AsPDLogClear_Comp;

typedef struct _AsPowerOffReq {
	uint8_t				poff_timer;
	uint8_t				resv[3];
} AsPowerOffReq;

typedef struct _AsPowerOffReq_Comp {
	uint32_t 			rc;
} AsPowerOffReq_Comp;

//******************************************************************************
// Internal Messages (DM)
//******************************************************************************
typedef struct _AsReqLEDOnOff{
	AsDriveID			drive_id;
	uint8_t				resv[7];
	uint32_t			mode;
	uint32_t			color;
} AsReqLEDOnOff;

typedef struct _AsReqVersionUp{
	AsDriveID			drive_id;
	uint8_t				type;			// AsFirmWareType
	uint16_t			file_name_len;
	char				file_name[0];
} AsReqVersionUp;

typedef struct _AsReqVersionUp_Comp {
	uint32_t rc;
} AsReqVersionUp_Comp;

typedef struct _AsReqU1MainteSendData{
	AsDriveID			drive_id;
	uint8_t				maintecode_0;
	uint8_t				maintecode_1;
	uint8_t				resv[5];
} AsReqU1MainteSendData;

typedef struct _AsReqU1MainteSendData_Comp {
	uint32_t 			rc;
	uint8_t				resv[4];
	char				reply_data[512];
} AsReqU1MainteSendData_Comp;

typedef struct _AsReqStandbyOn{
	AsDriveID			drive_id;
	uint8_t				resv[3];
} AsReqStandbyOn;

typedef struct _AsReqStandbyOn_Comp{
	uint32_t 			rc;
} AsReqStandbyOn_Comp;

typedef struct _AsNotifyStandbyStatus{
	AsDriveID			drive_id;
	uint8_t				resv[3];
	uint8_t				hkp;
	uint8_t				resv2[3];
} AsNotifyStandbyStatus;

typedef struct _AsReqAntiClogTimer {
	AsDriveID			drive_id;
	char				resv[3];
} AsReqAntiClogTimer;

typedef struct _AsReqAntiClogTimer_Comp {
	uint32_t rc;
} AsReqAntiClogTimer_Comp;

typedef struct _AsITStatusChangeNotify {
	AsITStatusArea		area;
	AsDriveID			drive_id;
	uint8_t				resv[3];
	uint32_t			data_size;
	uint8_t				data[0];
} AsITStatusChangeNotify;

typedef struct _AsNotifyPbCondition{
	AsDriveID			drive_id;
	uint8_t				resv[3];
	uint8_t				value;
	uint8_t				resv2[3];
} AsNotifyPbCondition;

typedef struct _AsNotifyDriveStatus{
	AsDriveID			drive_id;
	uint8_t				resv[3];
	uint8_t				media_condition;
	uint8_t				resv2[3];
} AsNotifyDriveStatus;

typedef struct _AsReqLampControl{
	AsDriveID			drive_id;
	uint8_t				resv[3];
	uint8_t				mode;
	uint8_t				resv2[3];
} AsReqLampControl;

//******************************************************************************
// Internal Messages (TimeLine)
//******************************************************************************
typedef struct _AsSetTLEnable{
	uint8_t				port;
	uint8_t				resv[3];
	uint8_t				enable;
	uint8_t				resv2[3];
} AsSetTLEnable;

typedef struct _AsSetTLEnable_Comp{
	uint32_t 			rc;
} AsSetTLEnable_Comp;

typedef struct _AsSetTLExecTime{
	uint8_t				port;
	uint8_t				resv[3];
	uint8_t				exec_time[4];
} AsSetTLExecTime;

typedef struct _AsSetTLExecTime_Comp{
	uint32_t 			rc;
} AsSetTLExecTime_Comp;

typedef struct _AsExecTL{
	uint8_t				port;
	uint8_t				resv[3];
} AsExecTL;

typedef struct _AsExecTL_Comp{
	uint32_t 			rc;
} AsExecTL_Comp;

typedef struct _AsStopTL{
	uint8_t				port;
	uint8_t				resv[3];
} AsStopTL;

typedef struct _AsStopTL_Comp{
	uint32_t 			rc;
} AsStopTL_Comp;

typedef struct _AsGetTLStatus{
	uint8_t				port;
	uint8_t				resv[3];
} AsGetTLStatus;

typedef struct _AsEventExecTime {
	uint8_t				Event[4];
} AsEventExecTime;

typedef struct _AsTimeLineStatus {
	uint8_t				timeline_stat[4];
	uint8_t				timeline_execution_time[4];
	uint8_t				registered_event[4];
	uint8_t				executed_event[4];
	uint32_t			inporogress_event_id;
	uint8_t				error_events[4];
	AsEventExecTime		event_exec_time[16];
} AsTimeLineStatus;

typedef struct _AsGetTLStatus_Comp{
	uint32_t 			rc;
	AsTimeLineStatus	status;
} AsGetTLStatus_Comp;

typedef struct _AsSetTLEvent{
	uint8_t				port;
	uint8_t				resv[3];
	uint32_t			event_id;
	uint32_t			event_offset;
	uint32_t			command_id;
	uint8_t				parameter[0];
} AsSetTLEvent;

typedef struct _AsSetTLEvent_Comp{
	uint32_t 			rc;
} AsSetTLEvent_Comp;

typedef struct _AsGetTLEventInfo{
	uint8_t				port;
	uint8_t				resv[3];
	uint32_t			event_id;
} AsGetTLEventInfo;

typedef struct _AsGetTLEventInfo_Comp{
	uint32_t 			rc;
	uint32_t			event_id;
	uint8_t				event_stat[4];
	uint32_t			err_code;
	uint32_t			event_offset;
	uint32_t			command_id;
	uint8_t				parameter[0];
} AsGetTLEventInfo_Comp;

typedef struct _AsRemoveTLEvent{
	uint8_t				port;
	uint8_t				resv[3];
	uint32_t			event_id;
} AsRemoveTLEvent;

typedef struct _AsRemoveTLEvent_Comp{
	uint32_t 			rc;
} AsRemoveTLEvent_Comp;

typedef struct _AsRemoveTLAllEvent{
	uint8_t				port;
	uint8_t				resv[3];
} AsRemoveTLAllEvent;

typedef struct _AsRemoveTLAllEvent_Comp{
	uint32_t 			rc;
} AsRemoveTLAllEvent_Comp;

typedef enum _AsTCGSource {
	AS_TCG_SRC_INT = 0,
	AS_TCG_SRC_EXT = 1,
	AS_TCG_SRC_SYSTEM_TC = 2,
	AS_TCG_SRC_SDI = 3
} AsTCGSource;

typedef enum _AsTCGGenerate {
	AS_TCG_RESET = 0,
	AS_TCG_TC = 1,
	AS_TCG_VITC = 2
} AsTCGGenerate;

typedef enum _AsTCGRunMode {
	AS_TCG_FREE_RUN = 0,
	AS_TCG_REC_RUN = 1
} AsTCGRunMode;

typedef enum _AsTCGCountMode {
	AS_TCG_DF = 0,
	AS_TCG_NDF = 1
} AsTCGCountMode;

typedef struct _AsSetTLTCGMode{
	uint8_t				port;
	uint8_t				resv[3];
	AsTCGSource			tcg_src;
	AsTCGGenerate		tcg_generate;
	AsTCGRunMode		tcg_run_mode;
	AsTCGCountMode		tcg_count_mode;
} AsSetTLTCGMode;

typedef struct _AsSetTLTCGMode_Comp{
	uint32_t 			rc;
} AsSetTLTCGMode_Comp;

typedef struct _AsPresetTLStartTCG{
	uint8_t				port;
	uint8_t				resv[3];
	uint8_t				preset_value[4];
} AsPresetTLStartTCG;

typedef struct _AsPresetTLStartTCG_Comp{
	uint32_t 			rc;
} AsPresetTLStartTCG_Comp;

//******************************************************************************
// Internal Messages (SxS Key Option)
//******************************************************************************
typedef struct _AsReqCheckKey {
	AsDriveID			drive_id;
	uint8_t				resv1[3];
	uint8_t				check_identifier;
	uint8_t				resv2[3];
	uint8_t				key_id[AS_KEY_ID_LEN];
	uint8_t		 		password[AS_PASSWORD_LEN];
	uint8_t 			answer[AS_ANSWER_LEN];
} AsReqCheckKey;

typedef struct _AsReqCheckKey_Comp {
	uint32_t rc;
} AsReqCheckKey_Comp;

typedef struct _AsReqDeleteKey {
	AsDriveID			drive_id;
	uint8_t				resv1[3];
	uint8_t				delete_identifier;
	uint8_t				resv2[3];
	uint8_t				key_id[AS_KEY_ID_LEN];
	uint8_t		 		password[AS_PASSWORD_LEN];
	uint8_t 			answer[AS_ANSWER_LEN];
} AsReqDeleteKey;

typedef struct _AsReqDeleteKey_Comp {
	uint32_t rc;
} AsReqDeleteKey_Comp;

typedef struct _AsReqSetKey {
	AsDriveID			drive_id;
	uint8_t				resv1[3];
	uint8_t				set_identifier;
	uint8_t				resv2[3];
	uint8_t				key_id[AS_KEY_ID_LEN];
	uint8_t		 		password[AS_PASSWORD_LEN];
	uint8_t 			answer[AS_ANSWER_LEN];
	uint8_t 			hint[AS_HINT_LEN];
} AsReqSetKey;

typedef struct _AsReqSetKey_Comp {
	uint32_t rc;
} AsReqSetKey_Comp;

#endif	// __ASCACMN_IMSG_H

/* end of $RCSfile: asca_imsg.h,v $
 * COPYRIGHT (C) 2009 SONY Corporation
 */
