/*
 * Part of Very Secure FTPd
 * Licence: GPL v2
 * Author: Chris Evans
 * postlogin.c
 */

#include <stdio.h>
#include <sys/time.h>	// add : 10/06/01 masuu
#include <errno.h>

#include "postlogin.h"
#include "session.h"
#include "oneprocess.h"
#include "twoprocess.h"
#include "ftpcodes.h"
#include "ftpcmdio.h"
#include "ftpdataio.h"
#include "utility.h"
#include "tunables.h"
#include "defs.h"
#include "str.h"
#include "sysstr.h"
#include "banner.h"
#include "sysutil.h"
#include "logging.h"
#include "sysdeputil.h"
#include "ipaddrparse.h"
#include "access.h"
#include "features.h"
#include "ssl.h"
#include "xdcstat.h"		/* add : for xdcam stat 10/06/21 masuu */
#include "vsftpver.h"
#include "common/ioctl.h"	/* add : for ioctl	09/12/10 masuu */

/* Private local functions */
static void handle_pwd(struct vsf_session* p_sess);
static void handle_cwd(struct vsf_session* p_sess);
static void handle_pasv(struct vsf_session* p_sess, int is_epsv);
static void handle_retr(struct vsf_session* p_sess);
static void handle_cdup(struct vsf_session* p_sess);
static void handle_list(struct vsf_session* p_sess);
static void handle_type(struct vsf_session* p_sess);
static void handle_port(struct vsf_session* p_sess);
static void handle_stor(struct vsf_session* p_sess);
static void handle_mkd(struct vsf_session* p_sess);
static void handle_rmd(struct vsf_session* p_sess);
static void handle_dele(struct vsf_session* p_sess);
static void handle_rest(struct vsf_session* p_sess);
static void handle_rnfr(struct vsf_session* p_sess);
static void handle_rnto(struct vsf_session* p_sess);
static void handle_nlst(struct vsf_session* p_sess);
static void handle_size(struct vsf_session* p_sess);
static void handle_site(struct vsf_session* p_sess);
static void handle_appe(struct vsf_session* p_sess);
static void handle_mdtm(struct vsf_session* p_sess);
static void handle_site_chmod(struct vsf_session* p_sess, struct mystr* p_arg_str);
static void handle_site_umask(struct vsf_session* p_sess, struct mystr* p_arg_str);
static void handle_site_ummd(struct vsf_session* p_sess, struct mystr* p_arg_str);		/* add : 10/04/22 masuu */
static void handle_site_func(struct vsf_session* p_sess, struct mystr* p_arg_str);		/* add : 10/12/06 masuu */
static void handle_site_df(struct vsf_session* p_sess, struct mystr* p_arg_str);		/* add : 11/03/10 masuu */
static void handle_site_logflush(struct vsf_session* p_sess, struct mystr* p_arg_str);	/* add : 10/09/08 masuu */
static void handle_site_repf(struct vsf_session* p_sess, struct mystr* p_arg_str);		/* add : 09/12/01 masuu */
static void handle_site_repfl(struct vsf_session* p_sess, struct mystr* p_arg_str);		/* add : 09/12/15 masuu */
static void handle_eprt(struct vsf_session* p_sess);
static void handle_help(struct vsf_session* p_sess);
static void handle_stou(struct vsf_session* p_sess);
static void handle_stat(struct vsf_session* p_sess);
static void handle_stat_file(struct vsf_session* p_sess);

static int pasv_active(struct vsf_session* p_sess);
static int port_active(struct vsf_session* p_sess);
static void pasv_cleanup(struct vsf_session* p_sess);
static void port_cleanup(struct vsf_session* p_sess);
static void handle_dir_common(struct vsf_session* p_sess, int full_details,
                              int stat_cmd);
static void prepend_path_to_filename(struct mystr* p_str);
static int get_remote_transfer_fd(struct vsf_session* p_sess,
                                  const char* p_status_msg);
static void check_abor(struct vsf_session* p_sess);
static void handle_sigurg(void* p_private);
static void handle_upload_common(struct vsf_session* p_sess, int is_append,
                                 int is_unique);
static void get_unique_filename(struct mystr* p_outstr,
                                const struct mystr* p_base);
static int data_transfer_checks_ok(struct vsf_session* p_sess);
static void resolve_tilde(struct mystr* p_str, struct vsf_session* p_sess);

/* add : 10/06/01 masuu */
static void transfer_speed_write(struct vsf_session* p_sess, struct timeval *pt_starttime, struct timeval *pt_endtime, filesize_t xferdsize);

/* add : CQ3313 10/12/13 masuu */
static void
handle_sigchld(int duff)
{
  struct vsf_sysutil_wait_retval wait_retval = vsf_sysutil_wait();
  (void) duff;
  /* Child died, so we'll do the same! Report it as an error unless the child
   * exited normally with zero exit code
   */
  if (vsf_sysutil_retval_is_error(vsf_sysutil_wait_get_retval(&wait_retval)) ||
      !vsf_sysutil_wait_exited_normally(&wait_retval) ||
      vsf_sysutil_wait_get_exitcode(&wait_retval) != 0)
  { 
    die("child died");
  }
  else
  {
    vsf_sysutil_exit(0);
  }
}

void
process_post_login(struct vsf_session* p_sess)
{
  str_getcwd(&p_sess->home_str);
  if (p_sess->is_anonymous)
  {
    vsf_sysutil_set_umask(tunable_anon_umask);
    p_sess->bw_rate_max = tunable_anon_max_rate;
  }
  else
  {
    vsf_sysutil_set_umask(tunable_local_umask);
    p_sess->bw_rate_max = tunable_local_max_rate;
  }
  if (tunable_async_abor_enable)
  {
    vsf_sysutil_install_sighandler(kVSFSysUtilSigURG, handle_sigurg, p_sess);
    vsf_sysutil_activate_sigurg(VSFTP_COMMAND_FD);
  }
  /* Handle any login message */
  vsf_banner_dir_changed(p_sess, FTP_LOGINOK);
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/27 masuu */
  vsf_cmdio_write(p_sess, FTP_LOGINOK, "User logged in, proceeds.");
#else /* _RES_TYPE_XDS_ */
  vsf_cmdio_write(p_sess, FTP_LOGINOK, "Login successful.");
#endif /* _RES_TYPE_XDS_ */
  while(1)
  {
    int cmd_ok = 1;
    if (tunable_setproctitle_enable)
    {
      vsf_sysutil_setproctitle("IDLE");
    }
    /* Blocks */
    vsf_cmdio_get_cmd_and_arg(p_sess, &p_sess->ftp_cmd_str,
                              &p_sess->ftp_arg_str, 1);
    if (tunable_setproctitle_enable)
    {
      struct mystr proctitle_str = INIT_MYSTR;
      str_copy(&proctitle_str, &p_sess->ftp_cmd_str);
      if (!str_isempty(&p_sess->ftp_arg_str))
      {
        str_append_char(&proctitle_str, ' ');
        str_append_str(&proctitle_str, &p_sess->ftp_arg_str);
      }
      /* Suggestion from Solar */
      str_replace_unprintable(&proctitle_str, '?');
      vsf_sysutil_setproctitle_str(&proctitle_str);
      str_free(&proctitle_str);
    }
    /* Test command against the allowed list.. */
    if (tunable_cmds_allowed)
    {
      static struct mystr s_src_str;
      static struct mystr s_rhs_str;
      str_alloc_text(&s_src_str, tunable_cmds_allowed);
      while (1)
      {
        str_split_char(&s_src_str, &s_rhs_str, ',');
        if (str_isempty(&s_src_str))
        {
          cmd_ok = 0;
          break;
        }
        else if (str_equal(&s_src_str, &p_sess->ftp_cmd_str))
        {
          break;
        }
        str_copy(&s_src_str, &s_rhs_str);
      }
    }
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/27 masuu */
    if (!cmd_ok)
    {
      vsf_cmdio_write(p_sess, FTP_BADCMD, "Syntax error, command unrecognized.");
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "QUIT"))
    {
      vsf_cmdio_write(p_sess, FTP_GOODBYE, "Service closing control connection.");
#if(1) /* chg : exit routine 11/04/25 masuu */
      vsf_sysutil_all_exit(p_sess);
#else
      vsf_sysutil_exit(0);
#endif
    }
#else /* _RES_TYPE_XDS_ */
    if (!cmd_ok)
    {
      vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "QUIT"))
    {
      vsf_cmdio_write(p_sess, FTP_GOODBYE, "Goodbye.");
      vsf_sysutil_exit(0);
    }
#endif /* _RES_TYPE_XDS_ */
    else if (str_equal_text(&p_sess->ftp_cmd_str, "PWD") ||
             str_equal_text(&p_sess->ftp_cmd_str, "XPWD"))
    {
      handle_pwd(p_sess);
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "CWD") ||
             str_equal_text(&p_sess->ftp_cmd_str, "XCWD"))
    {
      handle_cwd(p_sess);
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "CDUP") ||
             str_equal_text(&p_sess->ftp_cmd_str, "XCUP"))
    {
      handle_cdup(p_sess);
    }
#ifdef _CMD_TYPE_XDS_	/* chg : for XDS 10/05/20 masuu */
    else if (tunable_pasv_enable &&
             !p_sess->epsv_all &&
             str_equal_text(&p_sess->ftp_cmd_str, "PASV"))
    {
      handle_pasv(p_sess, 0);
    }
#else /* _CMD_TYPE_XDS_ */
    else if (tunable_pasv_enable &&
             !p_sess->epsv_all &&
             (str_equal_text(&p_sess->ftp_cmd_str, "PASV") ||
              str_equal_text(&p_sess->ftp_cmd_str, "P@SW")))
    {
      handle_pasv(p_sess, 0);
    }
    else if (tunable_pasv_enable &&
             str_equal_text(&p_sess->ftp_cmd_str, "EPSV"))
    {
      handle_pasv(p_sess, 1);
    }
#endif /* _CMD_TYPE_XDS_ */
    else if (tunable_download_enable &&
             str_equal_text(&p_sess->ftp_cmd_str, "RETR"))
    {
      handle_retr(p_sess);
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "NOOP"))
    {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/28 masuu */
      vsf_cmdio_write(p_sess, FTP_NOOPOK, "Command okay.");
#else /* _RES_TYPE_XDS_ */
      vsf_cmdio_write(p_sess, FTP_NOOPOK, "NOOP ok.");
#endif /* _RES_TYPE_XDS_ */
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "SYST"))
    {
      vsf_cmdio_write(p_sess, FTP_SYSTOK, "UNIX Type: L8");
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "HELP"))
    {
      handle_help(p_sess);
    }
    else if (tunable_dirlist_enable &&
             str_equal_text(&p_sess->ftp_cmd_str, "LIST"))
    {
      handle_list(p_sess);
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "TYPE"))
    {
      handle_type(p_sess);
    }
    else if (tunable_port_enable &&
             !p_sess->epsv_all &&
             str_equal_text(&p_sess->ftp_cmd_str, "PORT"))
    {
      handle_port(p_sess);
    }
    else if (tunable_write_enable &&
             (tunable_anon_upload_enable || !p_sess->is_anonymous) &&
             str_equal_text(&p_sess->ftp_cmd_str, "STOR"))
    {
      handle_stor(p_sess);
    }
    else if (tunable_write_enable &&
             (tunable_anon_mkdir_write_enable || !p_sess->is_anonymous) &&
             (str_equal_text(&p_sess->ftp_cmd_str, "MKD") ||
              str_equal_text(&p_sess->ftp_cmd_str, "XMKD")))
    {
      handle_mkd(p_sess);
    }
    else if (tunable_write_enable &&
             (tunable_anon_other_write_enable || !p_sess->is_anonymous) &&
             (str_equal_text(&p_sess->ftp_cmd_str, "RMD") ||
              str_equal_text(&p_sess->ftp_cmd_str, "XRMD")))
    {
      handle_rmd(p_sess);
    }
    else if (tunable_write_enable &&
             (tunable_anon_other_write_enable || !p_sess->is_anonymous) &&
             str_equal_text(&p_sess->ftp_cmd_str, "DELE"))
    {
      handle_dele(p_sess);
    }
#ifndef _CMD_TYPE_XDS_	/* chg : for XDS 10/05/20 masuu */
    else if (str_equal_text(&p_sess->ftp_cmd_str, "REST"))
    {
      handle_rest(p_sess);
    }
#endif /* _CMD_TYPE_XDS_ */
    else if (tunable_write_enable &&
             (tunable_anon_other_write_enable || !p_sess->is_anonymous) &&
             str_equal_text(&p_sess->ftp_cmd_str, "RNFR"))
    {
      handle_rnfr(p_sess);
    }
    else if (tunable_write_enable &&
             (tunable_anon_other_write_enable || !p_sess->is_anonymous) &&
             str_equal_text(&p_sess->ftp_cmd_str, "RNTO"))
    {
      handle_rnto(p_sess);
    }
    else if (tunable_dirlist_enable &&
             str_equal_text(&p_sess->ftp_cmd_str, "NLST"))
    {
      handle_nlst(p_sess);
    }
#ifndef _CMD_TYPE_XDS_	/* chg : for XDS 10/05/20 masuu */
    else if (str_equal_text(&p_sess->ftp_cmd_str, "SIZE"))
    {
      handle_size(p_sess);
    }
#endif /* _CMD_TYPE_XDS_ */
    else if (!p_sess->is_anonymous &&
             str_equal_text(&p_sess->ftp_cmd_str, "SITE"))
    {
      handle_site(p_sess);
    }
    /* Note - the weird ABOR string is checking for an async ABOR arriving
     * without a SIGURG condition.
     */
    else if (str_equal_text(&p_sess->ftp_cmd_str, "ABOR") ||
             str_equal_text(&p_sess->ftp_cmd_str, "\377\364\377\362ABOR"))
    {
      vsf_cmdio_write(p_sess, FTP_ABOR_NOCONN, "No transfer to ABOR.");
    }
#ifndef _CMD_TYPE_XDS_	/* chg : for XDS 10/05/20 masuu */
    else if (tunable_write_enable &&
             (tunable_anon_other_write_enable || !p_sess->is_anonymous) &&
             str_equal_text(&p_sess->ftp_cmd_str, "APPE"))
    {
      handle_appe(p_sess);
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "MDTM"))
    {
      handle_mdtm(p_sess);
    }
    else if (tunable_port_enable &&
             str_equal_text(&p_sess->ftp_cmd_str, "EPRT"))
    {
      handle_eprt(p_sess);
    }
#endif /* _CMD_TYPE_XDS_ */

#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/28 masuu */
    else if (str_equal_text(&p_sess->ftp_cmd_str, "STRU"))
    {
      str_upper(&p_sess->ftp_arg_str);
      if (str_equal_text(&p_sess->ftp_arg_str, "F"))
      {
        vsf_cmdio_write(p_sess, FTP_STRUOK, "Command okay.");
      }
      else
      {
        vsf_cmdio_write(p_sess, FTP_BADSTRU, "Command not implemented for that parameter.");
      }
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "MODE"))
    {
      str_upper(&p_sess->ftp_arg_str);
      if (str_equal_text(&p_sess->ftp_arg_str, "S"))
      {
        vsf_cmdio_write(p_sess, FTP_MODEOK, "Command okay.");
      }
      else
      {
        vsf_cmdio_write(p_sess, FTP_BADMODE, "Command not implemented for that parameter.");
      }
    }
#else /* _RES_TYPE_XDS_ */
    else if (str_equal_text(&p_sess->ftp_cmd_str, "STRU"))
    {
      str_upper(&p_sess->ftp_arg_str);
      if (str_equal_text(&p_sess->ftp_arg_str, "F"))
      {
        vsf_cmdio_write(p_sess, FTP_STRUOK, "Structure set to F.");
      }
      else
      {
        vsf_cmdio_write(p_sess, FTP_BADSTRU, "Bad STRU command.");
      }
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "MODE"))
    {
      str_upper(&p_sess->ftp_arg_str);
      if (str_equal_text(&p_sess->ftp_arg_str, "S"))
      {
        vsf_cmdio_write(p_sess, FTP_MODEOK, "Mode set to S.");
      }
      else
      {
        vsf_cmdio_write(p_sess, FTP_BADMODE, "Bad MODE command.");
      }
    }
#endif /* _RES_TYPE_XDS_ */

#ifndef _CMD_TYPE_XDS_	/* chg : for XDS 10/05/20 masuu */
    else if (str_equal_text(&p_sess->ftp_cmd_str, "STOU"))
    {
      handle_stou(p_sess);
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "ALLO"))
    {
      vsf_cmdio_write(p_sess, FTP_ALLOOK, "ALLO command ignored.");
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "REIN"))
    {
      vsf_cmdio_write(p_sess, FTP_COMMANDNOTIMPL, "REIN not implemented.");
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "ACCT"))
    {
      vsf_cmdio_write(p_sess, FTP_COMMANDNOTIMPL, "ACCT not implemented.");
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "SMNT"))
    {
      vsf_cmdio_write(p_sess, FTP_COMMANDNOTIMPL, "SMNT not implemented.");
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "FEAT"))
    {
      handle_feat(p_sess);
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "OPTS"))
    {
      vsf_cmdio_write(p_sess, FTP_BADOPTS, "Option not understood.");
    }
#endif /* _CMD_TYPE_XDS_ */
    else if (str_equal_text(&p_sess->ftp_cmd_str, "STAT") &&
             str_isempty(&p_sess->ftp_arg_str))
    {
      handle_stat(p_sess);
    }
    else if (tunable_dirlist_enable &&
             str_equal_text(&p_sess->ftp_cmd_str, "STAT"))
    {
      handle_stat_file(p_sess);
    }
#ifdef _CMD_TYPE_XDS_	/* chg : for XDS 10/05/20 masuu */
    else if (str_equal_text(&p_sess->ftp_cmd_str, "PASV") ||
             str_equal_text(&p_sess->ftp_cmd_str, "PORT") ||
             str_equal_text(&p_sess->ftp_cmd_str, "STOR") ||
             str_equal_text(&p_sess->ftp_cmd_str, "MKD") ||
             str_equal_text(&p_sess->ftp_cmd_str, "RMD") ||
             str_equal_text(&p_sess->ftp_cmd_str, "DELE") ||
             str_equal_text(&p_sess->ftp_cmd_str, "RNFR") ||
             str_equal_text(&p_sess->ftp_cmd_str, "RNTO") ||
             str_equal_text(&p_sess->ftp_cmd_str, "SITE") ||
             str_equal_text(&p_sess->ftp_cmd_str, "RETR") ||
             str_equal_text(&p_sess->ftp_cmd_str, "LIST") ||
             str_equal_text(&p_sess->ftp_cmd_str, "NLST") ||
             str_equal_text(&p_sess->ftp_cmd_str, "STAT"))
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/28 masuu */
    {
      vsf_cmdio_write(p_sess, FTP_BADCMD, "Syntax error, command unrecognized.");
    }
    else
    {
      vsf_cmdio_write(p_sess, FTP_BADCMD, "Syntax error, command unrecognized.");
    }
#else /* _RES_TYPE_XDS_ */
    {
      vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    }
    else
    {
      vsf_cmdio_write(p_sess, FTP_BADCMD, "Unknown command.");
    }
#endif /* _RES_TYPE_XDS_ */
#else /* _CMD_TYPE_XDS_ */
    else if (tunable_ssl_enable && str_equal_text(&p_sess->ftp_cmd_str, "PBSZ"))
    {
      handle_pbsz(p_sess);
    }
    else if (tunable_ssl_enable && str_equal_text(&p_sess->ftp_cmd_str, "PROT"))
    {
      handle_prot(p_sess);
    }
    else if (str_equal_text(&p_sess->ftp_cmd_str, "PASV") ||
             str_equal_text(&p_sess->ftp_cmd_str, "PORT") ||
             str_equal_text(&p_sess->ftp_cmd_str, "STOR") ||
             str_equal_text(&p_sess->ftp_cmd_str, "MKD") ||
             str_equal_text(&p_sess->ftp_cmd_str, "XMKD") ||
             str_equal_text(&p_sess->ftp_cmd_str, "RMD") ||
             str_equal_text(&p_sess->ftp_cmd_str, "XRMD") ||
             str_equal_text(&p_sess->ftp_cmd_str, "DELE") ||
             str_equal_text(&p_sess->ftp_cmd_str, "RNFR") ||
             str_equal_text(&p_sess->ftp_cmd_str, "RNTO") ||
             str_equal_text(&p_sess->ftp_cmd_str, "SITE") ||
             str_equal_text(&p_sess->ftp_cmd_str, "APPE") ||
             str_equal_text(&p_sess->ftp_cmd_str, "EPSV") ||
             str_equal_text(&p_sess->ftp_cmd_str, "EPRT") ||
             str_equal_text(&p_sess->ftp_cmd_str, "RETR") ||
             str_equal_text(&p_sess->ftp_cmd_str, "LIST") ||
             str_equal_text(&p_sess->ftp_cmd_str, "NLST") ||
             str_equal_text(&p_sess->ftp_cmd_str, "STOU") ||
             str_equal_text(&p_sess->ftp_cmd_str, "ALLO") ||
             str_equal_text(&p_sess->ftp_cmd_str, "REIN") ||
             str_equal_text(&p_sess->ftp_cmd_str, "ACCT") ||
             str_equal_text(&p_sess->ftp_cmd_str, "SMNT") ||
             str_equal_text(&p_sess->ftp_cmd_str, "FEAT") ||
             str_equal_text(&p_sess->ftp_cmd_str, "OPTS") ||
             str_equal_text(&p_sess->ftp_cmd_str, "STAT") ||
             str_equal_text(&p_sess->ftp_cmd_str, "PBSZ") ||
             str_equal_text(&p_sess->ftp_cmd_str, "PROT"))
    {
      vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    }
    else
    {
      vsf_cmdio_write(p_sess, FTP_BADCMD, "Unknown command.");
    }
#endif /* _CMD_TYPE_XDS_ */
    if (vsf_log_entry_pending(p_sess))
    {
      vsf_log_do_log(p_sess, 0);
    }
  }
}

static void
handle_pwd(struct vsf_session* p_sess)
{
  static struct mystr s_cwd_buf_mangle_str;
  static struct mystr s_pwd_res_str;
  str_getcwd(&s_cwd_buf_mangle_str);
  /* Double up any double-quotes in the pathname! */
  str_replace_text(&s_cwd_buf_mangle_str, "\"", "\"\"");
  /* Enclose pathname in quotes */
  str_alloc_text(&s_pwd_res_str, "\"");
  str_append_str(&s_pwd_res_str, &s_cwd_buf_mangle_str);
  str_append_text(&s_pwd_res_str, "\"");
  vsf_cmdio_write_str(p_sess, FTP_PWDOK, &s_pwd_res_str);
}

static void
handle_cwd(struct vsf_session* p_sess)
{
  int retval;
  static struct mystr s_tmp_str;				// add : 10/05/31 masuu

#ifdef _CMD_TYPE_XDS_	/* chg : for XDS 10/05/20 masuu */
  /* add : correspond to no argument 11/03/02 masuu */
  if( str_isempty(&p_sess->ftp_arg_str) ) {
    str_alloc_text(&s_tmp_str, "/");
    if ( !vsf_access_check_file(&s_tmp_str) )
    {
      vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
      return;
    }
    retval = str_chdir(&s_tmp_str);
  }
  else {
    str_copy(&s_tmp_str, &p_sess->ftp_arg_str);	// add : 10/05/31 masuu
    resolve_tilde(&p_sess->ftp_arg_str, p_sess);
    if (!vsf_access_check_file(&p_sess->ftp_arg_str))
    {
      vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
      return;
    }
    retval = str_chdir(&p_sess->ftp_arg_str);
  }
#else /* _CMD_TYPE_XDS_ */
  str_copy(&s_tmp_str, &p_sess->ftp_arg_str);	// add : 10/05/31 masuu
  resolve_tilde(&p_sess->ftp_arg_str, p_sess);
  if (!vsf_access_check_file(&p_sess->ftp_arg_str))
  {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }
  retval = str_chdir(&p_sess->ftp_arg_str);
#endif /* _CMD_TYPE_XDS_ */

#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/31 masuu */
  if (retval == 0)
  {
    /* Handle any messages */
    vsf_banner_dir_changed(p_sess, FTP_CWDOK);
    {
      static struct mystr s_cwd_res;
      /* Double up double quotes */
      str_replace_text(&s_tmp_str, "\"", "\"\"");
      /* Build result string */
      str_alloc_text(&s_cwd_res, "Changed to ");
      str_append_str(&s_cwd_res, &s_tmp_str);
      vsf_cmdio_write_str(p_sess, FTP_CWDOK, &s_cwd_res);
    }
  }
  else
  {
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
  }
#else /* _RES_TYPE_XDS_ */
  if (retval == 0)
  {
    /* Handle any messages */
    vsf_banner_dir_changed(p_sess, FTP_CWDOK);
    vsf_cmdio_write(p_sess, FTP_CWDOK, "Directory successfully changed.");
  }
  else
  {
    vsf_cmdio_write(p_sess, FTP_FILEFAIL, "Failed to change directory.");
  }
#endif /* _RES_TYPE_XDS_ */
}

static void
handle_cdup(struct vsf_session* p_sess)
{
  str_alloc_text(&p_sess->ftp_arg_str, "..");
  handle_cwd(p_sess);
}

static int
port_active(struct vsf_session* p_sess)
{
  int ret = 0;
  if (p_sess->p_port_sockaddr != 0)
  {
    ret = 1;
    if (pasv_active(p_sess))
    {
      bug("port and pasv both active");
    }
  }
  return ret;
}

static int
pasv_active(struct vsf_session* p_sess)
{
  int ret = 0;
  if (p_sess->pasv_listen_fd != -1)
  {
    ret = 1;
    if (port_active(p_sess))
    {
      bug("pasv and port both active");
    }
  }
  return ret;
}

static void
port_cleanup(struct vsf_session* p_sess)
{
  vsf_sysutil_sockaddr_clear(&p_sess->p_port_sockaddr);
}

static void
pasv_cleanup(struct vsf_session* p_sess)
{
  if (p_sess->pasv_listen_fd != -1)
  {
    vsf_sysutil_close(p_sess->pasv_listen_fd);
    p_sess->pasv_listen_fd = -1;
  }
}

static void
handle_pasv(struct vsf_session* p_sess, int is_epsv)
{
  static struct mystr s_pasv_res_str;
  static struct vsf_sysutil_sockaddr* s_p_sockaddr;
  int bind_retries = 10;
  unsigned short the_port = 0;
  int is_ipv6 = vsf_sysutil_sockaddr_is_ipv6(p_sess->p_local_addr);
  if (is_epsv && !str_isempty(&p_sess->ftp_arg_str))
  {
    int argval;
    str_upper(&p_sess->ftp_arg_str);
    if (str_equal_text(&p_sess->ftp_arg_str, "ALL"))
    {
      p_sess->epsv_all = 1;
#ifndef _RES_TYPE_XDS_	/* chg : for XDS 10/05/27 masuu */
      vsf_cmdio_write(p_sess, FTP_EPSVALLOK, "EPSV ALL ok.");
#endif /* _RES_TYPE_XDS_ */
      return;
    }
    argval = vsf_sysutil_atoi(str_getbuf(&p_sess->ftp_arg_str));
    if (argval < 1 || argval > 2 || (!is_ipv6 && argval == 2))
    {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/27 masuu */
      vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
#else /* _RES_TYPE_XDS_ */
      vsf_cmdio_write(p_sess, FTP_EPSVBAD, "Bad network protocol.");
#endif /* _RES_TYPE_XDS_ */
      return;
    }
  }
  pasv_cleanup(p_sess);
  port_cleanup(p_sess);
  if (is_ipv6)
  {
    p_sess->pasv_listen_fd = vsf_sysutil_get_ipv6_sock();
  }
  else
  {
    p_sess->pasv_listen_fd = vsf_sysutil_get_ipv4_sock();
  }
  vsf_sysutil_activate_reuseaddr(p_sess->pasv_listen_fd);
  while (--bind_retries)
  {
    int retval;
    double scaled_port;
    /* IPPORT_RESERVED */
    unsigned short min_port = 1024;
    unsigned short max_port = 65535;
    if (tunable_pasv_min_port > min_port && tunable_pasv_min_port <= max_port)
    {
      min_port = tunable_pasv_min_port;
    }
    if (tunable_pasv_max_port >= min_port && tunable_pasv_max_port < max_port)
    {
      max_port = tunable_pasv_max_port;
    }
    the_port = vsf_sysutil_get_random_byte();
    the_port <<= 8;
    the_port |= vsf_sysutil_get_random_byte();
    scaled_port = (double) min_port;
    scaled_port += ((double) the_port / (double) 65536) *
                   ((double) max_port - min_port + 1);
    the_port = (unsigned short) scaled_port;
    vsf_sysutil_sockaddr_clone(&s_p_sockaddr, p_sess->p_local_addr);
    vsf_sysutil_sockaddr_set_port(s_p_sockaddr, the_port);
    retval = vsf_sysutil_bind(p_sess->pasv_listen_fd, s_p_sockaddr);
    if (!vsf_sysutil_retval_is_error(retval))
    {
      break;
    }
    if ( (vsf_sysutil_get_error() == kVSFSysUtilErrADDRINUSE) ||
         (vsf_sysutil_get_error() == kVSFSysUtilErrEACCES) )
    {
      continue;
    }
    die("vsf_sysutil_bind");
  }
  if (!bind_retries)
  {
    die("vsf_sysutil_bind");
  }
  vsf_sysutil_listen(p_sess->pasv_listen_fd, 1);
  if (is_epsv)
  {
    str_alloc_text(&s_pasv_res_str, "Entering Extended Passive Mode (|||");
    str_append_ulong(&s_pasv_res_str, (unsigned long) the_port);
    str_append_text(&s_pasv_res_str, "|)");
    vsf_cmdio_write_str(p_sess, FTP_EPSVOK, &s_pasv_res_str);
    return;
  }
  if (tunable_pasv_address != 0)
  {
    /* Report passive address as specified in configuration */
    if (vsf_sysutil_inet_aton(tunable_pasv_address, s_p_sockaddr) == 0)
    {
      die("invalid pasv_address");
    }
  }
  str_alloc_text(&s_pasv_res_str, "Entering Passive Mode (");
  if (!is_ipv6)
  {
    str_append_text(&s_pasv_res_str, vsf_sysutil_inet_ntop(s_p_sockaddr));
  }
  else
  {
    const void* p_v4addr = vsf_sysutil_sockaddr_ipv6_v4(s_p_sockaddr);
    if (p_v4addr)
    {
      str_append_text(&s_pasv_res_str, vsf_sysutil_inet_ntoa(p_v4addr));
    }
    else
    {
      str_append_text(&s_pasv_res_str, "0,0,0,0");
    }
  }
  str_replace_char(&s_pasv_res_str, '.', ',');
  str_append_text(&s_pasv_res_str, ",");
  str_append_ulong(&s_pasv_res_str, the_port >> 8);
  str_append_text(&s_pasv_res_str, ",");
  str_append_ulong(&s_pasv_res_str, the_port & 255);
  str_append_text(&s_pasv_res_str, ")");
  vsf_cmdio_write_str(p_sess, FTP_PASVOK, &s_pasv_res_str);
}

static void
handle_retr(struct vsf_session* p_sess)
{
  static struct mystr s_mark_str, s_topdir_str, s_buf_str;	// add : 11/05/02 masuu
  static struct vsf_sysutil_statbuf* s_p_statbuf;
  struct vsf_transfer_ret trans_ret;
  struct timeval starttime, endtime;	// add : 10/06/01 masuu
  int remote_fd;
  int opened_file;
  int is_ascii = 0;
  filesize_t offset = p_sess->restart_pos;
  p_sess->restart_pos = 0;
  if (!data_transfer_checks_ok(p_sess))
  {
    return;
  }
  if (p_sess->is_ascii && offset != 0)
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/01 masuu */
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_FILEFAIL, "No support for resume of ASCII transfer.");
#endif /* _RES_TYPE_XDS_ */
    return;
  }
  resolve_tilde(&p_sess->ftp_arg_str, p_sess);
  vsf_log_start_entry(p_sess, kVSFLogEntryDownload);
  str_copy(&p_sess->log_str, &p_sess->ftp_arg_str);
  prepend_path_to_filename(&p_sess->log_str);
  if (!vsf_access_check_file(&p_sess->ftp_arg_str))
  {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }
  opened_file = str_open(&p_sess->ftp_arg_str, kVSFSysStrOpenReadOnly);
  if (vsf_sysutil_retval_is_error(opened_file))
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/01 masuu */
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_FILEFAIL, "Failed to open file.");
#endif /* _RES_TYPE_XDS_ */
    return;
  }
  /* Lock file if required */
  if (tunable_lock_upload_files)
  {
    vsf_sysutil_lock_file_read(opened_file);
  }
  vsf_sysutil_fstat(opened_file, &s_p_statbuf);
  /* No games please */
  if (!vsf_sysutil_statbuf_is_regfile(s_p_statbuf))
  {
    /* Note - pretend open failed */
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/01 masuu */
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_FILEFAIL, "Failed to open file.");
#endif /* _RES_TYPE_XDS_ */
    goto file_close_out;
  }
  /* Now deactive O_NONBLOCK, otherwise we have a problem on DMAPI filesystems
   * such as XFS DMAPI.
   */
  vsf_sysutil_deactivate_noblock(opened_file);
  /* Optionally, we'll be paranoid and only serve publicly readable stuff */
  if (p_sess->is_anonymous && tunable_anon_world_readable_only &&
      !vsf_sysutil_statbuf_is_readable_other(s_p_statbuf))
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/01 masuu */
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_FILEFAIL, "Failed to open file.");
#endif /* _RES_TYPE_XDS_ */
    goto file_close_out;
  }
  /* Set the download offset (from REST) if any */
  if (offset != 0)
  {
    vsf_sysutil_lseek_to(opened_file, offset);
  }
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/01 masuu */
  str_alloc_text(&s_mark_str, "File status okay; about to open data connection.");
#else /* _RES_TYPE_XDS_ */
  str_alloc_text(&s_mark_str, "Opening ");
  if (tunable_ascii_download_enable && p_sess->is_ascii)
  {
    str_append_text(&s_mark_str, "ASCII");
    is_ascii = 1;
  }
  else
  {
    str_append_text(&s_mark_str, "BINARY");
  }
  str_append_text(&s_mark_str, " mode data connection for ");
  str_append_str(&s_mark_str, &p_sess->ftp_arg_str);
  str_append_text(&s_mark_str, " (");
  str_append_filesize_t(&s_mark_str,
                        vsf_sysutil_statbuf_get_size(s_p_statbuf));
  str_append_text(&s_mark_str, " bytes).");
#endif /* _RES_TYPE_XDS_ */
  remote_fd = get_remote_transfer_fd(p_sess, str_getbuf(&s_mark_str));
  if (vsf_sysutil_retval_is_error(remote_fd))
  {
    goto port_pasv_cleanup_out;
  }

  /* add : PD area access check & save 11/05/02 masuu */
  if( str_equal_text(&p_sess->user_str, "root") || str_equal_text(&p_sess->user_str, "loggetter") ) {
  }
  else {
    str_copy(&s_buf_str, &p_sess->ftp_arg_str);
    prepend_path_to_filename(&s_buf_str);
    str_split_char(&s_buf_str, &s_topdir_str, '/');
    str_split_char(&s_topdir_str, &s_buf_str, '/');
    if( str_equal_text(&s_topdir_str, "DISC") ) {
      p_sess->pd_access = 1;
    }
  }

  /* file transfer */
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/01 masuu */
  gettimeofday(&starttime, NULL);
  /* chg : vsf_ftpdataio_transfer_file -> vsf_ftpdataio_transfer_file2 10/09/13 masuu */
  trans_ret = vsf_ftpdataio_transfer_file2(p_sess, remote_fd, opened_file, 0, is_ascii);
  gettimeofday(&endtime, NULL);
#else /* _RES_TYPE_XDS_ */
  trans_ret = vsf_ftpdataio_transfer_file(p_sess, remote_fd,
                                          opened_file, 0, is_ascii);
#endif /* _RES_TYPE_XDS_ */

  vsf_ftpdataio_dispose_transfer_fd(p_sess);
  p_sess->transfer_size = trans_ret.transferred;
  /* Log _after_ the blocking dispose call, so we get transfer times right */
  if (trans_ret.retval == 0)
  {
    vsf_log_do_log(p_sess, 1);
  }

  /* Emit status message _after_ blocking dispose call to avoid buggy FTP
   * clients truncating the transfer.
   */
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/01 masuu */
  if (trans_ret.retval == -1)
  {
    vsf_cmdio_write(p_sess, FTP_BADSENDFILE, "Requested action aborted. Local error in processing.");
  }
  else if (trans_ret.retval == -2)
  {
    vsf_cmdio_write(p_sess, FTP_BADWRITESEND, "Error during write to data connection.");
  }
  else
  {
    transfer_speed_write(p_sess, &starttime, &endtime, vsf_sysutil_statbuf_get_size(s_p_statbuf));
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Closing data connection. Requested file action successful.");
  }
#else /* _RES_TYPE_XDS_ */
  if (trans_ret.retval == -1)
  {
    vsf_cmdio_write(p_sess, FTP_BADSENDFILE, "Failure reading local file.");
  }
  else if (trans_ret.retval == -2)
  {
    vsf_cmdio_write(p_sess, FTP_BADSENDNET, "Failure writing network stream.");
  }
  else
  {
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "File send OK.");
  }
#endif /* _RES_TYPE_XDS_ */

  check_abor(p_sess);
port_pasv_cleanup_out:
  port_cleanup(p_sess);
  pasv_cleanup(p_sess);
file_close_out:
  vsf_sysutil_close(opened_file);
}

static void
handle_list(struct vsf_session* p_sess)
{
  handle_dir_common(p_sess, 1, 0);
}

static void
handle_dir_common(struct vsf_session* p_sess, int full_details, int stat_cmd)
{
  static struct mystr s_option_str;
  static struct mystr s_filter_str;
  static struct mystr s_dir_name_str;
  static struct mystr s_path_str;				// add : 10/06/17 masuu
  static struct mystr s_fname_str, s_ext_str;	// add : 10/06/17 masuu
  static struct vsf_sysutil_statbuf* s_p_dirstat;
  int dir_allow_read = 1;
  struct vsf_sysutil_dir* p_dir = 0;
  struct vsf_sysutil_statbuf* s_p_statbuf = 0;	// add : 10/06/17 masuu
  struct vsf_confs_xdc_stat_param ioc_param;	// add : 10/06/18 masuu
  int retval = 0;
  int use_control = 0;
  int mxf_file = 0;								// add : 10/06/17 masuu
  int opened_file;								// add : 10/06/17 masuu
  str_empty(&s_option_str);
  str_empty(&s_filter_str);
  /* By default open the current directory */
  str_alloc_text(&s_dir_name_str, ".");
  if (!stat_cmd && !data_transfer_checks_ok(p_sess))
  {
    return;
  }
  /* Do we have an option? Going to be strict here - the option must come
   * first. e.g. "ls -a .." fine, "ls .. -a" not fine
   */
  if (!str_isempty(&p_sess->ftp_arg_str) &&
      str_get_char_at(&p_sess->ftp_arg_str, 0) == '-')
  {
    /* Chop off the '-' */
    str_mid_to_end(&p_sess->ftp_arg_str, &s_option_str, 1);
    /* A space will separate options from filter (if any) */
    str_split_char(&s_option_str, &s_filter_str, ' ');
  }
  else
  {
    /* The argument, if any, is just a filter */
    str_copy(&s_filter_str, &p_sess->ftp_arg_str);
  }
  if (!str_isempty(&s_filter_str))
  {
    resolve_tilde(&s_filter_str, p_sess);
    if (!vsf_access_check_file(&s_filter_str))
    {
      vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
      return;
    }
    /* add : stat check 10/06/17 masuu */
    str_copy(&s_path_str, &s_filter_str);
    prepend_path_to_filename(&s_path_str);
    vsf_sysutil_stat(str_getbuf(&s_path_str), &s_p_statbuf);
    /* First check - is it an outright directory, as in "ls /pub" */
    p_dir = str_opendir(&s_filter_str);
    if (p_dir != 0)
    {
      /* Listing a directory! */
      str_copy(&s_dir_name_str, &s_filter_str);
      str_free(&s_filter_str);
    }
    else
    {
      /* add : fail reject 10/06/17 masuu */
      if (!vsf_sysutil_statbuf_is_regfile(s_p_statbuf) && stat_cmd) {
        vsf_cmdio_write(p_sess, FTP_FILEFAIL, "File not found, no access. code=3001");
        return;
      }
      struct str_locate_result locate_result =
        str_locate_char(&s_filter_str, '/');
      if (locate_result.found)
      {
        /* Includes a path! Reverse scan for / in the arg, to get the
         * base directory and filter (if any)
         */
        str_copy(&s_dir_name_str, &s_filter_str);
        str_split_char_reverse(&s_dir_name_str, &s_filter_str, '/');
        /* If we have e.g. "ls /.message", we just ripped off the leading
         * slash because it is the only one!
         */
        if (str_isempty(&s_dir_name_str))
        {
          str_alloc_text(&s_dir_name_str, "/");
        }
      }
    }
  }
  if (p_dir == 0)
  {
    /* NOTE - failure check done below, it's not forgotten */
    p_dir = str_opendir(&s_dir_name_str);
  }

  /* Fine, do it */
  if (stat_cmd)
  {
    use_control = 1;
    str_append_char(&s_option_str, 'a');
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/16 masuu */
    /* add : judge extension 10/06/17 masuu */
    if (!str_isempty(&s_filter_str))
    {
      str_copy(&s_fname_str, &s_filter_str);
      str_split_char_reverse(&s_fname_str, &s_ext_str, '.');
      str_upper(&s_ext_str);	/* add : 11/05/23 masuu */
      if (str_equal_text(&s_ext_str, "MXF"))
      {
        mxf_file = 1;
        /* chg : for select file 11/02/07 masuu */
        opened_file = str_dummy_open(0);
        if (vsf_sysutil_retval_is_error(opened_file)) {
          vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
          vsf_sysutil_close(opened_file);
          goto dir_close_out;
        }
        vsf_sysutil_strcpy(ioc_param.path, str_getbuf(&s_path_str), str_getlen(&s_path_str) + 1);
        if(vsf_sysutil_ioctl(opened_file, VSF_CONFS_IOCCMD_STAT, &ioc_param) < 0) {
//          err_no = errno;
          vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
          vsf_sysutil_close(opened_file);
          goto dir_close_out;
        }
      }
    }
    vsf_cmdio_write_hyphen(p_sess, FTP_STATFILE_OK, "File status");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write_hyphen(p_sess, FTP_STATFILE_OK, "Status follows:");
#endif /* _RES_TYPE_XDS_ */
  }
  else
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/31 masuu */
    int remote_fd = get_remote_transfer_fd(
      p_sess, "File status okay; about to open data connection.");
#else /* _RES_TYPE_XDS_ */
    int remote_fd = get_remote_transfer_fd(
      p_sess, "Here comes the directory listing.");
#endif /* _RES_TYPE_XDS_ */
    if (vsf_sysutil_retval_is_error(remote_fd))
    {
      goto dir_close_out;
    }
  }
  if (p_sess->is_anonymous && p_dir && tunable_anon_world_readable_only)
  {
    vsf_sysutil_dir_stat(p_dir, &s_p_dirstat);
    if (!vsf_sysutil_statbuf_is_readable_other(s_p_dirstat))
    {
      dir_allow_read = 0;
    }
  }
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/16 masuu */
#if(1)
  if (!str_isempty(&s_filter_str) && stat_cmd)
  {
    vsf_cmdio_write_raw(p_sess, "    ");
    vsf_cmdio_write_raw(p_sess, str_strdup(&s_filter_str));
    vsf_cmdio_write_raw(p_sess, "\r\n");
    if (mxf_file)
    {
      xdc_stat_write_raw(p_sess, ioc_param);
    }
  }
  else if (p_dir != 0 && dir_allow_read)
  {
    retval = vsf_ftpdataio_transfer_dir(p_sess, use_control, p_dir,
                                        &s_dir_name_str, &s_option_str,
                                        &s_filter_str, full_details);
  }
#else
  if (p_dir != 0 && dir_allow_read)
  {
    retval = vsf_ftpdataio_transfer_dir(p_sess, use_control, p_dir,
                                        &s_dir_name_str, &s_option_str,
                                        &s_filter_str, full_details);
  }
#endif
#else /* _RES_TYPE_XDS_ */
  if (p_dir != 0 && dir_allow_read)
  {
    retval = vsf_ftpdataio_transfer_dir(p_sess, use_control, p_dir,
                                        &s_dir_name_str, &s_option_str,
                                        &s_filter_str, full_details);
  }
#endif /* _RES_TYPE_XDS_ */
  if (!stat_cmd)
  {
    vsf_ftpdataio_dispose_transfer_fd(p_sess);
  }
  if (stat_cmd)
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/16 masuu */
    vsf_cmdio_write(p_sess, FTP_STATFILE_OK, "End of status.");
    if (str_equal_text(&s_ext_str, "MXF"))
    {
      vsf_sysutil_close(opened_file);
    }
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_STATFILE_OK, "End of status");
#endif /* _RES_TYPE_XDS_ */
  }

#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/31 masuu */
  else if (p_dir == 0 || !dir_allow_read)
  {
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Closing data connection. Requested file action successful.");
  }
  else if (retval == 0)
  {
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Closing data connection. Requested file action successful.");
  }
  else
  {
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
  }
#else /* _RES_TYPE_XDS_ */
  else if (p_dir == 0 || !dir_allow_read)
  {
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK,
                    "Transfer done (but failed to open directory).");
  }
  else if (retval == 0)
  {
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Directory send OK.");
  }
  else
  {
    vsf_cmdio_write(p_sess, FTP_BADSENDNET, "Failure writing network stream.");
  }
#endif /* _RES_TYPE_XDS_ */

  check_abor(p_sess);
dir_close_out:
  if (p_dir)
  {
    vsf_sysutil_closedir(p_dir);
  }
  if (!stat_cmd)
  {
    port_cleanup(p_sess);
    pasv_cleanup(p_sess);
  }
}

static void
handle_type(struct vsf_session* p_sess)
{
  str_upper(&p_sess->ftp_arg_str);

#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/28 masuu */
#if(1)	/* chg : CQ3429 11/01/06 masuu */
  if (str_equal_text(&p_sess->ftp_arg_str, "I") ||
      str_equal_text(&p_sess->ftp_arg_str, "L8") ||
      str_equal_text(&p_sess->ftp_arg_str, "L 8"))
  {
    p_sess->is_ascii = 0;
    vsf_cmdio_write(p_sess, FTP_TYPEOK, "Command okay.");
  }
  /* chg : CQ3429 11/01/11 masuu */
  else if (str_equal_text(&p_sess->ftp_arg_str, "A") ||
           str_equal_text(&p_sess->ftp_arg_str, "A N") ||
           str_equal_text(&p_sess->ftp_arg_str, "A T") ||
           str_equal_text(&p_sess->ftp_arg_str, "A C"))
  {
    p_sess->is_ascii = 0;
    vsf_cmdio_write(p_sess, FTP_TYPEOK, "Command okay.");
  }
  else
  {
    /* chg : CQ3429 10/12/23 masuu */
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
  }
#else
  if (str_equal_text(&p_sess->ftp_arg_str, "I") ||
      str_equal_text(&p_sess->ftp_arg_str, "L8") ||
      str_equal_text(&p_sess->ftp_arg_str, "L 8"))
  {
    p_sess->is_ascii = 0;
    vsf_cmdio_write(p_sess, FTP_TYPEOK, "Command okay.");
  }
  else if (str_equal_text(&p_sess->ftp_arg_str, "A") ||
           str_equal_text(&p_sess->ftp_arg_str, "A N"))
  {
    p_sess->is_ascii = 1;
    vsf_cmdio_write(p_sess, FTP_TYPEOK, "Command okay.");
  }
  else
  {
    /* chg : CQ3429 10/12/23 masuu */
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
  }
#endif
#else /* _RES_TYPE_XDS_ */
  if (str_equal_text(&p_sess->ftp_arg_str, "I") ||
      str_equal_text(&p_sess->ftp_arg_str, "L8") ||
      str_equal_text(&p_sess->ftp_arg_str, "L 8"))
  {
    p_sess->is_ascii = 0;
    vsf_cmdio_write(p_sess, FTP_TYPEOK, "Switching to Binary mode.");
  }
  else if (str_equal_text(&p_sess->ftp_arg_str, "A") ||
           str_equal_text(&p_sess->ftp_arg_str, "A N"))
  {
    p_sess->is_ascii = 1;
    vsf_cmdio_write(p_sess, FTP_TYPEOK, "Switching to ASCII mode.");
  }
  else
  {
    vsf_cmdio_write(p_sess, FTP_BADCMD, "Unrecognised TYPE command.");
  }
#endif /* _RES_TYPE_XDS_ */

}

static void
handle_port(struct vsf_session* p_sess)
{
  unsigned short the_port;
  unsigned char vals[6];
  const unsigned char* p_raw;
  pasv_cleanup(p_sess);
  port_cleanup(p_sess);
  p_raw = vsf_sysutil_parse_uchar_string_sep(&p_sess->ftp_arg_str, ',', vals,
                                             sizeof(vals));
  if (p_raw == 0)
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/27 masuu */
    vsf_cmdio_write(p_sess, FTP_BADCMD, "Syntax error, command unrecognized.");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_BADCMD, "Illegal PORT command.");
#endif /* _RES_TYPE_XDS_ */
    return;
  }
  the_port = vals[4] << 8;
  the_port |= vals[5];
  vsf_sysutil_sockaddr_clone(&p_sess->p_port_sockaddr, p_sess->p_local_addr);
  vsf_sysutil_sockaddr_set_ipv4addr(p_sess->p_port_sockaddr, vals);
  vsf_sysutil_sockaddr_set_port(p_sess->p_port_sockaddr, the_port);
  /* SECURITY:
   * 1) Reject requests not connecting to the control socket IP
   * 2) Reject connects to privileged ports
   */
  if (!tunable_port_promiscuous)
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/27 masuu */
    if (!vsf_sysutil_sockaddr_addr_equal(p_sess->p_remote_addr,
                                         p_sess->p_port_sockaddr) ||
        vsf_sysutil_is_port_reserved(the_port))
    {
      vsf_cmdio_write(p_sess, FTP_BADCMD, "Syntax error, command unrecognized.");
      port_cleanup(p_sess);
      return;
    }
  }
  vsf_cmdio_write(p_sess, FTP_PORTOK, "Command okay.");
#else /* _RES_TYPE_XDS_ */
    if (!vsf_sysutil_sockaddr_addr_equal(p_sess->p_remote_addr,
                                         p_sess->p_port_sockaddr) ||
        vsf_sysutil_is_port_reserved(the_port))
    {
      vsf_cmdio_write(p_sess, FTP_BADCMD, "Illegal PORT command.");
      port_cleanup(p_sess);
      return;
    }
  }
  vsf_cmdio_write(p_sess, FTP_PORTOK,
                  "PORT command successful. Consider using PASV.");
#endif /* _RES_TYPE_XDS_ */
}

static void
handle_stor(struct vsf_session* p_sess)
{
  handle_upload_common(p_sess, 0, 0);
}

static void
handle_upload_common(struct vsf_session* p_sess, int is_append, int is_unique)
{
  static struct vsf_sysutil_statbuf* s_p_statbuf;
  static struct mystr s_filename;
  static struct mystr s_topdir_str, s_buf_str;	// add : 11/05/02 masuu
  struct mystr* p_filename;
  struct vsf_transfer_ret trans_ret;
  struct timeval starttime, endtime;	// add : 10/06/03 masuu
  int new_file_fd;
  int remote_fd;
  int close_return;		// add : fot CQ1323 10/06/03 masuu
  int truncit = 0;
  filesize_t offset = p_sess->restart_pos;

/* add : for ioctl 10/04/26 masuu */
  unsigned int xds_mode = p_sess->xds_mode;
  p_sess->xds_mode = VSFTP_XDS_MODE_NONE;

  p_sess->restart_pos = 0;
  if (!data_transfer_checks_ok(p_sess))
  {
    return;
  }
  resolve_tilde(&p_sess->ftp_arg_str, p_sess);
  p_filename = &p_sess->ftp_arg_str;
  if (is_unique)
  {
    get_unique_filename(&s_filename, p_filename);
    p_filename = &s_filename;
  }
  vsf_log_start_entry(p_sess, kVSFLogEntryUpload);
  str_copy(&p_sess->log_str, &p_sess->ftp_arg_str);
  prepend_path_to_filename(&p_sess->log_str);
  if (!vsf_access_check_file(p_filename))
  {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }
  /* NOTE - actual file permissions will be governed by the tunable umask */
  /* XXX - do we care about race between create and chown() of anonymous
   * upload?
   */
  if (is_unique || (p_sess->is_anonymous && !tunable_anon_other_write_enable))
  {
    new_file_fd = str_create(p_filename);
  }
  else
  {
    /* For non-anonymous, allow open() to overwrite or append existing files */
    if (!is_append && offset == 0)
    {
      if (tunable_lock_upload_files)
      {
        new_file_fd = str_create_append(p_filename);
        truncit = 1;
      }
      else
      {
        new_file_fd = str_create_overwrite(p_filename);
      }
    }
    else
    {
      new_file_fd = str_create_append(p_filename);
    }
  }
  if (vsf_sysutil_retval_is_error(new_file_fd))
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/03 masuu */
    int saved_errno = errno;
    if( saved_errno == ENOSPC ) {
      vsf_cmdio_write(p_sess, FTP_DISKFULL, "Requested file action not taken. Insufficient space.");
    }
    else {
      /* chg : SYSD00002705 FTP_UPLOADFAIL -> FTP_BADSENDFILE 10/11/12 masuu */
      vsf_cmdio_write(p_sess, FTP_BADSENDFILE, "Requested action aborted. Local error in processing.");
    }
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_UPLOADFAIL, "Could not create file.");
#endif /* _RES_TYPE_XDS_ */
    return;
  }

/* add : ioctl 10/04/26 masuu */
  if( xds_mode == VSFTP_XDS_MODE_UMMD ) {
    vsf_sysutil_ioctl(new_file_fd, VSF_CONFS_IOCCMD_UMMD, NULL);
  }

  vsf_sysutil_fstat(new_file_fd, &s_p_statbuf);
  if (vsf_sysutil_statbuf_is_regfile(s_p_statbuf))
  {
    /* Now deactive O_NONBLOCK, otherwise we have a problem on DMAPI filesystems
     * such as XFS DMAPI.
     */
    vsf_sysutil_deactivate_noblock(new_file_fd);
  }
  /* Are we required to chown() this file for security? */
  if (p_sess->is_anonymous && tunable_chown_uploads)
  {
    vsf_sysutil_fchmod(new_file_fd, 0600);
    if (tunable_one_process_model)
    {
      vsf_one_process_chown_upload(p_sess, new_file_fd);
    }
    else
    {
      vsf_two_process_chown_upload(p_sess, new_file_fd);
    }
  }
  /* Are we required to lock this file? */
  if (tunable_lock_upload_files)
  {
    vsf_sysutil_lock_file_write(new_file_fd);
    if (truncit)
    {
      vsf_sysutil_truncate(new_file_fd, 0);
      vsf_sysutil_lseek_to(new_file_fd, 0);
    }
  }
  if (!is_append && offset != 0)
  {
    /* XXX - warning, allows seek past end of file! Check for seek > size? */
    vsf_sysutil_lseek_to(new_file_fd, offset);
  }
  if (is_unique)
  {
    struct mystr resp_str = INIT_MYSTR;
    str_alloc_text(&resp_str, "FILE: ");
    str_append_str(&resp_str, p_filename);
    remote_fd = get_remote_transfer_fd(p_sess, str_getbuf(&resp_str));
    str_free(&resp_str);
  }
  else
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/03 masuu */
    remote_fd = get_remote_transfer_fd(p_sess, "File status okay; about to open data connection.");
#else /* _RES_TYPE_XDS_ */
    remote_fd = get_remote_transfer_fd(p_sess, "Ok to send data.");
#endif /* _RES_TYPE_XDS_ */
  }
  if (vsf_sysutil_retval_is_error(remote_fd))
  {
    goto port_pasv_cleanup_out;
  }

  /* add : PD area access check & save 11/05/02 masuu */
  if(!is_append) {
    if( str_equal_text(&p_sess->user_str, "root") || str_equal_text(&p_sess->user_str, "loggetter") ) {
    }
    else {
      str_copy(&s_buf_str, &p_sess->ftp_arg_str);
      prepend_path_to_filename(&s_buf_str);
      str_split_char(&s_buf_str, &s_topdir_str, '/');
      str_split_char(&s_topdir_str, &s_buf_str, '/');
      if( str_equal_text(&s_topdir_str, "DISC") ) {
        p_sess->pd_access = 1;
      }
    }
  }

#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/03 masuu */
  if (tunable_ascii_upload_enable && p_sess->is_ascii)
  {
    gettimeofday(&starttime, NULL);
    trans_ret = vsf_ftpdataio_transfer_file(p_sess, remote_fd, new_file_fd, 1, 1);
    gettimeofday(&endtime, NULL);
  }
  else
  {
    gettimeofday(&starttime, NULL);
    trans_ret = vsf_ftpdataio_transfer_file(p_sess, remote_fd, new_file_fd, 1, 0);
    gettimeofday(&endtime, NULL);
  }
#else /* _RES_TYPE_XDS_ */
  if (tunable_ascii_upload_enable && p_sess->is_ascii)
  {
    trans_ret = vsf_ftpdataio_transfer_file(p_sess, remote_fd,
                                            new_file_fd, 1, 1);
  }
  else
  {
    trans_ret = vsf_ftpdataio_transfer_file(p_sess, remote_fd,
                                            new_file_fd, 1, 0);
  }
#endif /* _RES_TYPE_XDS_ */

  vsf_ftpdataio_dispose_transfer_fd(p_sess);
  p_sess->transfer_size = trans_ret.transferred;
  /* XXX - handle failure, delete file? */
  if (trans_ret.retval == 0)
  {
    vsf_log_do_log(p_sess, 1);
  }

  /* chg : close timing for CQ1323 10/06/03 masuu */
  /* chg : CQ3068 10/12/02 masuu */
  /* chg : ioctl close 11/02/28 masuu */
#if(1)
  if( str_equal_text(&p_sess->user_str, "root") || str_equal_text(&p_sess->user_str, "loggetter") ) {
    close_return = vsf_sysutil_close2(new_file_fd);
  }
  else {
    close_return = vsf_sysutil_ioctl_close(new_file_fd);
    vsf_sysutil_close2(new_file_fd);
  }
#else
  close_return = vsf_sysutil_close2(new_file_fd);
#endif

#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/03 masuu */
  if (trans_ret.retval == -1)
  {
    vsf_cmdio_write(p_sess, FTP_BADSENDFILE, "Requested action aborted. Local error in processing.");
  }
  else if (trans_ret.retval == -2)
  {
    vsf_cmdio_write(p_sess, FTP_BADREADSEND, "Error during read to data connection.");
  }
  /* add : response code 11/05/05 masuu */
  else if (trans_ret.retval == -3)
  {
    vsf_cmdio_write(p_sess, FTP_DISKFULL, "Requested file action not taken. Insufficient space.");
  }
  else if (close_return == 0)
  {
    transfer_speed_write(p_sess, &starttime, &endtime, p_sess->transfer_size);
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Closing data connection. Requested file action successful.");
  }
  /* add : response code 11/02/28 masuu */
  else if (close_return == Ioctl_Close_Err_Umid_Exist)
  {
    vsf_cmdio_write(p_sess, FTP_UMID_EXIST, "Problem of external reference in XML document. code=3020");
  }
  else if (close_return == Ioctl_Close_War_Umid_Changed)
  {
    transfer_speed_write(p_sess, &starttime, &endtime, p_sess->transfer_size);
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Requested file action successful. Revising UMID. code=4902");
  }
  else if (close_return == Ioctl_Close_War_Dur_Changed)
  {
    transfer_speed_write(p_sess, &starttime, &endtime, p_sess->transfer_size);
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Requested file action successful. Revising Rec Duration. code=4903");
  }
  else if (close_return == Ioctl_Close_War_Else)
  {
    transfer_speed_write(p_sess, &starttime, &endtime, p_sess->transfer_size);
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Requested file action successful. Revising Header Metadata. code=4901");
  }
  /* add : CQ4966 11/05/30 masuu */
  else if (close_return == Ioctl_Close_Err_Disk_Full)
  {
    vsf_cmdio_write(p_sess, FTP_DISKFULL, "Requested file action not taken. Insufficient space.");
  }
  else
  {
    vsf_cmdio_write(p_sess, FTP_BADSENDFILE, "Requested action aborted. Local error in processing.");
  }
#else /* _RES_TYPE_XDS_ */
  if (trans_ret.retval == -1)
  {
    vsf_cmdio_write(p_sess, FTP_BADSENDFILE, "Failure writing to local file.");
  }
  else if (trans_ret.retval == -2)
  {
    vsf_cmdio_write(p_sess, FTP_BADSENDNET, "Failure reading network stream.");
  }
  else if (close_return == 0)
  {
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "File receive OK.");
  }
  else
  {
    vsf_cmdio_write(p_sess, FTP_BADSENDNET, "Failure reading network stream.");
  }
#endif /* _RES_TYPE_XDS_ */
  check_abor(p_sess);
  port_cleanup(p_sess);
  pasv_cleanup(p_sess);
  return;

port_pasv_cleanup_out:
  port_cleanup(p_sess);
  pasv_cleanup(p_sess);
  vsf_sysutil_close(new_file_fd);
  return;
}

static void
handle_mkd(struct vsf_session* p_sess)
{
  static struct mystr s_topdir_str, s_buf_str;	// add : 11/05/02 masuu
  int retval;

  resolve_tilde(&p_sess->ftp_arg_str, p_sess);
  vsf_log_start_entry(p_sess, kVSFLogEntryMkdir);
  str_copy(&p_sess->log_str, &p_sess->ftp_arg_str);
  prepend_path_to_filename(&p_sess->log_str);
  if (!vsf_access_check_file(&p_sess->ftp_arg_str))
  {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }

  /* add : PD area access check & save 11/05/02 masuu */
  if( str_equal_text(&p_sess->user_str, "root") || str_equal_text(&p_sess->user_str, "loggetter") ) {
  }
  else {
    str_copy(&s_buf_str, &p_sess->ftp_arg_str);
    prepend_path_to_filename(&s_buf_str);
    str_split_char(&s_buf_str, &s_topdir_str, '/');
    str_split_char(&s_topdir_str, &s_buf_str, '/');
    if( str_equal_text(&s_topdir_str, "DISC") ) {
      p_sess->pd_access = 1;
    }
  }

  /* NOTE! Actual permissions will be governed by the tunable umask */
  retval = str_mkdir(&p_sess->ftp_arg_str, 0777);
  if (retval != 0)
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/31 masuu */
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_FILEFAIL,
                    "Create directory operation failed.");
#endif /* _RES_TYPE_XDS_ */
    return;
  }
  vsf_log_do_log(p_sess, 1);
  {
    static struct mystr s_mkd_res;
    static struct mystr s_tmp_str;
    str_copy(&s_tmp_str, &p_sess->ftp_arg_str);
    prepend_path_to_filename(&s_tmp_str);
    /* Double up double quotes */
    str_replace_text(&s_tmp_str, "\"", "\"\"");
    /* Build result string */
    str_alloc_text(&s_mkd_res, "\"");
    str_append_str(&s_mkd_res, &s_tmp_str);
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/31 masuu */
    str_append_text(&s_mkd_res, "\" new directory created.");
#else /* _RES_TYPE_XDS_ */
    str_append_text(&s_mkd_res, "\" created");
#endif /* _RES_TYPE_XDS_ */
    vsf_cmdio_write_str(p_sess, FTP_MKDIROK, &s_mkd_res);
  }
}

static void
handle_rmd(struct vsf_session* p_sess)
{
  static struct mystr s_topdir_str, s_buf_str;	// add : 11/05/02 masuu
  int retval;

  resolve_tilde(&p_sess->ftp_arg_str, p_sess);
  vsf_log_start_entry(p_sess, kVSFLogEntryRmdir);
  str_copy(&p_sess->log_str, &p_sess->ftp_arg_str);
  prepend_path_to_filename(&p_sess->log_str);
  if (!vsf_access_check_file(&p_sess->ftp_arg_str))
  {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }

  /* add : PD area access check & save 11/05/02 masuu */
  if( str_equal_text(&p_sess->user_str, "root") || str_equal_text(&p_sess->user_str, "loggetter") ) {
  }
  else {
    str_copy(&s_buf_str, &p_sess->ftp_arg_str);
    prepend_path_to_filename(&s_buf_str);
    str_split_char(&s_buf_str, &s_topdir_str, '/');
    str_split_char(&s_topdir_str, &s_buf_str, '/');
    if( str_equal_text(&s_topdir_str, "DISC") ) {
      p_sess->pd_access = 1;
    }
  }

  retval = str_rmdir(&p_sess->ftp_arg_str);
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/28 masuu */
  if (retval != 0)
  {
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
  }
  else
  {
    vsf_log_do_log(p_sess, 1);
    vsf_cmdio_write(p_sess, FTP_RMDIROK,
                    "Requested file action okay, completed.");
  }
#else /* _RES_TYPE_XDS_ */
  if (retval != 0)
  {
    vsf_cmdio_write(p_sess, FTP_FILEFAIL,
                    "Remove directory operation failed.");
  }
  else
  {
    vsf_log_do_log(p_sess, 1);
    vsf_cmdio_write(p_sess, FTP_RMDIROK,
                    "Remove directory operation successful.");
  }
#endif /* _RES_TYPE_XDS_ */
}

static void
handle_dele(struct vsf_session* p_sess)
{
  static struct mystr s_topdir_str, s_buf_str;	// add : 11/05/02 masuu
  int retval;

  resolve_tilde(&p_sess->ftp_arg_str, p_sess);
  vsf_log_start_entry(p_sess, kVSFLogEntryDelete);
  str_copy(&p_sess->log_str, &p_sess->ftp_arg_str);
  prepend_path_to_filename(&p_sess->log_str);
  if (!vsf_access_check_file(&p_sess->ftp_arg_str))
  {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }

  /* add : PD area access check & save 11/05/02 masuu */
  if( str_equal_text(&p_sess->user_str, "root") || str_equal_text(&p_sess->user_str, "loggetter") ) {
  }
  else {
    str_copy(&s_buf_str, &p_sess->ftp_arg_str);
    prepend_path_to_filename(&s_buf_str);
    str_split_char(&s_buf_str, &s_topdir_str, '/');
    str_split_char(&s_topdir_str, &s_buf_str, '/');
    if( str_equal_text(&s_topdir_str, "DISC") ) {
      p_sess->pd_access = 1;
    }
  }

  retval = str_unlink(&p_sess->ftp_arg_str);
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/28 masuu */
  if (retval != 0)
  {
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
  }
  else
  {
    vsf_log_do_log(p_sess, 1);
    vsf_cmdio_write(p_sess, FTP_DELEOK, "Requested file action okay, completed.");
  }
#else /* _RES_TYPE_XDS_ */
  if (retval != 0)
  {
    vsf_cmdio_write(p_sess, FTP_FILEFAIL, "Delete operation failed.");
  }
  else
  {
    vsf_log_do_log(p_sess, 1);
    vsf_cmdio_write(p_sess, FTP_DELEOK, "Delete operation successful.");
  }
#endif /* _RES_TYPE_XDS_ */
}

static void
handle_rest(struct vsf_session* p_sess)
{
  static struct mystr s_rest_str;
  filesize_t val = str_a_to_filesize_t(&p_sess->ftp_arg_str);
  if (val < 0)
  {
    val = 0;
  }
  p_sess->restart_pos = val;
  str_alloc_text(&s_rest_str, "Restart position accepted (");
  str_append_filesize_t(&s_rest_str, val);
  str_append_text(&s_rest_str, ").");
  vsf_cmdio_write_str(p_sess, FTP_RESTOK, &s_rest_str);
}

static void
handle_rnfr(struct vsf_session* p_sess)
{
  static struct vsf_sysutil_statbuf* p_statbuf;
  int retval;
  /* Clear old value */
  str_free(&p_sess->rnfr_filename_str);
  resolve_tilde(&p_sess->ftp_arg_str, p_sess);
  if (!vsf_access_check_file(&p_sess->ftp_arg_str))
  {
    vsf_log_start_entry(p_sess, kVSFLogEntryRename);
    str_copy(&p_sess->log_str, &p_sess->ftp_arg_str);
    prepend_path_to_filename(&p_sess->log_str);
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }
  /* Does it exist? */
  retval = str_stat(&p_sess->ftp_arg_str, &p_statbuf);

#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/28 masuu */
  if (retval == 0)
  {
    /* Yes */
    str_copy(&p_sess->rnfr_filename_str, &p_sess->ftp_arg_str);
    vsf_cmdio_write(p_sess, FTP_RNFROK, "Requested file action pending further information.");
  }
  else
  {
    vsf_log_start_entry(p_sess, kVSFLogEntryRename);
    str_copy(&p_sess->log_str, &p_sess->ftp_arg_str);
    prepend_path_to_filename(&p_sess->log_str);
    vsf_cmdio_write(p_sess, FTP_FILEFAIL, "File not found, no access. code=3001");
  }
#else /* _RES_TYPE_XDS_ */
  if (retval == 0)
  {
    /* Yes */
    str_copy(&p_sess->rnfr_filename_str, &p_sess->ftp_arg_str);
    vsf_cmdio_write(p_sess, FTP_RNFROK, "Ready for RNTO.");
  }
  else
  {
    vsf_log_start_entry(p_sess, kVSFLogEntryRename);
    str_copy(&p_sess->log_str, &p_sess->ftp_arg_str);
    prepend_path_to_filename(&p_sess->log_str);
    vsf_cmdio_write(p_sess, FTP_FILEFAIL, "RNFR command failed.");
  }
#endif /* _RES_TYPE_XDS_ */

}

static void
handle_rnto(struct vsf_session* p_sess)
{
  static struct mystr s_topdir_str, s_buf_str;	// add : 11/05/02 masuu
  static struct mystr s_tmp_str;
  int retval;
  /* If we didn't get a RNFR, throw a wobbly */
  if (str_isempty(&p_sess->rnfr_filename_str))
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/28 masuu */
    vsf_cmdio_write(p_sess, FTP_NEEDRNFR, "Bad sequence of commands.");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_NEEDRNFR, "RNFR required first.");
#endif /* _RES_TYPE_XDS_ */
    return;
  }
  resolve_tilde(&p_sess->ftp_arg_str, p_sess);
  vsf_log_start_entry(p_sess, kVSFLogEntryRename);
  str_copy(&p_sess->log_str, &p_sess->rnfr_filename_str);
  prepend_path_to_filename(&p_sess->log_str);
  str_append_char(&p_sess->log_str, ' ');
  str_copy(&s_tmp_str, &p_sess->ftp_arg_str);
  prepend_path_to_filename(&s_tmp_str);
  str_append_str(&p_sess->log_str, &s_tmp_str);
  if (!vsf_access_check_file(&p_sess->ftp_arg_str))
  {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }

  /* add : PD area access check & save 11/05/02 masuu */
  if( str_equal_text(&p_sess->user_str, "root") || str_equal_text(&p_sess->user_str, "loggetter") ) {
  }
  else {
    /* From file path check & save */
    str_copy(&s_buf_str, &p_sess->rnfr_filename_str);
    prepend_path_to_filename(&s_buf_str);
    str_split_char(&s_buf_str, &s_topdir_str, '/');
    str_split_char(&s_topdir_str, &s_buf_str, '/');
    if( str_equal_text(&s_topdir_str, "DISC") ) {
      p_sess->pd_access = 1;
    }
    /* To file path check & save */
    str_copy(&s_buf_str, &p_sess->ftp_arg_str);
    prepend_path_to_filename(&s_buf_str);
    str_split_char(&s_buf_str, &s_topdir_str, '/');
    str_split_char(&s_topdir_str, &s_buf_str, '/');
    if( str_equal_text(&s_topdir_str, "DISC") ) {
      p_sess->pd_access = 1;
    }
  }

  /* NOTE - might overwrite destination file. Not a concern because the same
   * could be accomplished with DELE.
   */
  retval = str_rename(&p_sess->rnfr_filename_str, &p_sess->ftp_arg_str);
  /* Clear the RNFR filename; start the two stage process again! */
  str_free(&p_sess->rnfr_filename_str);
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/28 masuu */
  if (retval == 0)
  {
    vsf_log_do_log(p_sess, 1);
    vsf_cmdio_write(p_sess, FTP_RENAMEOK, "Requested file action okay, completed.");
  }
  else
  {
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
  }
#else /* _RES_TYPE_XDS_ */
  if (retval == 0)
  {
    vsf_log_do_log(p_sess, 1);
    vsf_cmdio_write(p_sess, FTP_RENAMEOK, "Rename successful.");
  }
  else
  {
    vsf_cmdio_write(p_sess, FTP_FILEFAIL, "Rename failed.");
  }
#endif /* _RES_TYPE_XDS_ */
}

static void
handle_nlst(struct vsf_session* p_sess)
{
  handle_dir_common(p_sess, 0, 0);
}

static void
prepend_path_to_filename(struct mystr* p_str)
{
  static struct mystr s_tmp_str;
  /* Only prepend current working directory if the incoming filename is
   * relative
   */
  str_empty(&s_tmp_str);
  if (str_isempty(p_str) || str_get_char_at(p_str, 0) != '/')
  {
    str_getcwd(&s_tmp_str);
    /* Careful to not emit // if we are in directory / (common with chroot) */
    if (str_isempty(&s_tmp_str) ||
        str_get_char_at(&s_tmp_str, str_getlen(&s_tmp_str) - 1) != '/')
    {
      str_append_char(&s_tmp_str, '/');
    }
  }
  str_append_str(&s_tmp_str, p_str);
  str_copy(p_str, &s_tmp_str);
}


static void
handle_sigurg(void* p_private)
{
  struct mystr async_cmd_str = INIT_MYSTR;
  struct mystr async_arg_str = INIT_MYSTR;
  struct mystr real_cmd_str = INIT_MYSTR;
  unsigned int len;
  struct vsf_session* p_sess = (struct vsf_session*) p_private;
  /* Did stupid client sent something OOB without a data connection? */
  if (p_sess->data_fd == -1)
  {
    return;
  }
  /* Get the async command - blocks (use data timeout alarm) */
  vsf_cmdio_get_cmd_and_arg(p_sess, &async_cmd_str, &async_arg_str, 0);
  /* Chop off first four characters; they are telnet characters. The client
   * should have sent the first two normally and the second two as urgent
   * data.
   */
  len = str_getlen(&async_cmd_str);
  if (len >= 4)
  {
    str_right(&async_cmd_str, &real_cmd_str, len - 4);
  }
  if (str_equal_text(&real_cmd_str, "ABOR"))
  {
    p_sess->abor_received = 1;
    /* This is failok because of a small race condition; the SIGURG might
     * be raised after the data socket is closed, but before data_fd is
     * set to -1.
     */
    vsf_sysutil_shutdown_failok(p_sess->data_fd);
  }
  else
  {
    /* Sorry! */
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/31 masuu */
    vsf_cmdio_write(p_sess, FTP_BADCMD, "Syntax error, command unrecognized.");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_BADCMD, "Unknown command.");
#endif /* _RES_TYPE_XDS_ */
  }
  str_free(&async_cmd_str);
  str_free(&async_arg_str);
  str_free(&real_cmd_str);
}

static int
get_remote_transfer_fd(struct vsf_session* p_sess, const char* p_status_msg)
{
  int remote_fd;
  if (!pasv_active(p_sess) && !port_active(p_sess))
  {
    bug("neither PORT nor PASV active in get_remote_transfer_fd");
  }
  p_sess->abor_received = 0;
  if (pasv_active(p_sess))
  {
    remote_fd = vsf_ftpdataio_get_pasv_fd(p_sess);
  }
  else
  {
    remote_fd = vsf_ftpdataio_get_port_fd(p_sess);
  }
  if (vsf_sysutil_retval_is_error(remote_fd))
  {
    return remote_fd;
  }
  vsf_cmdio_write(p_sess, FTP_DATACONN, p_status_msg);
  if (vsf_ftpdataio_post_mark_connect(p_sess) != 1)
  {
    vsf_ftpdataio_dispose_transfer_fd(p_sess);
    return -1;
  }
  return remote_fd;
}

static void
check_abor(struct vsf_session* p_sess)
{
  /* If the client sent ABOR, respond to it here */
  if (p_sess->abor_received)
  {
    p_sess->abor_received = 0;
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/31 masuu */
    vsf_cmdio_write(p_sess, FTP_BADSENDNET, "Connection closed: transfer aborted.");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_ABOROK, "ABOR successful.");
#endif /* _RES_TYPE_XDS_ */
  }
}

static void
handle_size(struct vsf_session* p_sess)
{
  /* Note - in ASCII mode, are supposed to return the size after taking into
   * account ASCII linefeed conversions. At least this is what wu-ftpd does in
   * version 2.6.1. Proftpd-1.2.0pre fails to do this.
   * I will not do it because it is a potential I/O DoS.
   */
  static struct vsf_sysutil_statbuf* s_p_statbuf;
  int retval;
  resolve_tilde(&p_sess->ftp_arg_str, p_sess);
  if (!vsf_access_check_file(&p_sess->ftp_arg_str))
  {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }
  retval = str_stat(&p_sess->ftp_arg_str, &s_p_statbuf);
  if (retval != 0 || !vsf_sysutil_statbuf_is_regfile(s_p_statbuf))
  {
    vsf_cmdio_write(p_sess, FTP_FILEFAIL, "Could not get file size.");
  }
  else
  {
    static struct mystr s_size_res_str;
    str_alloc_filesize_t(&s_size_res_str,
                         vsf_sysutil_statbuf_get_size(s_p_statbuf));
    vsf_cmdio_write_str(p_sess, FTP_SIZEOK, &s_size_res_str);
  }
}

static void
handle_site(struct vsf_session* p_sess)
{
  static struct mystr s_site_args_str;
  /* What SITE sub-command is it? */
  str_split_char(&p_sess->ftp_arg_str, &s_site_args_str, ' ');
  str_upper(&p_sess->ftp_arg_str);
#ifdef _CMD_TYPE_XDS_	/* chg : for XDS 10/05/20 masuu */
  /* add : SITE UMMD 10/04/22 masuu */
  if (str_equal_text(&p_sess->ftp_arg_str, "UMMD"))
  {
    handle_site_ummd(p_sess, &s_site_args_str);
  }
  /* add : SITE REPF 09/12/01 masuu */
  else if (tunable_download_enable && str_equal_text(&p_sess->ftp_arg_str, "REPF"))
  {
    handle_site_repf(p_sess, &s_site_args_str);
  }
  /* add : SITE REPFL 09/12/15 masuu */
  else if (tunable_download_enable && str_equal_text(&p_sess->ftp_arg_str, "REPFL"))
  {
    handle_site_repfl(p_sess, &s_site_args_str);
  }
  /* add : SITE DF 11/03/10 masuu */
  else if (str_equal_text(&p_sess->ftp_arg_str, "DF"))
  {
    handle_site_df(p_sess, &s_site_args_str);
  }
  /* add : SITE LOGF 10/09/08 masuu */
  else if (str_equal_text(&p_sess->ftp_arg_str, "LOGF"))
  {
    handle_site_logflush(p_sess, &s_site_args_str);
  }
#if(0)
  /* add : SITE FUNC 10/12/06 masuu */
  else if (str_equal_text(&p_sess->ftp_arg_str, "FUNC"))
  {
    handle_site_func(p_sess, &s_site_args_str);
  }
  else if (str_equal_text(&p_sess->ftp_arg_str, "HELP"))
  {
    vsf_cmdio_write(p_sess, FTP_SITEHELP, "UMMD REPF REPFL FUNC DF LOGF HELP");
  }
#else
  else if (str_equal_text(&p_sess->ftp_arg_str, "HELP"))
  {
    vsf_cmdio_write(p_sess, FTP_SITEHELP, "UMMD REPF REPFL DF LOGF HELP");
  }
#endif
#else /* _CMD_TYPE_XDS_ */
  if (tunable_write_enable &&
      tunable_chmod_enable &&
      str_equal_text(&p_sess->ftp_arg_str, "CHMOD"))
  {
    handle_site_chmod(p_sess, &s_site_args_str);
  }
  else if (str_equal_text(&p_sess->ftp_arg_str, "UMASK"))
  {
    handle_site_umask(p_sess, &s_site_args_str);
  }
  else if (str_equal_text(&p_sess->ftp_arg_str, "HELP"))
  {
    vsf_cmdio_write(p_sess, FTP_SITEHELP, "CHMOD UMASK HELP");
  }
#endif /* _CMD_TYPE_XDS_ */
  else
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/31 masuu */
    vsf_cmdio_write(p_sess, FTP_BADCMD, "Syntax error, command unrecognized.");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_BADCMD, "Unknown SITE command.");
#endif /* _RES_TYPE_XDS_ */
  }
}

static void
handle_site_chmod(struct vsf_session* p_sess, struct mystr* p_arg_str)
{
  static struct mystr s_chmod_file_str;
  unsigned int perms;
  int retval;
  if (str_isempty(p_arg_str))
  {
    vsf_cmdio_write(p_sess, FTP_BADCMD, "SITE CHMOD needs 2 arguments.");
    return;
  }
  str_split_char(p_arg_str, &s_chmod_file_str, ' ');
  if (str_isempty(&s_chmod_file_str))
  {
    vsf_cmdio_write(p_sess, FTP_BADCMD, "SITE CHMOD needs 2 arguments.");
    return;
  }
  resolve_tilde(&s_chmod_file_str, p_sess);
  vsf_log_start_entry(p_sess, kVSFLogEntryChmod);
  str_copy(&p_sess->log_str, &s_chmod_file_str);
  prepend_path_to_filename(&p_sess->log_str);
  str_append_char(&p_sess->log_str, ' ');
  str_append_str(&p_sess->log_str, p_arg_str);
  if (!vsf_access_check_file(&s_chmod_file_str))
  {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }
  /* Don't worry - our chmod() implementation only allows 0 - 0777 */
  perms = str_octal_to_uint(p_arg_str);
  retval = str_chmod(&s_chmod_file_str, perms);
  if (vsf_sysutil_retval_is_error(retval))
  {
    vsf_cmdio_write(p_sess, FTP_FILEFAIL, "SITE CHMOD command failed.");
  }
  else
  {
    vsf_log_do_log(p_sess, 1);
    vsf_cmdio_write(p_sess, FTP_CHMODOK, "SITE CHMOD command ok.");
  }
}

static void
handle_site_umask(struct vsf_session* p_sess, struct mystr* p_arg_str)
{
  static struct mystr s_umask_resp_str;
  if (str_isempty(p_arg_str))
  {
    /* Empty arg => report current umask */
    str_alloc_text(&s_umask_resp_str, "Your current UMASK is ");
    str_append_text(&s_umask_resp_str,
                    vsf_sysutil_uint_to_octal(vsf_sysutil_get_umask()));
  }
  else
  {
    /* Set current umask */
    unsigned int new_umask = str_octal_to_uint(p_arg_str);
    vsf_sysutil_set_umask(new_umask);
    str_alloc_text(&s_umask_resp_str, "UMASK set to ");
    str_append_text(&s_umask_resp_str,
                    vsf_sysutil_uint_to_octal(vsf_sysutil_get_umask()));
  }
  vsf_cmdio_write_str(p_sess, FTP_UMASKOK, &s_umask_resp_str);
}

/* add : for SITE UMMD 10/04/22 masuu */
static void handle_site_ummd(struct vsf_session* p_sess, struct mystr* p_arg_str) {
  if (str_isempty(p_arg_str)) {
  	p_sess->xds_mode = VSFTP_XDS_MODE_UMMD;
//    vsf_log_do_log(p_sess, 1);
    vsf_cmdio_write(p_sess, FTP_UMMDOK, "Switched UMID mode to \"unchange\"");
  }
  else {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
  }
}

/* add : for SITE FUNC 10/12/06 masuu */
static void handle_site_func(struct vsf_session* p_sess, struct mystr* p_arg_str) {
  if (str_isempty(p_arg_str)) {
//    vsf_log_do_log(p_sess, 1);
    vsf_cmdio_write(p_sess, FTP_FUNCOK, "MXF DISK 1");
  }
  else {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
  }
}

/* add : for SITE DF 11/03/10 masuu */
static void handle_site_df(struct vsf_session* p_sess, struct mystr* p_arg_str) {
  if (str_isempty(p_arg_str)) {
    static struct vsf_sysutil_statfs_buf* statfs_buf;
    static struct mystr s_cwd_str, s_topdir_str, s_buf_str;
    struct vsf_confs_pd_size ioc_param;
    struct vsf_df_size df_size;
    long free_size, block_size, total_size;
    int opened_file;
    int retval = 0;

    /* Fine, do it */
    int remote_fd = get_remote_transfer_fd(p_sess, "File status okay; about to open data connection.");
    if (vsf_sysutil_retval_is_error(remote_fd)) {
      goto dir_close_out;
    }

    /* New Function */
    str_getcwd(&s_cwd_str);
    str_copy(&s_buf_str, &s_cwd_str);
    str_split_char(&s_buf_str, &s_topdir_str, '/');
    str_split_char(&s_topdir_str, &s_buf_str, '/');
    if( str_equal_text(&s_topdir_str, "DISC") ) {
      opened_file = str_dummy_open(0);
      if (vsf_sysutil_retval_is_error(opened_file)) {
        vsf_ftpdataio_dispose_transfer_fd(p_sess);
        vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
        vsf_sysutil_close(opened_file);
        goto dir_close_out;
      }
      if(vsf_sysutil_ioctl(opened_file, VSF_CONFS_IOCCMD_PDSIZE, &ioc_param) < 0) {
        vsf_ftpdataio_dispose_transfer_fd(p_sess);
        vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
        vsf_sysutil_close(opened_file);
        goto dir_close_out;
      }
      df_size.rt_remain_size = ioc_param.rt_remain_size;
      df_size.nrt_remain_size = ioc_param.nrt_remain_size;
      if( ioc_param.rt_remain_size > ioc_param.rt_total_size ) {
        vsf_ftpdataio_dispose_transfer_fd(p_sess);
        vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
        vsf_sysutil_close(opened_file);
        goto dir_close_out;
      }
      else if( ioc_param.rt_total_size == 0 ) {
        df_size.rt_percent = 100;
      }
      else {
        df_size.rt_percent = ( ioc_param.rt_total_size - ioc_param.rt_remain_size ) * 100 / ioc_param.rt_total_size;
      }
      if( ioc_param.nrt_remain_size > ioc_param.nrt_total_size ) {
        vsf_ftpdataio_dispose_transfer_fd(p_sess);
        vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
        vsf_sysutil_close(opened_file);
        goto dir_close_out;
      }
      else if( ioc_param.nrt_total_size == 0 ) {
        df_size.nrt_percent = 100;
      }
      else {
        df_size.nrt_percent = ( ioc_param.nrt_total_size - ioc_param.nrt_remain_size ) * 100 / ioc_param.nrt_total_size;
      }
      vsf_sysutil_close(opened_file);
    }
    else {
      vsf_sysutil_statfs(str_strdup(&s_cwd_str), &statfs_buf);
      free_size = vsf_sysutil_statfs_buf_get_free_size(statfs_buf);
      block_size = vsf_sysutil_statfs_buf_get_block_size(statfs_buf);
      total_size = vsf_sysutil_statfs_buf_get_total_size(statfs_buf);
//      df_size.nrt_remain_size = ( free_size * block_size ) / 40;
      df_size.nrt_remain_size = 0;
      if( ( free_size * block_size ) < df_size.nrt_remain_size ) {
        vsf_ftpdataio_dispose_transfer_fd(p_sess);
        vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
        vsf_sysutil_close(opened_file);
        goto dir_close_out;
      }
      else if( total_size == 0 ) {
        df_size.rt_remain_size = ( free_size * block_size ) - df_size.nrt_remain_size;
        df_size.rt_percent = 100;
        df_size.nrt_percent = 100;
      }
      else {
        df_size.rt_remain_size = ( free_size * block_size ) - df_size.nrt_remain_size;
        df_size.rt_percent = ( total_size - free_size ) * 100 / total_size;
//        df_size.nrt_percent = ( total_size - free_size ) * 100 / total_size;
        df_size.nrt_percent = 100;
      }
    }

    retval = vsf_ftpdataio_transfer_df(p_sess, &df_size);
    vsf_ftpdataio_dispose_transfer_fd(p_sess);

    if (retval == 0) {
      vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Closing data connection. Requested file action successful.");
    }
    else {
      vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
    }

    check_abor(p_sess);
dir_close_out:
    port_cleanup(p_sess);
    pasv_cleanup(p_sess);
  }
  else {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
  }
}

/* add : for SITE LOGF 10/09/08 masuu */
static void handle_site_logflush(struct vsf_session* p_sess, struct mystr* p_arg_str) {
  if (str_isempty(p_arg_str)) {
//    vsf_log_do_log(p_sess, 1);
    /* add : CQ3313 10/12/13 masuu */
    vsf_sysutil_install_null_sighandler(kVSFSysUtilSigCHLD);
    vsf_sysutil_unblock_sig(kVSFSysUtilSigCHLD);
    if( vsf_sysutil_system("/usr/local/sony/asca/sys/bin/fwc --logflush") == 0 ) {
        vsf_cmdio_write(p_sess, FTP_LOGFOK, "Command okay.");
    }
    else {
        vsf_cmdio_write(p_sess, FTP_LOGFNG, "Requested action aborted. Local error in processing.");
    }
    /* add : CQ3313 10/12/13 masuu */
    vsf_sysutil_block_sig(kVSFSysUtilSigCHLD);
    vsf_sysutil_install_async_sighandler(kVSFSysUtilSigCHLD, handle_sigchld);
  }
  else {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
  }
}

/* add : for SITE REPF 09/12/01 masuu */
static void handle_site_repf(struct vsf_session* p_sess, struct mystr* p_arg_str) {
  static struct mystr s_path_str, s_offset_str, s_size_str, s_buf_str;
  static struct mystr s_mark_str;
  static struct vsf_sysutil_statbuf* s_p_statbuf;
  struct vsf_transfer_ret trans_ret;
  struct timeval starttime, endtime;	// add : 10/06/01 masuu
  struct vsf_confs_popen_param ioc_param;
  int remote_fd;
  int opened_file;
  int is_ascii = 0;
//  int err_no;
  filesize_t offset = p_sess->restart_pos;

/* add : argument check & separate 09/12/07 masuu */
/* chg : output message 10/06/01 masuu */
  str_copy(&s_path_str, p_arg_str);
  if (str_isempty(&s_path_str)) {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }
  str_split_char(&s_path_str, &s_offset_str, ' ');
  if (str_isempty(&s_offset_str)) {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }
  str_split_char(&s_offset_str, &s_size_str, ' ');
  if (str_isempty(&s_size_str)) {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }
  str_split_char(&s_size_str, &s_buf_str, ' ');
  if (str_isempty(&s_buf_str)) {
  }
  else {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }

  /* pasv & port check */
  p_sess->restart_pos = 0;
  if (!data_transfer_checks_ok(p_sess)) {
    return;
  }

/* chg : output message 10/06/01 masuu */
  /* binary mode check */
  if (p_sess->is_ascii && offset != 0) {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }

/* chg : argument 09/12/07 masuu */
  /* log setting */
  resolve_tilde(&s_path_str, p_sess);
  vsf_log_start_entry(p_sess, kVSFLogEntryDownload);
  str_copy(&p_sess->log_str, &s_path_str);
  prepend_path_to_filename(&p_sess->log_str);
  /* add : offset + size 09/12/15 masuu */
  str_append_char(&p_sess->log_str, ' ');
  str_append_str(&p_sess->log_str, &s_offset_str);
  str_append_char(&p_sess->log_str, ' ');
  str_append_str(&p_sess->log_str, &s_size_str);

  /* access check */
  if (!vsf_access_check_file(&s_path_str)) {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }

/* chg : open -> dummy open + ioctl 09/12/07 masuu */
/* chg : output message 10/06/01 masuu */
/* chg : for select file 11/02/07 masuu */
  /* open */
  opened_file = str_dummy_open(1);
  if (vsf_sysutil_retval_is_error(opened_file)) {
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
    return;
  }

/* chg : output message 10/06/01 masuu */
  /* ioctl */
#if(1)
  ioc_param.offset = str_atoi(&s_offset_str);
  ioc_param.size = str_atoi(&s_size_str);
  prepend_path_to_filename(&s_path_str);
  vsf_sysutil_strcpy(ioc_param.path, str_getbuf(&s_path_str), str_getlen(&s_path_str) + 1);
  if(vsf_sysutil_ioctl(opened_file, VSF_CONFS_IOCCMD_POPEN, &ioc_param) < 0) {
//    err_no = errno;
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
    goto file_close_out;
  }
#endif

/* chg : fstat -> stat 09/12/11 masuu */
/* chg : output message 10/06/01 masuu */
  /* stat */
  vsf_sysutil_stat(str_getbuf(&s_path_str), &s_p_statbuf);
  /* No games please */
  if (!vsf_sysutil_statbuf_is_regfile(s_p_statbuf)) {
    /* Note - pretend open failed */
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
    goto file_close_out;
  }

  /* Optionally, we'll be paranoid and only serve publicly readable stuff */
  if (p_sess->is_anonymous && tunable_anon_world_readable_only &&
      !vsf_sysutil_statbuf_is_readable_other(s_p_statbuf)) {
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
    goto file_close_out;
  }

  /* Set the download offset (from REST) if any */
  if (offset != 0) {
    vsf_sysutil_lseek_to(opened_file, offset);
  }

/* chg : output message 10/06/01 masuu */
  /* get file handle for recv */
  str_alloc_text(&s_mark_str, "File status okay; about to open data connection.");
  remote_fd = get_remote_transfer_fd(p_sess, str_getbuf(&s_mark_str));
  if (vsf_sysutil_retval_is_error(remote_fd)) {
    goto port_pasv_cleanup_out;
  }

/* chg : output message 10/06/01 masuu */
  /* sendfile */
  gettimeofday(&starttime, NULL);
  /* chg : vsf_ftpdataio_transfer_file -> vsf_ftpdataio_transfer_file2 10/09/13 masuu */
  /* chg : for get file size with PATH 11/02/07 masuu */
  trans_ret = vsf_ftpdataio_transfer_file3(p_sess, remote_fd, opened_file, str_getbuf(&s_path_str), is_ascii);
  gettimeofday(&endtime, NULL);
  vsf_ftpdataio_dispose_transfer_fd(p_sess);
  p_sess->transfer_size = trans_ret.transferred;
  /* Log _after_ the blocking dispose call, so we get transfer times right */
  if (trans_ret.retval == 0) {
    vsf_log_do_log(p_sess, 1);
  }
  /* Emit status message _after_ blocking dispose call to avoid buggy FTP
   * clients truncating the transfer.
   */
  if (trans_ret.retval == -1) {
    vsf_cmdio_write(p_sess, FTP_BADSENDFILE, "Requested action aborted. Local error in processing.");
  }
/* chg : transfer size no check 11/03/09 masuu
  else if (trans_ret.retval == -2) {
    vsf_cmdio_write(p_sess, FTP_BADWRITESEND, "Error during write to data connection.");
  }
*/
  else {
    transfer_speed_write(p_sess, &starttime, &endtime, vsf_sysutil_statbuf_get_size(s_p_statbuf));
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Closing data connection. Requested file action successful.");
  }

  /* abort check */
  check_abor(p_sess);

  /* clean up */
port_pasv_cleanup_out:
  port_cleanup(p_sess);
  pasv_cleanup(p_sess);
file_close_out:
  vsf_sysutil_close(opened_file);
}

/* add : for SITE REPFL 09/12/15 masuu */
static void handle_site_repfl(struct vsf_session* p_sess, struct mystr* p_arg_str) {
  static struct mystr s_path_str, s_offset_str, s_size_str, s_buf_str;
  static struct mystr s_mark_str;
  static struct vsf_sysutil_statbuf* s_p_statbuf;
  struct vsf_transfer_ret trans_ret;
  struct timeval starttime, endtime;	// add : 10/06/01 masuu
  struct vsf_confs_popen_param ioc_param;
  int remote_fd;
  int opened_file;
  int is_ascii = 0;
//  int err_no;
  filesize_t offset = p_sess->restart_pos;

/* chg : argument check & separate 09/12/15 masuu */
/* chg : output message 10/06/01 masuu */
  str_copy(&s_buf_str, p_arg_str);
  if (str_isempty(&s_buf_str)) {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }
  str_split_char(&s_buf_str, &s_path_str, '\"');
  if (str_isempty(&s_path_str)) {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }
  str_split_char(&s_path_str, &s_buf_str, '\"');
  if (str_isempty(&s_buf_str)) {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }
  str_split_char(&s_buf_str, &s_offset_str, ' ');
  if (str_isempty(&s_offset_str)) {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }
  str_split_char(&s_offset_str, &s_size_str, ' ');
  if (str_isempty(&s_size_str)) {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }
  str_split_char(&s_size_str, &s_buf_str, ' ');
  if (str_isempty(&s_buf_str)) {
  }
  else {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }

  /* pasv & port check */
  p_sess->restart_pos = 0;
  if (!data_transfer_checks_ok(p_sess)) {
    return;
  }

/* chg : output message 10/06/01 masuu */
  /* binary mode check */
  if (p_sess->is_ascii && offset != 0) {
    vsf_cmdio_write(p_sess, FTP_BADPARAM, "Command not implemented for that parameter.");
    return;
  }

/* chg : argument 09/12/07 masuu */
  /* log setting */
  resolve_tilde(&s_path_str, p_sess);
  vsf_log_start_entry(p_sess, kVSFLogEntryDownload);
  str_copy(&p_sess->log_str, &s_path_str);
  prepend_path_to_filename(&p_sess->log_str);
  /* add : offset + size 09/12/15 masuu */
  str_append_char(&p_sess->log_str, ' ');
  str_append_str(&p_sess->log_str, &s_offset_str);
  str_append_char(&p_sess->log_str, ' ');
  str_append_str(&p_sess->log_str, &s_size_str);

  /* access check */
  if (!vsf_access_check_file(&s_path_str)) {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }

/* chg : open -> dummy open + ioctl 09/12/07 masuu */
/* chg : output message 10/06/01 masuu */
/* chg : for select file 11/02/07 masuu */
  /* open */
  opened_file = str_dummy_open(1);
  if (vsf_sysutil_retval_is_error(opened_file)) {
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
    return;
  }

/* chg : output message 10/06/01 masuu */
  /* ioctl */
#if(1)
  ioc_param.offset = str_atoi(&s_offset_str);
  ioc_param.size = str_atoi(&s_size_str);
  prepend_path_to_filename(&s_path_str);
  vsf_sysutil_strcpy(ioc_param.path, str_getbuf(&s_path_str), str_getlen(&s_path_str) + 1);
  if(vsf_sysutil_ioctl(opened_file, VSF_CONFS_IOCCMD_POPEN, &ioc_param) < 0) {
//    err_no = errno;
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
    goto file_close_out;
  }
#endif

/* chg : fstat -> stat 09/12/11 masuu */
/* chg : output message 10/06/01 masuu */
  /* stat */
  vsf_sysutil_stat(str_getbuf(&s_path_str), &s_p_statbuf);
  /* No games please */
  if (!vsf_sysutil_statbuf_is_regfile(s_p_statbuf)) {
    /* Note - pretend open failed */
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
    goto file_close_out;
  }

  /* Optionally, we'll be paranoid and only serve publicly readable stuff */
  if (p_sess->is_anonymous && tunable_anon_world_readable_only &&
      !vsf_sysutil_statbuf_is_readable_other(s_p_statbuf)) {
    vsf_cmdio_write(p_sess, FTP_INTERNALERR, "Requested action aborted. Local error in processing.");
    goto file_close_out;
  }

  /* Set the download offset (from REST) if any */
  if (offset != 0) {
    vsf_sysutil_lseek_to(opened_file, offset);
  }

/* chg : output message 10/06/01 masuu */
  /* get file handle for recv */
  str_alloc_text(&s_mark_str, "File status okay; about to open data connection.");
  remote_fd = get_remote_transfer_fd(p_sess, str_getbuf(&s_mark_str));
  if (vsf_sysutil_retval_is_error(remote_fd)) {
    goto port_pasv_cleanup_out;
  }

/* chg : output message 10/06/01 masuu */
  /* sendfile */
  gettimeofday(&starttime, NULL);
  /* chg : vsf_ftpdataio_transfer_file -> vsf_ftpdataio_transfer_file2 10/09/13 masuu */
  /* chg : for get file size with PATH 11/02/07 masuu */
  trans_ret = vsf_ftpdataio_transfer_file3(p_sess, remote_fd, opened_file, str_getbuf(&s_path_str), is_ascii);
  gettimeofday(&endtime, NULL);
  vsf_ftpdataio_dispose_transfer_fd(p_sess);
  p_sess->transfer_size = trans_ret.transferred;
  /* Log _after_ the blocking dispose call, so we get transfer times right */
  if (trans_ret.retval == 0) {
    vsf_log_do_log(p_sess, 1);
  }
  /* Emit status message _after_ blocking dispose call to avoid buggy FTP
   * clients truncating the transfer.
   */
  if (trans_ret.retval == -1) {
    vsf_cmdio_write(p_sess, FTP_BADSENDFILE, "Requested action aborted. Local error in processing.");
  }
/* chg : transfer size no check 11/03/09 masuu
  else if (trans_ret.retval == -2) {
    vsf_cmdio_write(p_sess, FTP_BADWRITESEND, "Error during write to data connection.");
  }
*/
  else {
    transfer_speed_write(p_sess, &starttime, &endtime, vsf_sysutil_statbuf_get_size(s_p_statbuf));
    vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Closing data connection. Requested file action successful.");
  }

  /* abort check */
  check_abor(p_sess);

  /* clean up */
port_pasv_cleanup_out:
  port_cleanup(p_sess);
  pasv_cleanup(p_sess);
file_close_out:
  vsf_sysutil_close(opened_file);
}

static void
handle_appe(struct vsf_session* p_sess)
{
  handle_upload_common(p_sess, 1, 0);
}

static void
handle_mdtm(struct vsf_session* p_sess)
{
  static struct mystr s_filename_str;
  static struct vsf_sysutil_statbuf* s_p_statbuf;
  int do_write = 0;
  long modtime = 0;
  struct str_locate_result loc = str_locate_char(&p_sess->ftp_arg_str, ' ');
  int retval = str_stat(&p_sess->ftp_arg_str, &s_p_statbuf);
  if (tunable_mdtm_write && retval != 0 && loc.found &&
      vsf_sysutil_isdigit(str_get_char_at(&p_sess->ftp_arg_str, 0)))
  {
    if (loc.index == 8 || loc.index == 14 ||
        (loc.index > 15 && str_get_char_at(&p_sess->ftp_arg_str, 14) == '.'))
    {
      do_write = 1;
    }
  }
  if (do_write != 0)
  {
    str_split_char(&p_sess->ftp_arg_str, &s_filename_str, ' ');
    modtime = vsf_sysutil_parse_time(str_getbuf(&p_sess->ftp_arg_str));
    str_copy(&p_sess->ftp_arg_str, &s_filename_str);
  }
  resolve_tilde(&p_sess->ftp_arg_str, p_sess);
  if (!vsf_access_check_file(&p_sess->ftp_arg_str))
  {
    vsf_cmdio_write(p_sess, FTP_NOPERM, "Permission denied.");
    return;
  }
  if (do_write && tunable_write_enable &&
      (tunable_anon_other_write_enable || !p_sess->is_anonymous))
  {
    retval = str_stat(&p_sess->ftp_arg_str, &s_p_statbuf);
    if (retval != 0 || !vsf_sysutil_statbuf_is_regfile(s_p_statbuf))
    {
      vsf_cmdio_write(p_sess, FTP_FILEFAIL,
                      "Could not set file modification time.");
    }
    else
    {
      retval = vsf_sysutil_setmodtime(
        str_getbuf(&p_sess->ftp_arg_str), modtime, tunable_use_localtime);
      if (retval != 0)
      {
        vsf_cmdio_write(p_sess, FTP_FILEFAIL,
                        "Could not set file modification time.");
      }
      else
      {
        vsf_cmdio_write(p_sess, FTP_MDTMOK,
                        "File modification time set.");
      }
    }
  }
  else
  {
    if (retval != 0 || !vsf_sysutil_statbuf_is_regfile(s_p_statbuf))
    {
      vsf_cmdio_write(p_sess, FTP_FILEFAIL,
                      "Could not get file modification time.");
    }
    else
    {
      static struct mystr s_mdtm_res_str;
      str_alloc_text(&s_mdtm_res_str,
                     vsf_sysutil_statbuf_get_numeric_date(
                       s_p_statbuf, tunable_use_localtime));
      vsf_cmdio_write_str(p_sess, FTP_MDTMOK, &s_mdtm_res_str);
    }
  }
}

static void
handle_eprt(struct vsf_session* p_sess)
{
  static struct mystr s_part1_str;
  static struct mystr s_part2_str;
  int proto;
  int port;
  const unsigned char* p_raw_addr;
  int is_ipv6 = vsf_sysutil_sockaddr_is_ipv6(p_sess->p_local_addr);
  port_cleanup(p_sess);
  pasv_cleanup(p_sess);
  str_copy(&s_part1_str, &p_sess->ftp_arg_str);
  str_split_char(&s_part1_str, &s_part2_str, '|');
  if (!str_isempty(&s_part1_str))
  {
    goto bad_eprt;
  }
  /* Split out the protocol and check it */
  str_split_char(&s_part2_str, &s_part1_str, '|');
  proto = str_atoi(&s_part2_str);
  if (proto < 1 || proto > 2 || (!is_ipv6 && proto == 2))
  {
    vsf_cmdio_write(p_sess, FTP_BADCMD, "Bad EPRT protocol.");
    return;
  }
  /* Split out address and parse it */
  str_split_char(&s_part1_str, &s_part2_str, '|');
  if (proto == 2)
  {
    p_raw_addr = vsf_sysutil_parse_ipv6(&s_part1_str);
  }
  else
  {
    p_raw_addr = vsf_sysutil_parse_ipv4(&s_part1_str);
  }
  if (!p_raw_addr)
  {
    goto bad_eprt;
  }
  /* Split out port and parse it */
  str_split_char(&s_part2_str, &s_part1_str, '|');
  if (!str_isempty(&s_part1_str) || str_isempty(&s_part2_str))
  {
    goto bad_eprt;
  }
  port = str_atoi(&s_part2_str);
  if (port < 0 || port > 65535)
  {
    goto bad_eprt;
  }
  vsf_sysutil_sockaddr_clone(&p_sess->p_port_sockaddr, p_sess->p_local_addr);
  if (proto == 2)
  {
    vsf_sysutil_sockaddr_set_ipv6addr(p_sess->p_port_sockaddr, p_raw_addr);
  }
  else
  {
    vsf_sysutil_sockaddr_set_ipv4addr(p_sess->p_port_sockaddr, p_raw_addr);
  }
  vsf_sysutil_sockaddr_set_port(p_sess->p_port_sockaddr, (unsigned short) port);
  /* SECURITY:
   * 1) Reject requests not connecting to the control socket IP
   * 2) Reject connects to privileged ports
   */
  if (!tunable_port_promiscuous)
  {
    if (!vsf_sysutil_sockaddr_addr_equal(p_sess->p_remote_addr,
                                         p_sess->p_port_sockaddr) ||
        vsf_sysutil_is_port_reserved(port))
    {
      vsf_cmdio_write(p_sess, FTP_BADCMD, "Illegal EPRT command.");
      port_cleanup(p_sess);
      return;
    }
  }
  vsf_cmdio_write(p_sess, FTP_EPRTOK,
                  "EPRT command successful. Consider using EPSV.");
  return;
bad_eprt:
  vsf_cmdio_write(p_sess, FTP_BADCMD, "Bad EPRT command.");
}

/* XXX - add AUTH etc. */
static void
handle_help(struct vsf_session* p_sess)
{
#ifdef _CMD_TYPE_XDS_	/* chg : for XDS 10/05/20 masuu */
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/28 masuu */
  static struct mystr s_help_args_str;

  if( str_isempty(&p_sess->ftp_arg_str) ) {
    vsf_cmdio_write_hyphen(p_sess, FTP_HELP, "Following commands are implemented.");
    vsf_cmdio_write_raw(p_sess, "   USER    PASS    QUIT    ABOR    SYST    HELP    NOOP    PORT\r\n");
    vsf_cmdio_write_raw(p_sess, "   PASV    TYPE    STRU    MODE    LIST    NLST    RETR    STOR\r\n");
    vsf_cmdio_write_raw(p_sess, "   RNFR    RNTO    DELE    STAT    PWD     XPWD    CWD     XCWD\r\n");
    vsf_cmdio_write_raw(p_sess, "   CDUP    XCUP    MKD     XMKD    RMD     XRMD    SITE\r\n");
    vsf_cmdio_write(p_sess, FTP_HELP, "Help end");

  }
  /* add : correspond to argument 11/03/02 masuu */
  else {
    str_split_char(&p_sess->ftp_arg_str, &s_help_args_str, ' ');
    str_upper(&p_sess->ftp_arg_str);
    if( str_equal_text(&p_sess->ftp_arg_str, "USER") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: USER <sp> user_name");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "PASS") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: PASS <sp> password");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "QUIT") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: QUIT (close control connection)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "ABOR") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: ABOR (abort current operation)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "SYST") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: SYST (return OS type)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "HELP") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: HELP [<sp> command]");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "NOOP") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: NOOP (no operation)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "PORT") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: PORT <sp> h1,h2,h3,h4,p1,p2");
    }

    else if( str_equal_text(&p_sess->ftp_arg_str, "PASV") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: PASV (return address/port)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "TYPE") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: TYPE <sp> type_code (A, I)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "STRU") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: STRU <sp> structure_code (F)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "MODE") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: MODE <sp> mode_code (S)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "LIST") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: LIST [<sp> pathname | <sp> options (-aF)]");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "NLST") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: NLST [<sp> pathname | <sp> options (-alF)]");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "RETR") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: RETR <sp> pathname");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "STOR") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: STOR <sp> pathname");
    }

    else if( str_equal_text(&p_sess->ftp_arg_str, "RNFR") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: RNFR <sp> pathname");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "RNTO") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: RNTO <sp> pathname");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "DELE") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: DELE <sp> pathname");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "STAT") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: STAT [<sp> pathname]");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "PWD") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: PWD (return current working directory)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "XPWD") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: XPWD (return current working directory)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "CWD") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: CWD <sp> pathname");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "XCWD") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: XCWD <sp> pathname");
    }

    else if( str_equal_text(&p_sess->ftp_arg_str, "CDUP") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: CDUP (up one directory)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "XCUP") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: XCUP (up one directory)");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "MKD") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: MKD <sp> pathname");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "XMKD") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: XMKD <sp> pathname");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "RMD") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: RMD <sp> pathname");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "XRMD") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: XRMD <sp> pathname");
    }
    else if( str_equal_text(&p_sess->ftp_arg_str, "SITE") ) {
      vsf_cmdio_write(p_sess, FTP_HELP, "Syntax: SITE <sp> site_command [<sp> site_arg...]");
    }
    else {
      vsf_cmdio_write(p_sess, FTP_BADOPTS, "Syntax error, in parameters or arguments.");
    }
  }
#else /* _RES_TYPE_XDS_ */
  vsf_cmdio_write_hyphen(p_sess, FTP_HELP, "The following commands are recognized.");
  vsf_cmdio_write_raw(p_sess, " ABOR CDUP CWD  DELE HELP LIST MKD  MODE NLST NOOP PASS PASV PORT PWD\r\n");
  vsf_cmdio_write_raw(p_sess, " QUIT RETR RMD  RNFR RNTO SITE STAT STOR STRU SYST TYPE USER XCUP XCWD\r\n");
  vsf_cmdio_write_raw(p_sess, " XMKD XPWD XRMD\r\n");
  vsf_cmdio_write(p_sess, FTP_HELP, "Help OK.");
#endif /* _RES_TYPE_XDS_ */
#else /* _CMD_TYPE_XDS_ */
  vsf_cmdio_write_hyphen(p_sess, FTP_HELP, "The following commands are recognized.");
  vsf_cmdio_write_raw(p_sess, " ABOR ACCT ALLO APPE CDUP CWD  DELE EPRT EPSV FEAT HELP LIST MDTM MKD\r\n");
  vsf_cmdio_write_raw(p_sess, " MODE NLST NOOP OPTS PASS PASV PORT PWD  QUIT REIN REST RETR RMD  RNFR\r\n");
  vsf_cmdio_write_raw(p_sess, " RNTO SITE SIZE SMNT STAT STOR STOU STRU SYST TYPE USER XCUP XCWD XMKD\r\n");
  vsf_cmdio_write_raw(p_sess, " XPWD XRMD\r\n");
  vsf_cmdio_write(p_sess, FTP_HELP, "Help OK.");
#endif /* _CMD_TYPE_XDS_ */
}

static void
handle_stou(struct vsf_session* p_sess)
{
  handle_upload_common(p_sess, 0, 1);
}

static void
get_unique_filename(struct mystr* p_outstr, const struct mystr* p_base_str)
{
  /* Use silly wu-ftpd algorithm for compatibility */
  static struct vsf_sysutil_statbuf* s_p_statbuf;
  unsigned int suffix = 1;
  int retval;
  retval = str_stat(p_base_str, &s_p_statbuf);
  if (vsf_sysutil_retval_is_error(retval))
  {
    str_copy(p_outstr, p_base_str);
    return;
  }
  while (1)
  {
    str_copy(p_outstr, p_base_str);
    str_append_char(p_outstr, '.');
    str_append_ulong(p_outstr, suffix);
    retval = str_stat(p_outstr, &s_p_statbuf);
    if (vsf_sysutil_retval_is_error(retval))
    {
      return;
    }
    ++suffix;
  }
}

static void
handle_stat(struct vsf_session* p_sess)
{
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/06/16 masuu */
  vsf_cmdio_write(p_sess, FTP_STATOK, "0");
#else /* _RES_TYPE_XDS_ */
  vsf_cmdio_write_hyphen(p_sess, FTP_STATOK, "FTP server status:");
  vsf_cmdio_write_raw(p_sess, "     Connected to ");
  vsf_cmdio_write_raw(p_sess, str_getbuf(&p_sess->remote_ip_str));
  vsf_cmdio_write_raw(p_sess, "\r\n");
  vsf_cmdio_write_raw(p_sess, "     Logged in as ");
  vsf_cmdio_write_raw(p_sess, str_getbuf(&p_sess->user_str));
  vsf_cmdio_write_raw(p_sess, "\r\n");
  vsf_cmdio_write_raw(p_sess, "     TYPE: ");
  if (p_sess->is_ascii)
  {
    vsf_cmdio_write_raw(p_sess, "ASCII\r\n");
  }
  else
  {
    vsf_cmdio_write_raw(p_sess, "BINARY\r\n");
  }
  if (p_sess->bw_rate_max == 0)
  {
    vsf_cmdio_write_raw(p_sess, "     No session bandwidth limit\r\n");
  }
  else
  {
    vsf_cmdio_write_raw(p_sess, "     Session bandwidth limit in byte/s is ");
    vsf_cmdio_write_raw(p_sess, vsf_sysutil_ulong_to_str(p_sess->bw_rate_max));
    vsf_cmdio_write_raw(p_sess, "\r\n");
  }
  if (tunable_idle_session_timeout == 0)
  {
    vsf_cmdio_write_raw(p_sess, "     No session timeout\r\n");
  }
  else
  {
    vsf_cmdio_write_raw(p_sess, "     Session timeout in seconds is ");
    vsf_cmdio_write_raw(p_sess,
      vsf_sysutil_ulong_to_str(tunable_idle_session_timeout));
    vsf_cmdio_write_raw(p_sess, "\r\n");
  }
  if (p_sess->control_use_ssl)
  {
    vsf_cmdio_write_raw(p_sess, "     Control connection is encrypted\r\n"); 
  }
  else
  {
    vsf_cmdio_write_raw(p_sess, "     Control connection is plain text\r\n"); 
  }
  if (p_sess->data_use_ssl)
  {
    vsf_cmdio_write_raw(p_sess, "     Data connections will be encrypted\r\n"); 
  }
  else
  {
    vsf_cmdio_write_raw(p_sess, "     Data connections will be plain text\r\n");
  }
  if (p_sess->num_clients > 0)
  {
    vsf_cmdio_write_raw(p_sess, "     At session startup, client count was ");
    vsf_cmdio_write_raw(p_sess, vsf_sysutil_ulong_to_str(p_sess->num_clients));
    vsf_cmdio_write_raw(p_sess, "\r\n");
  }
  vsf_cmdio_write_raw(p_sess,
    "     vsFTPd " VSF_VERSION " - secure, fast, stable\r\n");
  vsf_cmdio_write(p_sess, FTP_STATOK, "End of status");
#endif /* _RES_TYPE_XDS_ */
}

static void
handle_stat_file(struct vsf_session* p_sess)
{
  handle_dir_common(p_sess, 1, 1);
}

static int
data_transfer_checks_ok(struct vsf_session* p_sess)
{
  if (!pasv_active(p_sess) && !port_active(p_sess))
  {
#ifdef _RES_TYPE_XDS_	/* chg : for XDS 10/05/31 masuu */
    vsf_cmdio_write(p_sess, FTP_BADSENDCONN, "Can't open data connection.");
#else /* _RES_TYPE_XDS_ */
    vsf_cmdio_write(p_sess, FTP_BADSENDCONN, "Use PORT or PASV first.");
#endif /* _RES_TYPE_XDS_ */
    return 0;
  }
  if (tunable_ssl_enable && !p_sess->data_use_ssl &&
      ((tunable_force_local_data_ssl && !p_sess->is_anonymous) ||
       (tunable_force_anon_data_ssl && p_sess->is_anonymous)))
  {
    vsf_cmdio_write(
      p_sess, FTP_NEEDENCRYPT, "Data connections must be encrypted.");
    return 0;
  }
  return 1;
}

static void
resolve_tilde(struct mystr* p_str, struct vsf_session* p_sess)
{
  unsigned int len = str_getlen(p_str);
  if (len > 0 && str_get_char_at(p_str, 0) == '~')
  {
    static struct mystr s_rhs_str;
    if (len == 1 || str_get_char_at(p_str, 1) == '/')
    {
      str_split_char(p_str, &s_rhs_str, '~');
      str_copy(p_str, &p_sess->home_str);
      str_append_str(p_str, &s_rhs_str);
    }
    else if (tunable_tilde_user_enable && len > 1)
    {
      static struct mystr s_user_str;
      struct vsf_sysutil_user* p_user;
      str_copy(&s_rhs_str, p_str);
      str_split_char(&s_rhs_str, &s_user_str, '~');
      str_split_char(&s_user_str, &s_rhs_str, '/');
      p_user = str_getpwnam(&s_user_str);
      if (p_user != 0)
      {
        str_alloc_text(p_str, vsf_sysutil_user_get_homedir(p_user));
        if (!str_isempty(&s_rhs_str))
        {
          str_append_char(p_str, '/');
          str_append_str(p_str, &s_rhs_str);
        }
      }
    }
  }
}

/* add : Response Code for Transfer Speed 10/06/01 masuu */
static void
transfer_speed_write(struct vsf_session* p_sess, struct timeval *pt_starttime, struct timeval *pt_endtime, filesize_t xferdsize)
{
	static struct mystr s_tmp_str;
	double d_time;		// [sec]
	double d_speed;		// [bps]

	d_time = (pt_endtime->tv_sec + pt_endtime->tv_usec / 1000000.0) -
		(pt_starttime->tv_sec + pt_starttime->tv_usec / 1000000.0);

	/* transfer rate */
	if (d_time && xferdsize) {
		d_speed = (xferdsize / d_time) * 8;		// byte -> bit
	} else {
		d_speed = 0.0;
	}

	/* transfer message */
	str_alloc_double(&s_tmp_str, d_time);
	str_append_text(&s_tmp_str, " seconds (measured by the server), ");
	if( d_speed > (1024 * 1024 / 2) ) {
		str_append_double(&s_tmp_str, d_speed / (1024 * 1024));
		str_append_text(&s_tmp_str, " Mbit/s");
	} else {
		str_append_double(&s_tmp_str, d_speed / 1024);
		str_append_text(&s_tmp_str, " Kbit/s");
	}
	vsf_cmdio_write_str_hyphen(p_sess, FTP_TRANSFEROK, &s_tmp_str);
//	vsf_cmdio_write(p_sess, FTP_TRANSFEROK, "Closing data connection. Requested file action successful.");
}
