/*
 * xdcstat.c
 *
 * Copyright (C) 2010 Sony Corporation.
 * All rights reserved.
 *
 * $Id: xdcstat.c,v 1.2 2010/06/25 09:49:11 masuu Exp $
 */

#include <stdio.h>		// for NULL

#include "session.h"
#include "ftpcmdio.h"
#include "sysutil.h"
#include "common/ioctl.h"
#include "xdcstat.h"

typedef struct _xdcVideoCodecTbl {
  int i_codec_no;
  const char *pcc_reply_str;
} xdcVideoCodecTbl;

typedef struct _xdcFpsTbl {
  int i_fps_no;
  const char *pcc_reply_str;
} xdcFpsTbl;

static const xdcVideoCodecTbl xdc_codec_tbl[] = {
  { AS_VC_UNKNOWN,		"unknown"	},
  { AS_VC_NONE,			"none"		},
  { AS_VC_DV25_411,		"DV25"		},
  { AS_VC_DV25D_411,	"DV25"		},
  { AS_VC_DV25_420,		"DV25"		},
  { AS_VC_DV25D_420,	"DV25"		},
  { AS_VC_IMX30,		"IMX30"		},
  { AS_VC_IMX40,		"IMX40"		},
  { AS_VC_IMX50,		"IMX50"		},
  { AS_VC_MPEG2HD25CBR_1440_1080_MPH14L,	"MPEG2HD25CBR_1440_1080_MP@H-14"	},
  { AS_VC_MPEG2HD12_5_CBR_1440_540_MPH14L,	"MPEG2HD12.5CBR_1440_540_MP@H-14"	},
  { AS_VC_MPEG2HD17_5_1440_1080_MPHL,		"MPEG2HD17.5_1440_1080_MP@HL"		},
  { AS_VC_MPEG2HD25_1440_1080_MPHL,			"MPEG2HD25_1440_1080_MP@HL"			},
  { AS_VC_MPEG2HD35_1440_1080_MPHL,			"MPEG2HD35_1440_1080_MP@HL"			},
  { AS_VC_MPEG2HD8_75_1440_540_MPHL,		"MPEG2HD8.75_1440_540_MP@HL"		},
  { AS_VC_MPEG2HD12_5_1440_540_MPHL,		"MPEG2HD12.5_1440_540_MP@HL"		},
  { AS_VC_MPEG2HD17_5_1440_540_MPHL,		"MPEG2HD17.5_1440_540_MP@HL"		},
  { AS_VC_MPEG2HD35_1920_1080_422PHL,		"MPEG2HD35_1920_1080_422P@HL"		},
  { AS_VC_MPEG2HD50_1920_1080_422PHL,		"MPEG2HD50_1920_1080_422P@HL"		},
  { AS_VC_MPEG2HD25_1920_540_422PHL,		"MPEG2HD25_1920_540_422P@HL"		},
  { AS_VC_MPEG2HD50_CBR_1920_1080_422PHL,	"MPEG2HD50CBR_1920_1080_422P@HL"	},
  { AS_VC_MPEG2HD25_CBR_1920_540_422PHL,	"MPEG2HD25CBR_1920_540_422P@HL"		},
  { AS_VC_MPEG2HD50_1280_720_422PHL,		"MPEG2HD50_1280_720_422P@HL"		},
  { AS_VC_MPEG2HD50_CBR_1280_720_422PHL,	"MPEG2HD50CBR_1280_720_422P@HL"		},
  { AS_VC_MPEG2HD35_1280_720_MPHL,			"MPEG2HD35_1280_720_MP@HL"			},
  { AS_VC_MPEG2HD35_1920_1080_MPHL,			"MPEG2HD35_1920_1080_MP@HL"			},
  { AS_VC_MPEG2HD25_1280_720_MPHL,			"MPEG2HD25_1280_720_MP@HL"			},
  { AS_VC_MPEG2HD25CBR_1280_720_MPHL,		"MPEG2HD25CBR_1280_720_MP@HL"		},
  { 0,	NULL }
};

static const xdcFpsTbl xdc_fps_tbl[] = {
  { AS_FPS_UNKNOWN,	"unknown"	},
  { AS_FPS_NONE,	"none"		},
  { AS_FPS_50_I,	"50i"		},
  { AS_FPS_59_94_I,	"59.94i"	},
  { AS_FPS_23_98_P,	"23.98P"	},
  { AS_FPS_59_94_P,	"59.94P"	},
  { AS_FPS_50_P,	"50P"		},
  { AS_FPS_25_P,	"25P"		},
  { AS_FPS_29_97_P,	"29.97P"	},
  { AS_FPS_24_P,	"24P"		},
  { 0,				NULL		}
};

const char * xdc_GetCodecString( int i_codec_no )
{
  xdcVideoCodecTbl *pt_tbl = (xdcVideoCodecTbl *)xdc_codec_tbl;

  while ( pt_tbl->pcc_reply_str ) {
    if ( i_codec_no == pt_tbl->i_codec_no ) {
      break;
    }
    pt_tbl++;
  }

  if ( pt_tbl->pcc_reply_str != NULL ) {
    return pt_tbl->pcc_reply_str;
  }
  return NULL;
}

const char * xdc_GetFpsString( int i_fps_no )
{
  xdcFpsTbl *pt_tbl = (xdcFpsTbl *)xdc_fps_tbl;

  while ( pt_tbl->pcc_reply_str ) {
    if ( i_fps_no == pt_tbl->i_fps_no ) {
      break;
    }
    pt_tbl++;
  }

  if ( pt_tbl->pcc_reply_str != NULL ) {
    return pt_tbl->pcc_reply_str;
  }
  return NULL;
}

void xdc_stat_write_raw(struct vsf_session* p_sess, struct vsf_confs_xdc_stat_param ioc_param)
{
  const char *pcc_tmp;

  vsf_cmdio_write_raw(p_sess, "    MXF\r\n");

  if ((pcc_tmp = xdc_GetCodecString(ioc_param.xdc_stat.v_codec )) != NULL ) {
    vsf_cmdio_write_raw(p_sess, "    ");
    vsf_cmdio_write_raw(p_sess, pcc_tmp);
    vsf_cmdio_write_raw(p_sess, "\r\n");
  }
  else {
    vsf_cmdio_write_raw(p_sess, "    Undefined CODEC type.\r\n");
  }

  if ((pcc_tmp = xdc_GetFpsString(ioc_param.xdc_stat.fps )) != NULL ) {
    vsf_cmdio_write_raw(p_sess, "    ");
    vsf_cmdio_write_raw(p_sess, pcc_tmp);
    vsf_cmdio_write_raw(p_sess, "\r\n");
  }
  else {
    vsf_cmdio_write_raw(p_sess, "    Undefined FRAME RATE type.\r\n");
  }

  vsf_cmdio_write_raw(p_sess, "    ");
  vsf_cmdio_write_raw(p_sess, vsf_sysutil_ulong_to_str(ioc_param.xdc_stat.num_of_a_ch));
  vsf_cmdio_write_raw(p_sess, "\r\n");

  vsf_cmdio_write_raw(p_sess, "    ");
  vsf_cmdio_write_raw(p_sess, vsf_sysutil_ulong_to_str(ioc_param.xdc_stat.clip_duration));
  vsf_cmdio_write_raw(p_sess, "\r\n");

  vsf_cmdio_write_raw(p_sess, "    ");
  vsf_cmdio_write_raw(p_sess, ioc_param.xdc_stat.umidstr);
  vsf_cmdio_write_raw(p_sess, "\r\n");
}
